if (!window.WT) {
    window.WT = {};
}

;(function () {
    var self = {
        /**
         * 判断是否为原生dom元素
         */
        isDOM: ( typeof HTMLElement === 'object' ) ?
            function (obj) {
                return obj instanceof HTMLElement;
            } :
            function (obj) {
                return obj && typeof obj === 'object' && obj.nodeType === 1 && typeof obj.nodeName === 'string';
            },
        /**
         * 创建一个原生dom元素
         * @param tag {string} 标签，如'div', 'span'
         * @param classNames? {string} 样式类名，多个用空格隔开
         * @param contentText? {string} 字符串内容
         * @return {HTMLElement}
         */
        createElement: function (tag, classNames, contentText) {
            var el = document.createElement(tag);
            if (classNames) {
                el.className = classNames;
            }
            if (contentText) {
                el.innerHTML = contentText;
            }
            return el;
        },
        /**
         * 创建一个jQuery元素
         * @param tag {string} 标签，如'div', 'span'
         * @param classNames? {string} 样式类名，多个用空格隔开
         * @param contentText? {string} 字符串内容
         * @return {*|jQuery|HTMLElement}
         */
        createJQueryElement: function (tag, classNames, contentText) {
            return $(self.createElement.apply(null, arguments));
        },
        /**
         * 将子元素插入到父元素中
         * @param parent {jQuery|HTMLElement} 父元素
         * @param child {jQuery|HTMLElement} 子元素
         */
        appendChild: function (parent, child) {
            var elp, elc;
            if ((parent instanceof $) && parent.length > 0) {
                elp = parent[0];
            } else if (parent.appendChild) {
                elp = parent;
            } else {
                return;
            }
            if ((child instanceof $) && child.length > 0) {
                elc = child[0];
            } else if (self.isDOM(child)) {
                elc = child;
            } else {
                return;
            }
            elp.appendChild(elc);
        },
        /**
         * 把元素从html树中移除
         * @param el {jQuery|HTMLElement}
         */
        detachFromHTML: function (el) {
            var elt;
            if ((el instanceof $) && el.length > 0) {
                elt = el[0];
            }
            if (!self.isDOM(elt)) {
                return;
            }
            var pe = elt.parentNode;
            if (pe) {
                pe.removeChild(elt);
            }
        }
    };
    $.extend(WT, {
        dom: self
    });
})();
