//include underscope/lodash

;(function () {

    if (!window.WT) {
        window.WT = {};
    }

    $.extend(WT, {

        createWidget: function (xtype, options) {
            var cls = eval(xtype);
            if (!cls) {
                throw new Error('Cannot find widget: ' + xtype)
            }
            return new cls(options);
        },
        isBoolean: function (val) {
            return _.isBoolean(val);
        },
        isArray: function (val) {
            return _.isArray(val);
        },
        isString: function (val) {
            return _.isString(val);
        },
        isObject: function (val) {
            return _.isObject(val) && !_.isArray(val);
        },
        isNumber: function (val) {
            return _.isNumber(val) && !isNaN(val);
        },
        isEmpty: function (obj) {
            return _.isEmpty(obj);
        },
        isNotEmptyObject: function (obj) {
            return WT.isObject(obj) && !WT.isEmpty(obj);
        },
        isFunction: function (obj) {
            return _.isFunction(obj);
        },
        keys: function (obj) {
            return _.keys(obj);
        },
        isIntString: function (val) {
            if (WT.isNumber(val)) {
                return true;
            }
            if (!WT.isString(val)) {
                return false;
            }
            var n = parseInt(val);
            return !isNaN(n) && (n + '') === val;
        },
        /**
         * 比较两个对象是否相等 deep equal
         * @param obj
         * @param another
         */
        isEqual: function (obj, another) {
            if (obj === another) {
                return true;
            }
            return _.isEqual(obj, another);
        },
        isNullOrUndefined: function (obj) {
            return obj === undefined || obj === null;
        },
        nextTick: function (callback, caller) {
            if (!WT.isFunction(callback)) {
                return WT.createPromise(function (resolve) {
                    setTimeout(function () {
                        resolve();
                    }, 1);
                });
            }
            setTimeout(callback.bind(caller), 1);
        },
        setTimeout: function (timeout) {
            return WT.createPromise(function (resolve) {
                setTimeout(resolve, timeout);
            });
        },
        isKey: function () {
            return BI.isKey.apply(null, arguments);
        },
        md5: function (s) {
            return BI.MD5.hex_md5(s);
        },
        isNotEmptyString: function (v) {
            return BI.isNotEmptyString(v);
        },
        copyArray: function (arr) {
            var result = [];
            _.isArray(arr) && WT.each(arr, function (item) {
                result.push(item);
            });
            return result;
        },
        each: function () {
            return _.each.apply(_, arguments);
        },
        find: function () {
            return _.find.apply(_, arguments);
        },
        findIndex: function () {
            return _.findIndex.apply(_, arguments);
        },
        map: function () {
            return _.map.apply(_, arguments);
        },
        reduce: function () {
            return _.reduce.apply(_, arguments)
        },
        uniq: function () {
            return _.uniq.apply(_, arguments);
        },
        filter: function () {
            return _.filter.apply(_, arguments);
        },
        emptyFn: function () {
        },
        trim: function (str) {
            return str && str.replace(/(^\s*)|(\s*$)/g, "");
        },
        arrayContains: function (arr, val) {
            if (WT.isArray(arr)) {
                for (var i = 0; i < arr.length; i++) {
                    if (arr[i] + '' === val + '') {
                        return true;
                    }
                }
            }
            return false;
        },
        arrayPull: function (list, idx) {
            var removed = list && list.splice(idx, 1);
            if (removed && removed.length === 1) {
                return removed[0];
            }
            return null;
        },
        throttle: function () {
            return _.throttle.apply(_, arguments);
        },
        debounce: function () {
            return _.debounce.apply(_, arguments);
        },
        promiseReduce: function () {
            return Promise.reduce.apply(Promise, arguments);
        },
        promiseEach: function () {
            return Promise.each.apply(Promise, arguments);
        },
        object2String: function (obj) {
            if (WT.isArray(obj)) {
                return WT.map(obj, function (o) {
                    if (WT.isString(o) || WT.isNumber(o)) {
                        return o;
                    }
                    return FR.jsonEncode(o);
                }).join(',');
            } else if (WT.isObject(obj)) {
                return FR.jsonEncode(obj);
            } else {
                return obj + '';
            }
        },

        /**
         * 设置背景
         * @param $dom jquery对象
         * @param background 颜色对象
         */
        setBackground: function ($dom, background) {
            if (WT.isString(background)) {
                //是字符串直接设置css
                $dom.css('background', background);
            }
            if (WT.isObject(background)) {
                FR.setBackground($dom, background);
            }
        },

        setWidgetEnable: function (widget, enable) {
            if (!widget) {
                return;
            }
            if (widget instanceof $) {
                if (enable) {
                    widget.enable();
                } else {
                    widget.disable();
                }
            } else if (WT.isFunction(widget.setEnable)) {
                widget.setEnable(enable);
            }
        },

        /**
         * 显示导出时的进度条
         */
        showProgressBar: function (sessionID, type) {
            FR.progressBar.call({
                createProgressBar: function () {
                    if ($('#progressBar').length <= 0) {
                        var progressBar = $('<div id = ' + '\"' + 'progressBar' + '\"' + '>' + '</div>').appendTo("body");
                        var tips = $('<div>' + FR.i18nText("Fine-Engine_Exporting") + '...' + '</div>').appendTo($('#progressBar'));
                        var $maskDiv = $('<div id="transparent_mask" style="position:absolute;filter:alpha(opacity=50);opacity:0.5;"></div>');
                        $maskDiv.css('left', "0px").css('top', "0px").css('width', "100%").css('height', "100%").css('z-index', 2001).appendTo("body");
                        progressBar.addClass("progressBar");
                        tips.addClass("progressBarTips");
                    }
                },
                removeProgressBar: function () {
                    $('#progressBar').remove();
                    $('#transparent_mask').remove();
                }
            }, sessionID, type);
        },

        /**
         * 创建Promise对象
         * @param fn
         * @return {Promise.<T>}
         */
        createPromise: function (fn) {
            return new Promise(fn);
        },

        /**
         * 创建处理成功的静态Promise
         * @param value
         * @return {Promise.<T>}
         */
        createPromiseResult: function (value) {
            return value === undefined ? Promise.resolve() : Promise.resolve(value);
        },

        /**
         * 创建处理失败的静态Promise
         * @param value
         * @return {Promise.<*>}
         */
        createPromiseError: function (value) {
            return value === undefined ? Promise.reject() : Promise.reject(value);
        },

        /**
         * 使用FR.Msg.toast提示错误
         * @param e WT.ajax抛出的错误对象或者普通错误对象
         */
        toastError: function (e) {
            if (!e) {
                return;
            }
            var err = e;
            if (err.isErrorInHTML === true) {
                return;
            }
            if (e.xhr) {
                //为ajax抛出的错
                err = e.err;
            }
            var msg = err.message || err;
            if (msg) {
                FR.Msg.toast(msg);
            }
        },

        /**
         * 使用FR.Msg.alert提示错误
         * @param e WT.ajax抛出的错误对象或者普通错误对象
         */
        alertError: function (e) {
            if (!e) {
                return;
            }
            var err = e;
            if (err.isErrorInHTML === true) {
                return;
            }
            if (e.xhr) {
                //为ajax抛出的错
                err = e.err;
            }
            var msg = err.message || err;
            if (msg) {
                FR.Msg.alert(FR.i18nText("Fine-Engine_Failed"), msg);
            }
        },
        /**
         * 这个函数用来过滤错误
         * 正常情况后台返回json结果，但是像session超时这样的错误后台直接返回html，这种情况下应该直接显示html
         *
         * @param result WT.ajax的返回结果
         * @param el 错误时显示html的jquery对象
         */
        dealWithHtmlResult: function (result, el) {
            if (WT.isString(result.data) && (result.data.indexOf('<!DOCTYPE html>') === 0 || result.data.indexOf('<!doctype html>') === 0 || result.data.indexOf('<html>') === 0)) {
                //代表返回了错误页面
                if (!el || !(el instanceof $)) {
                    el = _g().getSheetElement() || $('body');
                }
                setTimeout(function () {
                    //然后显示返回的html错误页
                    el.empty();
                    el.html(result.data);
                }, 100);   //做个延迟是为了等待进度框收起
                var err = new Error('error in html format');
                err.isErrorInHTML = true;
                throw err;
            }
            return result;
        },

        /**
         * 转换单元格边框样式
         * @param type 边框样式 @see com.fr.stable.Constants#LINE_NONE等
         * @param color 颜色  如：'#000000'
         */
        transformBorderType: function (type, color) {
            switch (type) {
                case 1:
                    //LINE_THIN
                    return {
                        style: '1px solid ' + color,
                        width: 1
                    };
                case 2:
                    //LINE_MEDIUM
                    return {
                        style: '2px solid ' + color,
                        width: 2
                    };
                case 3:
                    //LINE_DASH
                    return {
                        style: '1px dashed ' + color,
                        width: 1
                    };
                case 4:
                    //LINE_HAIR
                    return {
                        style: '1px dashed ' + color,
                        width: 1
                    };
                case 5:
                    //LINE_THICK
                    return {
                        style: '3px solid ' + color,
                        width: 3
                    };
                case 6:
                    //LINE_DOUBLE
                    return {
                        style: '2px double ' + color,
                        width: 2
                    };
                case 7:
                    //LINE_DOT
                    return {
                        style: '1px dotted ' + color,
                        width: 1
                    };
                case 8:
                    //LINE_MEDIUM_DASH
                    return {
                        style: '2px dashed ' + color,
                        width: 2
                    };
                case 9:
                    //LINE_DASH_DOT
                    return {
                        style: '2px dotted ' + color,
                        width: 2
                    };
                case 10:
                    //LINE_MEDIUM_DASH_DOT
                    return {
                        style: '2px dashed ' + color,
                        width: 2
                    };
                case 11:
                    //LINE_DASH_DOT_DOT
                    return {
                        style: '2px dotted ' + color,
                        width: 2
                    };
                case 12:
                    //LINE_MEDIUM_DASH_DOT_DOT
                    return {
                        style: '2px dashed ' + color,
                        width: 2
                    };
                case 13:
                    //LINE_SLANTED_DASH_DOT
                    return {
                        style: '2px dashed ' + color,
                        width: 2
                    };
                case 14:
                    //LINE_HAIR2
                    return {
                        style: '1px dashed ' + color,
                        width: 1
                    };
                case 15:
                    //LINE_DOUBLE_DOT
                    return {
                        style: '1px dotted ' + color,
                        width: 1
                    };
                case 16:
                    //LINE_LARGE
                    return {
                        style: '3px solid ' + color,
                        width: 3
                    };
                case 17:
                    //LINE_SLIM
                    return {
                        style: '1px solid ' + color,
                        width: 1
                    };
                default:
                    return {
                        style: 'none',
                        width: 0
                    };
            }
        },

        /**
         * 判断控件是否为异步加载的数据控件，有WT.ComboBox, WT.ComboCheckBox
         * @param target 控件对象实例或者控件config
         */
        isDataWidget: function (target) {
            if (target instanceof WT.ComboBox) {
                return true;
            }
            if (target instanceof WT.ComboCheckBox) {
                return true;
            }
            if (target instanceof WT.WebBox) {
                return true;
            }
            if (target instanceof WT.RadioGroup) {
                return true;
            }
            if (target instanceof WT.CheckBoxGroup) {
                return true;
            }
            if (target instanceof WT.DateEditor) {
                return true;
            }
            if (target.xtype) {
                if (target.xtype === 'WT.ComboBox') {
                    return true;
                }
                if (target.xtype === 'WT.ComboCheckBox') {
                    return true;
                }
                if (target.xtype === 'WT.WebBox') {
                    return true;
                }
                if (target.xtype === 'WT.RadioGroup') {
                    return true;
                }
                if (target.xtype === 'WT.CheckBoxGroup') {
                    return true;
                }
                if (target.xtype === 'WT.DateEditor') {
                    return true;
                }
            }
            return false;
        },

        /**
         * 判断控件是否为异步加载的数据控件，有WT.ComboBox, WT.ComboCheckBox
         * @param target 控件对象实例或者控件config
         */
        isRadioOrCheckBox: function (target) {
            if (target instanceof WT.CheckBox) {
                return true;
            }
            if (target instanceof WT.RadioGroup) {
                return true;
            }
            if (target instanceof WT.CheckBoxGroup) {
                return true;
            }
            if (target.xtype) {
                if (target.xtype === 'WT.CheckBox') {
                    return true;
                }
                if (target.xtype === 'WT.RadioGroup') {
                    return true;
                }
                if (target.xtype === 'WT.CheckBoxGroup') {
                    return true;
                }
            }
            return false;
        },

        isTreeWidget: function (target) {
            if (target === 'WT.TreeCombo' || target === 'WT.TreeView') {
                return true;
            }
            if (target instanceof WT.TreeCombo) {
                return true;
            }
            if (target instanceof WT.TreeView) {
                return true;
            }
            if (target && target.xtype) {
                if (target.xtype === 'WT.TreeCombo') {
                    return true;
                }
                if (target.xtype === 'WT.TreeView') {
                    return true;
                }
            }
            return false;
        },

        replaceSpecialChar4CellDisplay: function (str) {
            if (!str) {
                return str;
            }
            if (str.indexOf('  ') !== -1) {
                str = str.replaceAll('\\s{2,}', function (word) {
                    var s = ' ';
                    for (var i = 0; i < word.length - 1; i++) {
                        s += '&nbsp;';
                    }
                    s += ' ';
                    return s;
                });
            }
            if (str.indexOf('\\n') !== -1) {
                str = str.replaceAll('\\\\n', '<br/>')
            }
            return str;
        },

        /**
         * 转换从单元格中取出的值类型
         * Date类型转换为：{date_milliseconds}
         * Attachment类型转换为：
         * Array类型转换为：
         *
         * @param value
         * @return {*}
         */
        transformValueType: function (value) {
            if (value instanceof Date) {
                return {
                    date_milliseconds: value.getTime()
                }
            }
            return value;
        },
        generateReportData: function (cellList) {
            //临时方案,json传输稳定后直接同意采用json传输
            return report.dataTransType === 'json' ? this.generateReportJson(cellList) : this.generateReportXML(cellList);
        },
        generateReportJson: function (cellList) {
            var reportJson = [];
            var hasDirtySheet = false;
            var list = cellList;
            WT.each(list, function (item) {
                var sheetJson = {};
                sheetJson.sheetIndex = item.sheetIndex;
                var cellsJsonArr = [];
                var cells = item.cells;
                WT.each(cells, function (cell) {
                    hasDirtySheet = true;
                    var tdJson = {};
                    tdJson.c = cell.column;
                    tdJson.r = cell.row;
                    tdJson.v = writeCellValue(cell);
                    cellsJsonArr.push(tdJson);

                });
                sheetJson.sheetData = cellsJsonArr;
                reportJson.push(sheetJson);
            });
            if (!hasDirtySheet) {
                return '';//优化下性能，如果没有脏格子，返回空
            }
            return FR.jsonEncode(reportJson);
        },
        generateReportXML: function (cellList) {
            var list = cellList;

            var xml = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>";
            xml += "<WorkBook>";
            xml += ("<Version>6.5</" + "Version>");

            WT.each(list, function (item) {
                var i = item.sheetIndex;
                xml += "<Report class=\"com.fr.report.WorkSheet\" name=\"" + i + "\">";
                xml += "<CellElementList>";
                var cells = item.cells;
                WT.each(cells, function (cell) {
                    xml += "<C c=\"" + cell.column + "\" r=\""
                            + cell.row + "\">";
                    xml += writeCellValue(cell);
                    xml += "</" + "C>";
                });
                xml += "</" + "CellElementList>";
                xml += "</" + "Report>";
            });

            xml += "</" + "WorkBook>";

            return xml;
        },

        createParaProvider: function (paraMap, sheetIndex) {
            var location = sheetIndex !== undefined ? {
                reportIndex: sheetIndex,
                column: 0,
                row: 0
            } : undefined;
            return {
                getPara: function (key) {
                    if (!WT.isString(key)) {
                        return null
                    }
                    var uk = key.toUpperCase();
                    var val = WT.isFunction(paraMap) ? paraMap(uk) : paraMap[uk];
                    if (val !== undefined) {
                        return val;
                    }
                    return WT.net.remoteEvaluateSync('=' + uk, location);
                },
                getParaAsync: function (key) {
                    if (!WT.isString(key)) {
                        return WT.createPromiseResult(null)
                    }
                    var uk = key.toUpperCase();
                    var val = WT.isFunction(paraMap) ? paraMap(key) : paraMap[uk];
                    if (val !== undefined) {
                        return WT.createPromiseResult(val);
                    }
                    return WT.net.remoteEvaluate('=' + key, location)
                }
            }
        },

        getImageAttachURL: function (imageId) {
            return FR.servletURL + '?op=fr_attach&cmd=ah_image&id=' + imageId;
        },

        /**
         * 是否为字符按键
         * @param keyCode {number}
         * @return {boolean}
         */
        isPrintableKey: function (keyCode) {
            return (keyCode >= 33 && keyCode <= 126);
        },

        /**
         * 判断两个rect包含关系
         * @param container {{left: number, top: number, width: number, height: number}}
         * @param rect {{left: number, top: number, width: number, height: number}}
         * @param isTolerant? {boolean} 是否为宽容模式，宽容模式时只要rect与container有相交部分即认为包含
         * @return {boolean} container包含rect返回true，否则false
         */
        rectContains: function (container, rect, isTolerant) {
            if (isTolerant) {
                if (rect.left + rect.width < container.left) {
                    return false;
                }
                if (rect.left > container.left + container.width) {
                    return false;
                }
                if (rect.top + rect.height < container.top) {
                    return false;
                }
                if (rect.top > container.top + container.height) {
                    return false;
                }
                return true;
            }
            if (rect.left < container.left) {
                return false;
            }
            if (rect.top < container.top) {
                return false;
            }
            if (rect.left + rect.width > container.left + container.width) {
                return false;
            }
            if (rect.top + rect.height > container.top + container.height) {
                return false;
            }
            return true;
        },

        isElementContains: function (parent, child) {
            var pEl = (parent instanceof $) ? parent[0] : parent;
            var cEl = (child instanceof $) ? child[0] : child;
            while (cEl && cEl !== document.body) {
                if (cEl === pEl) {
                    return true;
                }
                cEl = cEl.parentNode;
            }
            return false;
        },

        Keys: function () {
            var fn_array = [];
            FR.Keys.reg(function (e) {
                if ($.isFunction(fn_array[0])) {
                    fn_array[0](e);
                }
            });
            return {
                /**
                 * 给全局的document注册一个指定的函数
                 * @param {Function} fn 事件函数
                 */
                reg: function (fn) {
                    fn_array.removeWeakEqual(fn);
                    fn_array.unshift(fn);
                },

                /**
                 * 从全局的document中移除指定的函数
                 * @param {Function} fn 事件函数
                 */
                unreg: function (fn) {
                    fn_array.removeWeakEqual(fn)
                },

                enable: FR.Keys.enable
            }
        }()

    });

    /******************************************** private function ************************************************/

    function writeCellValue(cell) {
        var formula = cell.formula;
        var value = cell.value;
        if (formula != null) {
            if (report.dataTransType === 'json') {
                return {
                    t: "RFormula",
                    attr: ("" + formula).replace(/\n/ig, "\\\nn"),
                    fmov: writeObject(value)
                };
            }
            return "<O t=\"RFormula\"><Attributes>" + "<![CDATA[" + ("" + formula).replace(/\n/ig, "\\\nn") + "]]>"
                + "</" + "Attributes>" + writeObject(value)
                + "</" + "O>";
        }

        return writeObject(value);
    }

    function writeObject(value) {
        var xmlString = "";
        if (value == null) {
            return xmlString;
        }

        if (WT.isString(value)) {
            value = '"' + value.replace(/\n/ig, "\\n") + '"';
            value = FR.jsonDecode(value);
        }

        var xtype = "S", xcontent = null;
        if (typeof value == 'number') {
            xcontent = value;
            if (/\./.test("" + value) ) {
                xtype = "D";
                //科学计数法
            } else if((/e/i).test("" + value)){
                xtype = "D";
                // carl:因为sqltype中暂时没有BigInteger，所以都是BigDecimal吧
            }else if (("" + value).length >= 10 ) {
                xtype = "BigDecimal";
            } else {
                xtype = "I";
            }
        }
        // alex:因为是保存在attr里面的,所以日期型和Attachment的value用{}形式来表示
        else if (typeof value == 'object') {
            if (value.date_milliseconds != null) {
                xtype = "Date";
                xcontent = value.date_milliseconds;
            } else if (value.attach_id != null) {
                xtype = "Attachment";
                xcontent = value.attach_id;
            } else if ($.isArray(value) && value.length > 0 && value[0].attach_id) {
                xtype = "Attachments";
                xcontent = "";
                for (var i = 0; i < value.length; i++) {
                    if (i !== 0) {
                        xcontent += ",";
                    }
                    xcontent += value[i].attach_id;
                }
            } else if ($.isArray(value)) {
                xtype = "A";
                xcontent = FR.jsonEncode(value);
            } else if ($.isEmptyObject(value)) {
                xcontent = "";
            } else if (value instanceof Date) {
                xtype = 'Date';
                xcontent = value.getTime();
            }
        } else if (typeof value == 'boolean') {
            xtype = "B";
        } else if (typeof value == 'string') {
            if (value.encryption === true) {
                xtype = "Encrypt";
                xcontent = FR.encrypt(value);
            } else {
                value = FR.jsonEncode(value);
                value = value.substring(1, value.length - 1);
            }
        }
        if (xcontent == null) {
            xcontent = "" + value;
        }
        if (report.dataTransType === 'json') {
            return {
                t: xtype,
                ov: ("" + xcontent).replace(/\n/ig, "\\\nn")
            };
        }
        return "<O t=\"" + xtype + "\">" +
                "<![CDATA[" + ("" + xcontent).replace(/\n/ig, "\\\nn") + "]]>" +
                "</" + "O>";
    }

})();