/**
 * 新填报下数据源处理类
 * @class WT.DataSource
 * @extends WT.Widget
 *
 * @cfg {JSON} options 配置属性
 * @cfg {Array} options.data 数据数组
 * @cfg {Function} options.dataLoader 数据加载器方法，返回Promise
 */
WT.DataSource = FR.extend(FR.OB, {

    _defaultConfig: function () {
        return $.extend(WT.DataSource.superclass._defaultConfig.apply(), {
            data: null,
            dataLoader: null
        });
    },

    _init: function () {
        WT.DataSource.superclass._init.apply(this);
        /**
         * 任务序号计数器
         * @type {number}
         */
        this.counter = 0;
    },

    _addCounter: function () {
        if (this.counter === Number.MAX_SAFE_INTEGER) {
            this.counter = 1;
        }
        else {
            this.counter++;
        }
    },

    /**
     * 加载数据
     * @param args 参数
     * @param hash 任务对应标识
     */
    loadData: function (args, hash) {
        var o = this.options, self = this;
        if (o.dataLoader) {
            this.argsHash = hash;
            this.args = args;
            this._addCounter();
            var thisCounter = this.counter;
            o.dataLoader(o.data, args).then(function (result) {
                if (result.type === 'chunk') {
                    var lastData = self.options.data;
                    self.options.data = [];
                    WT.each(lastData, function (d) {
                        self.options.data.push(d);
                    });
                    WT.each(result.data, function (d) {
                        self.options.data.push(d);
                    });
                    self.fireEvent(WT.DataSource.EVENT_DATA_APPEND, result.data, args);
                }
                else {
                    //判断该任务是否已经过时
                    if (thisCounter !== self.counter) {
                        return;
                    }
                    self.options.data = result.data;
                    self.fireEvent(WT.DataSource.EVENT_DATA_UPDATED, result.data);
                }
            })['catch'](function (err) {
                //判断该任务是否已经过时
                if (thisCounter !== self.counter) {
                    return;
                }
                self.fireEvent(WT.DataSource.EVENT_DATA_UPDATE_ERROR, err);
            });
        }
        else {
            WT.nextTick(function () {
                self.fireEvent(WT.DataSource.EVENT_DATA_UPDATE_ERROR);
            });
        }
    },

    loadRawDataSync: function (args) {
        var a = $.extend({}, args);
        var o = this.options;
        return o.dataLoaderSync(o.data, a).data;
    },

    hasData: function (hash) {
        return (this.options.data != null) && this.argsHash === hash;
    },

    clearData: function () {
        this.options.data = null;
    },

    getData: function () {
        return this.options.data;
    },

    getArgs: function () {
        return this.args;
    }

});
WT.DataSource.EVENT_DATA_UPDATED = 'event-data-updated';
WT.DataSource.EVENT_DATA_APPEND = 'event-data-append';
WT.DataSource.EVENT_DATA_UPDATE_ERROR = 'event-data-update-error';

WT.DataSource.EMPTY = new WT.DataSource({
    dataLoader: function () {
        return WT.createPromiseResult([]);
    }
});