(function () {
    //定义树扩展的方法
    var treePrototype = {
        _init: function (renderEl, config, node, forestIn) {
            this._initFields(renderEl, config, node, forestIn);
            this._initAttrs();
            this._initRoot();
            this._addClickListener();
            //只有当当前节点不是叶子节点，且节点expand属性为真时，才在开始时初始化子节点
            if (node.isexpand()) {
                this.toggleShowChildren();
            }
        },

        /**
         *  判断是不是叶子节点
         */
        isLeaf: function () {
            return !this.node.hasChild();
        },

        /**
         * 判断是否只允许选择叶子节点
         */
        isSelectLeafOnly: function () {
            return this.config.selectLeafOnly;
        },

        /**
         * 执行用户自定义点击check事件
         */
        customClickEvent: function () {
            var result = true;
            var dfop = this.config;
            if (dfop.oncheckboxclick) {
                var newState = this.getNodeState() !== 1 ? 1 : 0;
                result = dfop.oncheckboxclick.call(null, this.node, newState);
            }
            return result;
        },

        /**
         * 选中根节点，通过设置model的状态:0-没选；1-选中；2-不全选
         * 并触发选中事件
         */
        singleSelect: function () {
            this._unSelectAll();
            $(this.root).addClass('fr-tree-selected');
            this.config.currentnode = this.node;
            this._selectedChanged();
        },

        _unSelectAll: function() {
            var f = this.forest;
            while (f.father) {
                f = f.father;
            }
            $('.fr-tree-selected', $(f.view)).removeClass('fr-tree-selected');
        },

        /**
         * 发送选中更新事件
         */
        _selectedChanged: function () {
            var dfop = this.config;
            if (dfop.onnodeclick) {
                dfop.onnodeclick.call(this.node);
            }
        },
        /**
         * 点击check按钮
         */
        checkClicked: function () {
            var config = this.config;
            if (!config.showcheck) {
                return;
            }
            //可选择中间层节点
            if (!config.selectLeafOnly) {
                this.toggleNodeState();
                this.toggleCheckBoxState();
            } else {
                this.traversalSelect();
            }
            this._selectedChanged();
        },
        /**
         * 以遍历的方式来选中节点：
         * 1、根据当前节点的选择状态，进行不同的操作：
         *  当前节点是未选：选中该节点及所有子节点，并重新计算父节点的选中状态
         *  当前节点是未全选：同上
         *  当前节点是全选状态：取消选中所有子节点，并重新计算父节点的选中状态
         */
        traversalSelect: function () {
            var currentState = this.getCheckBoxState();
            //非全选
            if (currentState !== 1) {
                //全选
                this.setAllNodeState(1);
            }
            else {
                //全不选
                this.setAllNodeState(0);
            }
            this.informFather();
        },
        setAllNodeState: function (state) {
            this.setNodeState(state);
            this.setCheckBoxState(state);
            if (!this.isLeaf()) {
                if (!this.childrenForest) {
                    this.initChildren();
                }
                this.childrenForest.setAllNodeState(state);
            }
        },
        informFather: function () {
            if (this.father) {
                this.father.childrenStateChanged();
            }
        },
        childrenStateChanged: function () {
            //通常不会有这种情况
            if (!this.childrenForest) {
                this.initChildren();
            }
            //在更新该节点的状态之前是什么状态
            var oriState = this.getCheckBoxState();
            var childrenState = this.childrenForest.getCheckState();
            //如果当前节点的状态发生了改变，则更新状态
            if (oriState != childrenState) {
                this.setCheckBoxState(childrenState);
                this.setNodeState(childrenState === 0 ? 0 : 1);
                this.informFather();
            }
        },
        /**
         * 更改节点的选择状态
         */
        toggleNodeState: function () {
            if (this.getNodeState()) {
                this.setNodeState(0);
            } else {
                this.setNodeState(1);
            }
        },
        /**
         * 设置节点的选择状态：0未选中，1选中
         * @param state
         */
        setNodeState: function (state) {
            this.node.setcheckstate(state);
        },
        /**
         * 获取节点的选择状态
         */
        getNodeState: function () {
            return this.node.getcheckstate();
        },
        toggleCheckBoxState: function () {
            if (this.getCheckBoxState()) {
                this.setCheckBoxState(0);
            } else {
                this.setCheckBoxState(1);
            }
        },
        getCheckBoxState: function () {
            return this.checkBoxState;
        },

        /**
         * 设置选择框的状态，0未选，1全选，2未全选
         */
        setCheckBoxState: function (state) {
            this.checkBoxState = state;
            var dfop = this.config;
            $(this.checkIcon).attr("src", dfop.cbiconpath + dfop.icons[state]);
        },

        /**
         * 改变子节点ul的可见性
         */
        toggleShowChildren: function () {
            if (this.isLeaf()) {
                return;
            }
            //还没有初始化dom节点，初始化并显示
            if (!this.childrenForest) {
                this.initChildren(true);
            }
            this._toggleExpandCss();

        },
        _toggleExpandCss: function () {
            var dfop = this.config;
            //当前没显示
            var $expandIcon = $(this.expandIcon);
            if (!this.showChildren) {
                //当前没有显示，则把icon变为减号
                $expandIcon.addClass(dfop.closeClass).removeClass(dfop.expandClass);
                $(this.root).addClass('fr-tree-node-expanded').removeClass('fr-tree-node-collapsed');
                if (this.childrenForest) {
                    //把根节点的样式该为显示
                    $(this.childrenForest.view).show();
                    this.showChildren = true;
                }
            }
            else {
                //加号的样式切换
                $expandIcon.addClass(dfop.expandClass).removeClass(dfop.closeClass);
                $(this.root).addClass('fr-tree-node-collapsed').removeClass('fr-tree-node-expanded');
                if (this.childrenForest) {
                    $(this.childrenForest.view).hide();
                    this.showChildren = false;
                }
            }
        },
        /**
         * 初始化子节点，用于第一次展开或者默认展开的情况
         */

        initChildren: function (forceShow) {
            $(this.view).addClass("fr-tree-node-loading");
            var childrenNodes = this._getChildrenNodes();
            //展开则说明 不可见性为false
            this.childrenForest = FR.Core.Plugin.Tree.createForest(this.config, childrenNodes, true, true, this, forceShow);
            if (this.childrenForest) {
                this.view.appendChild(this.childrenForest.getView());
            }
            $(this.view).removeClass("fr-tree-node-loading");
        },

        applyTreeState: function () {
            this.setCheckBoxState(this.node.getcheckstate());
            if (this.showChildren !== this.node.isexpand()) {
                this.toggleShowChildren();
            }
            if (!this.isLeaf()) {
                if (!this.childrenForest) {
                    this.initChildren();
                }
                this.childrenForest.applyTreesState();
            }
        },

        /**
         * 获取子节点数据
         */
        _getChildrenNodes: function () {
            return this.node.getChildren(this.config.afterAjax);
        },
        _initAttrs: function () {
            this.view.className = 'fr-tree-node';
        },
        _isLastSon: function () {
            return this.forest.getTree(this.forest.getSize() - 1) == this;
        },

        _initFields: function (view, config, node, forest) {
            //标识符，用于表示它是一个扩展后的树dom
            this.view = view;
            this.node = node;
            this.config = config;
            this.forest = forest;
            this.father = forest.father;
            this.root = null;
            this.expandIcon = null;
            this.packageIcon = null;
            this.checkIcon = null;
            this.checkBoxState = 0;
            //子节点，扩展后的dom对象
            this.childrenForest = null;
            this.showChildren = false;
        },
        _initRoot: function () {
            var root = document.createElement('div');
            //标记
            root.setAttribute('id', this.getId());
            root.setAttribute('unselectable', 'on');
            root.setAttribute('title', this.node.getShowValue());
            root.className = this._generateClass();
            root.appendChild(this._createPlace());
            root.appendChild(this._createExpandIcon());
            root.appendChild(this._createPackageIcon());
            if (this.config.showcheck && this.node.getshowcheck()) {
                root.appendChild(this._createCheckIcon());
            }
            root.appendChild(this._createValue());
            this.root = root;
            this._addHoverListener();
            this.view.appendChild(root);
        },
        _addHoverListener: function () {
            var tree = this;
            this.root.onmouseover = function () {
                if (!tree.widgetDisabled()) {
                    if (tree) {
                        $(tree.root).addClass("fr-tree-node-over");
                    }
                }
            };
            this.root.onmouseout = function () {
                if (!tree.widgetDisabled()) {
                    $(tree.root).removeClass("fr-tree-node-over");
                }
            };
        },
        _addClickListener: function () {
            if (this.config.showcheck) {
                this._addMultiSelectClickListener();
            } else {
                this._addSingleSelectClickListener();
            }
        },
        _addMultiSelectClickListener: function () {
            var tree = this;
            this.checkIcon.onclick = function () {
                if (!tree.widgetDisabled()) {
                    var r = tree.customClickEvent();
                    if (r !== false) {
                        tree.checkClicked();
                    }
                }
            };
            this.expandIcon.onclick = function () {
                tree.toggleShowChildren();
            };
        },
        _addSingleSelectClickListener: function () {
            var tree = this;
            //展开按钮，如果不是叶子则展开
            this.expandIcon.onclick = function (e) {
                if (FR.Browser.isIE()) {
                    e = window.event;
                }
                if (!tree.isLeaf()) {
                    tree.toggleShowChildren();
                    if (FR.Browser.isIE()) {
                        e.cancelBubble = true;
                    } else {
                        e.stopPropagation();
                    }
                }
            };
            this.root.onclick = function () {
                //点击的是叶子节点，则选中，否则展开
                if (tree.isLeaf() || !tree.isSelectLeafOnly()) {
                    tree.singleSelect();
                } else {
                    tree.toggleShowChildren();
                }
            };
            this.root.ondblclick = function () {
                if (!tree.isLeaf()) {
                    tree.toggleShowChildren();
                }
            }
        },
        _createPlace: function () {
            var span = document.createElement('span');
            span.className = 'fr-tree-node-indent';
            for (var i = 0, len = this.node.getAncestorsLength(); i < len; i++) {
                var cs = i === 0 ? 'fr-tree-icon' : 'fr-tree-elbow-line';
                var placeImg = this._createEmptyImg();
                placeImg.className = cs;
                span.appendChild(placeImg);
            }
            return span;
        },
        _createExpandIcon: function () {
            var img = this._createEmptyImg();
            img.className = this._generateExpandIconClass();
            this.expandIcon = img;
            return img;
        },
        _createPackageIcon: function () {
            var img = this._createEmptyImg();
            img.className = 'fr-tree-node-icon';
            this.packageIcon = img;
            return img;
        },
        _createCheckIcon: function () {
            var node = this.node;
            var img = document.createElement('img');
            img.setAttribute('src', this.config.cbiconpath + this.config.icons[node.getcheckstate() ? node.getcheckstate() : 0]);
            img.className = this._getCheckIconClass();
            img.setAttribute('id', this.getId() + '_cb');
            this.checkIcon = img;
            return img;
        },
        _getCheckIconClass: function () {
            var cls = ['fr-tree-node-cb'];
            if (this.node.isUnselectable()) {
                cls.push('fr-tree-node-unselectable');
            }
            return cls.join(' ');
        },
        _createEmptyImg: function () {
            var img = document.createElement('img');
            img.setAttribute('src', FR.servletURL + '?op=resource&resource=/com/fr/web/core/js/jquerytree/images/s.gif');
            return img;
        },
        _generateExpandIconClass: function () {
            var node = this.node;
            var cs = [];
            var isend = this._isLastSon();
            if (node.hasChild()) {
                if (node.isexpand()) {
                    cs.push(isend ? this.config.closeEndClass : this.config.closeClass);
                } else {
                    cs.push(isend ? this.config.expandEndClass : this.config.expandClass);
                }
            } else {
                cs.push(isend ? "fr-tree-elbow-end" : "fr-tree-elbow");
            }
            return cs.join(" ");
        },
        _createValue: function () {
            var a = document.createElement('a');
            a.setAttribute('hideFocus', 'true');
            a.className = 'fr-tree-node-anchor';
            a.setAttribute('tabIndex', '1');
            var span = document.createElement('span');
            span.setAttribute('unselectable', 'on');
            span.innerHTML = this.node.getShowValue();
            a.appendChild(span);
            return a;
        },
        /**
         * 获取id
         */
        getId: function () {
            return this.config.id_prefix + '_' + this.node.getID();
        },
        /**
         * 生成根节点样式
         */
        _generateClass: function () {
            var node = this.node;
            var classes = ['fr-tree-node-el'];
            node.hasChild() ? (classes.push(node.isexpand() ? "fr-tree-node-expanded" : "fr-tree-node-collapsed")) :
                (classes.push(node.getnodeicon() ? 'fr-tree-node-' + node.getnodeicon() : "fr-tree-node-leaf"));
            if (node == this.config.currentnode) {
                classes.push("fr-tree-selected")
            }
            classes.push(node.getclasses());
            return classes.join(' ');
        },
        widgetDisabled: function () {
            return this.config.widget.options.disabled;
        },
        getView: function () {
            return this.view;
        }
    };
    //定义森林扩展的方法
    var forestPrototype = {
        _init: function (renderEl, config, nodes, invisible, father, forceShow) {
            this._initFields(renderEl, config, nodes, father, forceShow);
            this._initAttrs(invisible);
            this._initElements();
        },
        /**
         * 遍历设置所有树
         */
        setAllNodeState: function (state) {
            var trees = this.trees;
            var len = trees.length;
            for (var i = 0; i < len; i++) {
                trees[i].setAllNodeState(state);
            }
        },
        /**
         * 查看当前森林的状态：0未选，1全选，2未全选
         */
        getCheckState: function () {
            var state = 1;
            var trees = this.trees;
            var len = trees.length;
            var childState;
            //统计子节点为1的个数
            var checkedCount = 0;
            for (var i = 0; i < len; i++) {
                childState = trees[i].getCheckBoxState();
                //如果存在状态未2的子节点，则森林的状态一定是2
                if (childState == 2) {
                    state = 2;
                    return state;
                }
                else if (childState == 1) {
                    ++checkedCount;
                }
            }
            state = checkedCount === len ? 1 : (checkedCount == 0 ? 0 : 2);
            return state;
        },
        _initAttrs: function (invisible) {
            this.view.className = 'fr-tree-root ' + this.config.theme;
            if (invisible) {
                $(this.view).hide();
            }
        },
        getTree: function (i) {
            return this.trees[i];
        },
        getSize: function () {
            return this.trees.length;
        },
        getView: function () {
            return this.view;
        },
        applyTreesState: function () {
            WT.each(this.trees, function (tree) {
                tree.applyTreeState();
            });
        },
        _initFields: function (view, config, nodes, father, forceShow) {
            this.view = view;
            this.config = config;
            this.father = father;
            this._forceShow = forceShow;
            this.nodes = nodes;
            this.trees = [];
        },
        _initElements: function () {
            var len = this.nodes.length;
            var tree;
            var editVal = null;
            if (this.config.widget && this.config.widget.getEditValue) {
                editVal = this.config.widget.getEditValue();
            }
            for (var i = 0; i < len; i++) {
                if (editVal && this.config.widget.options.directEdit) {
                    if (editVal.length > 1 || (editVal.length === 1 && editVal[0].length !== 0)) {
                        var node = this.nodes[i];
                        if (!node.isexpand() && this.father && !this._forceShow) {
                            continue;
                        }
                    }
                }
                tree = FR.Core.Plugin.Tree.createTree(this.config, this.nodes[i], this);
                this.trees.push(tree);
                this.view.appendChild(tree.getView());
            }
        }

    };

    function TreeCreator() {
    }

    function ForestCreator() {
    }

    $.extend(TreeCreator.prototype, treePrototype);
    $.extend(ForestCreator.prototype, forestPrototype);
    //外部接口
    $.extend(FR.Core.Plugin.Tree, {
        /**
         * 树工厂方法
         * @param config 全局配置信息
         * @param node 树model
         */
        createTree: function (config, node, forestIn) {
            //获取一个扩展后的Dom节点，用于表示一棵树
            var tree = new TreeCreator();
            tree._init(document.createElement('li'), config, node, forestIn);
            return tree;
        },

        /**
         * 森林构造函数
         * @param config 全局配置信息
         * @param nodes 森林model
         * @param addNoListener 内部使用，表示不添加任何事件
         * @param invisible 内部使用，表示开始时不显示
         */
        createForest: function (config, nodes, addNoListener, invisible, father, forceShow) {

            if (nodes) {
                var forest = new ForestCreator();
                forest._init(document.createElement('ul'), config, nodes, invisible, father, forceShow);
                if (config.treeCreator && !father) {
                    config.treeCreator.forest = forest;
                }
                return forest;
            }
            else {
                return null;
            }
        }
    });
})();