/**
 * 包含各种网络请求方法
 * Created by loy on 2017/6/2.
 */

if (!window.WT) {
    window.WT = {};
}

$.extend(window.WT, {

    /**
     * ajax, 返回Promise对象
     * @param options ajax参数
     * @return {*|Promise.<T>}
     */
    ajax: function (options) {
        return WT.createPromise(function (resolve, reject) {
            $.extend(options, {
                success: function (data, status, xhr) {
                    resolve({
                        data: data,
                        status: status,
                        xhr: xhr
                    });
                },
                error: function (xhr, status, err) {
                    reject({
                        err: err,
                        status: status,
                        xhr: xhr
                    });
                }
            });
            FR.ajax(options);
        });
    },

    ajaxSync: function (options) {
        var result = null;
        $.extend(options, {
            async: false,
            success: function (data, status, xhr) {
                result = {
                    data: data,
                    status: status,
                    xhr: xhr
                };
            },
            error: function (xhr, status, err) {
                result = {
                    err: err,
                    status: status,
                    xhr: xhr
                };
            }
        });
        FR.ajax(options);
        return result;
    },

    ajaxSubmit: function ($form, options) {
        if (!($form instanceof $)) {
            $form = $($form);
        }
        return WT.createPromise(function (resolve, reject) {
            $.extend(options, {
                success: function (data, status, xhr) {
                    resolve({
                        data: data,
                        status: status,
                        xhr: xhr
                    });
                },
                error: function (xhr, status, err) {
                    reject({
                        err: err,
                        status: status,
                        xhr: xhr
                    });
                }
            });
            FR.dealWithSessionIDParam(options);
            $form.ajaxSubmit(options);
        });
    }
});

window.WT.net = {};

$.extend(WT.net, {
    /**
     * 获取控件数据（WT.ComboBox和WT.ComboCheckBox）
     * @param widgetName
     * @param opts {Object} {search?, start?, limit?}
     * @return {Promise.<TResult>}
     */
    getWidgetData: function (widgetName, opts) {
        var searchStr = opts.search;
        var startIndex = opts.start || 0;
        var limitIndex = WT.isNumber(opts.limit) ? opts.limit : 500;
        var query = {
            op: 'widget',
            cmd: 'getdata',
            widgetname: widgetName,
            reload: true,
            startIndex: startIndex,
            limitIndex: limitIndex,
            sessionID: FR.SessionMgr.getSessionID()
        };
        if (searchStr) {
            query.dataFilter = searchStr;
        }
        if (opts.dependence) {
            query.dependence = opts.dependence;
        }
        var url = FR.buildServletUrl(query);
        return WT.ajax({
            url: url
        }).then(function (result) {
            return FR.jsonDecode(result.data);
        }).then(function (data) {
            if (startIndex > 0) {
                return {
                    data: data,
                    type: 'chunk'
                }
            }
            else {
                return {
                    data: data,
                    type: 'all'
                }
            }
        });
    },
    /**
     * 获取控件数据（WT.ComboBox和WT.ComboCheckBox）
     * @param widgetName
     * @param opts {Object} {search?, start?, limit?}
     * @return {Promise.<TResult>}
     */
    getWidgetDataSync: function (widgetName, opts) {
        var searchStr = opts.search;
        var startIndex = opts.start || 0;
        var limitIndex = WT.isNumber(opts.limit) ? opts.limit : 500;
        var query = {
            op: 'widget',
            cmd: 'getdata',
            widgetname: widgetName,
            reload: true,
            startIndex: startIndex,
            limitIndex: limitIndex,
            sessionID: FR.SessionMgr.getSessionID()
        };
        if (searchStr) {
            query.dataFilter = searchStr;
        }
        if (opts.dependence) {
            query.dependence = opts.dependence;
        }
        var url = FR.buildServletUrl(query);
        var result = WT.ajaxSync({
            url: url
        });
        var data = FR.jsonDecode(result.data);
        if (startIndex > 0) {
            return {
                data: data,
                type: 'chunk'
            }
        } else {
            return {
                data: data,
                type: 'all'
            }
        }
    },
    /**
     * 获取单元格中的控件数据（WT.ComboBox和WT.ComboCheckBox）
     * @param location 包含{sheetIndex, column, row}
     * @param opts {Object} {search?, start?, limit?}
     */
    getCellWidgetData: function (location, opts) {
        location.reportIndex = location.sheetIndex;
        opts = opts || {};
        var searchStr = opts.search;
        var query = {
            op: 'widget',
            cmd: 'getdata',
            location: location,
            reload: true,
            sessionID: FR.SessionMgr.getSessionID()
        };
        if (opts.start !== undefined) {
            query.startIndex = opts.start || 0;
        }
        if (WT.isNumber(opts.limit)) {
            query.limitIndex = opts.limit || 500;
        }
        if (searchStr) {
            query.dataFilter = searchStr;
        }
        if (opts.dependence) {
            query.dependence = opts.dependence;
        }
        var url = FR.buildServletUrl(query);
        return WT.ajax({
            url: url
        }).then(function (result) {
            return FR.jsonDecode(result.data);
        }).then(function (data) {
            if (query.startIndex !== undefined && query.startIndex > 0) {
                return {
                    data: data,
                    type: 'chunk'
                }
            } else {
                return {
                    data: data,
                    type: 'all'
                }
            }
        });
    },
    /**
     * 获取单元格中的控件数据（WT.ComboBox和WT.ComboCheckBox）
     * @param location 包含{sheetIndex, column, row}
     * @param opts {Object} {search?, start?, limit?}
     */
    getCellWidgetDataSync: function (location, opts) {
        location.reportIndex = location.sheetIndex;
        opts = opts || {};
        var searchStr = opts.search;
        var query = {
            op: 'widget',
            cmd: 'getdata',
            location: location,
            reload: true,
            sessionID: FR.SessionMgr.getSessionID()
        };
        if (opts.start !== undefined) {
            query.startIndex = opts.start || 0;
        }
        if (WT.isNumber(opts.limit)) {
            query.limitIndex = opts.limit || 500;
        }
        if (searchStr) {
            query.dataFilter = searchStr;
        }
        if (opts.dependence) {
            query.dependence = opts.dependence;
        }
        var url = FR.buildServletUrl(query);
        var result = WT.ajaxSync({
            url: url
        });
        var data = FR.jsonDecode(result.data);
        if (query.startIndex !== undefined && query.startIndex > 0) {
            return {
                data: data,
                type: 'chunk'
            }
        } else {
            return {
                data: data,
                type: 'all'
            }
        }
    },
    /**
     * 同步参数面板里联动的控件数据
     * @param para 参数面板参数
     * @param widgetNameList 需要返回控件值的控件名列表
     * @return {Promise.<TResult>}
     */
    synchronizeFormData: function (para, widgetNameList) {
        var url = FR.buildServletUrl({
            op: 'fr_form',
            cmd: 'form_getsource',
            sessionID: FR.SessionMgr.getSessionID()
        }, true);
        return WT.ajax({
            url: url,
            type: 'POST',
            data: {
                __parameters__: para,
                __widgetname__: widgetNameList
            }
        }).then(function (result) {
            return FR.jsonDecode(result.data);
        });
    },
    /**
     * 提交参数面板的参数
     * @param para 参数面板参数
     * @return {*|Promise.<T>}
     */
    submitFormData: function (para) {
        var url = FR.buildServletUrl({
            op: 'fr_dialog',
            cmd: 'parameters_d',
            sessionID: FR.SessionMgr.getSessionID()
        }, true);
        return WT.ajax({
            url: url,
            type: 'POST',
            data: {
                __parameters__: para
            }
        });
    },
    /**
     * 获取sheet的基本信息
     * @param sheetIndex sheet序号
     * @param el 出错时显示错误信息的jquery对象
     * @return {Promise.<TResult>}
     */
    getSheetInfo: function (sheetIndex, el) {
        // 收集前台埋点
        FR.collectPreviewInfo(FR.SessionMgr.getSessionID());

        var para = {
            op: 'fr_write_plus',
            cmd: 'sheet_info',
            sessionID: FR.SessionMgr.getSessionID(),
            sheetIndex: sheetIndex
        };
        this.extendPara(para, contentPane);
        var url = FR.buildServletUrl(para, true);
        return WT.ajax({
            url: url
        }).then(function (result) {
            var r = WT.dealWithHtmlResult(result, el);
            return FR.jsonDecode(r.data);
        });
    },


    extendPara: function (para, contentPane) {
        var documentHeight = FR.windowHeight;
        if (documentHeight === 0) {
            return;
        }
        var topBar = _g().topToolBar;
        var bottomBar = _g().sheetContainer.tabBar;
        var parameterPanel = _g().parameterPanel;
        var extraHeight = 0;
        if (topBar !== undefined && topBar.element) {
            extraHeight += parseInt(topBar.element.css("height"));
        }
        if (bottomBar !== undefined && bottomBar.element) {
            extraHeight += parseInt(bottomBar.element.css("height"));
        }
        if (parameterPanel !== undefined) {
            extraHeight += parameterPanel.options.height;
        }
        documentHeight -= extraHeight;
        $.extend(para, {
            __webpage__: true,
            _paperWidth: FR.windowWidth,
            _paperHeight: documentHeight,
            __fit__: contentPane.__fit__
        });
    },
    /**
     * 根据范围获取单元格数据
     * @param sheetIndex sheet序号
     * @param block block名称 'corner','north','west','center'
     * @param rect 范围信息{left, top, width, height}
     * @param el 出错时显示错误信息的jquery对象
     * @return {Promise.<TResult>}
     */
    getCellsByRange: function (sheetIndex, block, rect, el) {
        var url = FR.buildServletUrl({
            op: 'fr_write_plus',
            cmd: 'cells_by_range',
            sessionID: FR.SessionMgr.getSessionID(),
            sheetIndex: sheetIndex,
            block: block,
            left: rect.left,
            top: rect.top,
            width: rect.width,
            height: rect.height
        }, true);
        return WT.ajax({
            url: url
        }).then(function (result) {
            var r = WT.dealWithHtmlResult(result, el);
            return FR.jsonDecode(r.data);
        });
    },

    /**
     * 插入行
     * @param location 包含{sheetIndex, column, row}
     * @param count 插入行数
     * @param isBeforePoint 是否在插入点前插入
     * @param isSimple 是否简单模式
     * @param el 出错时显示错误信息的jquery对象
     * @return {Promise.<TResult>}
     */
    appendRow: function (location, count, isBeforePoint, isSimple, el) {
        var url = FR.buildServletUrl({
            op: 'fr_write_plus',
            cmd: 'append_delete_row',
            sessionID: FR.SessionMgr.getSessionID()
        }, true);
        return WT.ajax({
            url: url,
            type: 'POST',
            data: {
                mode: 'insert',
                sheetIndex: location.sheetIndex,
                column: location.column,
                row: location.row,
                isBeforePoint: isBeforePoint,
                isSimple: isSimple,
                count: count
            }
        }).then(function (result) {
            var r = WT.dealWithHtmlResult(result, el);
            return FR.jsonDecode(r.data);
        }).then(function (data) {
            if (data.error) {
                throw new Error(data.error);
            }
            return data;
        });
    },

    /**
     * 删除行
     * @param location 包含{sheetIndex, column, row}
     * @param isDelete 是否为删除（或者为撤销删除）
     * @param isSimple 是否简单模式
     * @param el 出错时显示错误信息的jquery对象
     * @return {Promise.<TResult>}
     */
    deleteRow: function (location, isDelete, isSimple, el) {
        var url = FR.buildServletUrl({
            op: 'fr_write_plus',
            cmd: 'append_delete_row',
            sessionID: FR.SessionMgr.getSessionID()
        }, true);
        var data = {
            mode: 'delete',
            sheetIndex: location.sheetIndex,
            isSimple: isSimple,
            isDelete: isDelete
        };
        if (WT.isArray(location.crs)) {
            data.cr = WT.reduce(location.crs, function (prev, next) {
                if (!prev) {
                    return next.column + ',' + next.row;
                } else {
                    return prev + ';' + next.column + ',' + next.row;
                }
            }, null);
        } else {
            data.column = location.column;
            data.row = location.row;
        }
        return WT.ajax({
            url: url,
            type: 'POST',
            data: data
        }).then(function (result) {
            var r = WT.dealWithHtmlResult(result, el);
            return FR.jsonDecode(r.data);
        }).then(function (data) {
            if (data.error) {
                throw new Error(data.error);
            }
            return data;
        });
    },

    /**
     * 计算相关联的其他单元格值
     * @param location 包含{sheetIndex, column, row}
     * @param dirtyCellList
     * @param value? 若提供了value，则会包含在返回的受影响的格子中
     * @return {Promise.<TResult>}
     */
    calculateCellValue: function (location, dirtyCellList, value, fm) {
        var url = FR.buildServletUrl({
            op: 'fr_write',
            cmd: 'cal_write_cell',
            sessionID: FR.SessionMgr.getSessionID()
        }, true);
        var loadidxs = [];
        WT.each(_g().getSheetsInfo(), function (sheetInfo, idx) {
            if (sheetInfo.loaded) {
                loadidxs.push(idx);
            }
        });
        if (value !== undefined) {
            for (var i = 0; i < dirtyCellList.length; i++) {
                if (dirtyCellList[i].sheetIndex === location.sheetIndex) {
                    dirtyCellList[i].cells.push({
                        column: location.column,
                        row: location.row,
                        value: value,
                        formula: fm
                    });
                    break;
                }
            }
        }
        return WT.ajax({
            url: url,
            type: 'POST',
            data: {
                editcol: location.column,
                editrow: location.row,
                editReportIndex: location.sheetIndex,
                reportXML: WT.generateReportData(dirtyCellList),
                loadidxs: loadidxs
            }
        }).then(function (result) {
            var r = WT.dealWithHtmlResult(result);
            return FR.jsonDecode(r.data);
        }).then(function (data) {
            if (data.error) {
                if (Array.isArray(data.error)) {
                    FR.hideLoadingDialog();
                    throw new Error(FR.i18nText.apply(FR, data.error));
                } else {
                    throw new Error(data.error);
                }
            }
            if (!data.success) {
                throw new Error(FR.i18nText("Fine-Engine_Failed"));
            }
            return WT.map(data.array, function (item) {
                var value = item.cv;
                var pv = item.pv;
                var hy = item.hy;
                if (typeof value == 'object') {
                    if (value.date_milliseconds) {
                        value = new Date(value.date_milliseconds);
                    }
                }
                if (typeof pv == 'object') {
                    if (pv.date_milliseconds) {
                        pv = new Date(value.date_milliseconds);
                    }
                }
                return {
                    value: value,
                    pv: pv,
                    hy: hy,
                    location: {
                        sheetIndex: item.idx,
                        column: item.col,
                        row: item.row
                    }
                }
            });
        }).then(function (cellInfoList) {
            if (value !== undefined) {
                var cI = WT.find(cellInfoList, function (cellInfo) {
                    return cellInfo.location.column === location.column
                        && cellInfo.location.row === location.row;
                });
                if (!cI) {
                    cellInfoList.push({
                        value: value,
                        location: location
                    });
                }
            }
            var info = WT.map(cellInfoList, function (cellInfo) {
                return cellInfo.location;
            });
            if (info.length === 0) {
                return cellInfoList;
            }
            return WT.net.getCellID(info).then(function (result) {
                var cellIDList = result.idList;
                WT.each(cellIDList, function (obj, idx) {
                    var id = obj.id;
                    var widgetName = obj.widgetName;
                    if (cellInfoList[idx]) {
                        cellInfoList[idx].id = id;
                        cellInfoList[idx].widgetName = widgetName
                    }
                });
                return cellInfoList;
            });
        })
    },

    /**
     * 计算相关联的其他单元格值，批量
     * @param locations {Array<{col: number, row: number}>}
     * @param sheetIndex
     * @param dirtyCellList
     * @return {Promise.<TResult>}
     */
    calculateCellsValue: function (locations, sheetIndex, dirtyCellList) {
        var url = FR.buildServletUrl({
            op: 'fr_write',
            cmd: 'cal_write_cell',
            sessionID: FR.SessionMgr.getSessionID()
        }, true);
        var loadidxs = [];
        WT.each(_g().getSheetsInfo(), function (sheetInfo, idx) {
            if (sheetInfo.loaded) {
                loadidxs.push(idx);
            }
        });
        return WT.ajax({
            url: url,
            type: 'POST',
            data: {
                editcells: locations,
                editReportIndex: sheetIndex,
                reportXML: WT.generateReportData(dirtyCellList),
                loadidxs: loadidxs
            }
        }).then(function (result) {
            var r = WT.dealWithHtmlResult(result);
            return FR.jsonDecode(r.data);
        }).then(function (data) {
            if (data.error) {
                throw new Error(data.error);
            }
            if (!data.success) {
                throw new Error(FR.i18nText("Fine-Engine_Failed"));
            }
            return WT.map(data.array, function (item) {
                var value = item.cv;
                var pv = item.pv;
                var hy = item.hy;
                if (typeof value == 'object') {
                    if (value.date_milliseconds) {
                        value = new Date(value.date_milliseconds);
                    }
                }
                if (typeof pv == 'object') {
                    if (pv.date_milliseconds) {
                        pv = new Date(value.date_milliseconds);
                    }
                }
                return {
                    value: value,
                    pv: pv,
                    hy: hy,
                    location: {
                        sheetIndex: item.idx,
                        column: item.col,
                        row: item.row
                    }
                }
            });
        }).then(function (cellInfoList) {
            var info = WT.map(cellInfoList, function (cellInfo) {
                return cellInfo.location;
            });
            if (info.length === 0) {
                return cellInfoList;
            }
            return WT.net.getCellID(info).then(function (result) {
                var cellIDList = result.idList;
                WT.each(cellIDList, function (obj, idx) {
                    var id = obj.id;
                    var widgetName = obj.widgetName;
                    if (cellInfoList[idx]) {
                        cellInfoList[idx].id = id;
                        cellInfoList[idx].widgetName = widgetName
                    }
                });
                return cellInfoList;
            });
        })
    },

    getCellID: function (cellInfoList, needRect) {
        var url = FR.buildServletUrl({
            op: 'fr_write_plus',
            cmd: 'get_cell_id',
            sessionID: FR.SessionMgr.getSessionID()
        }, true);
        var data = {
            info: cellInfoList
        };
        if (needRect) {
            data.needRect = true;
        }
        return WT.ajax({
            url: url,
            type: 'POST',
            data: data
        }).then(function (result) {
            var r = WT.dealWithHtmlResult(result);
            return FR.jsonDecode(r.data);
        });
    },

    /**
     * 获取单元格显示值
     * @param value 实际值
     * @param col 列
     * @param row 行
     * @param reportIndex sheet号
     * @param dep 依赖
     */
    getCellDisplayValue: function (value, col, row, reportIndex, dep) {
        var url = FR.buildServletUrl({
            op: 'fr_write',
            cmd: 'write_present',
            sessionID: FR.SessionMgr.getSessionID()
        }, true);
        var v = value;
        if (WT.isObject(v) || WT.isArray(v)) {
            v = JSON.stringify(v);
        }
        return WT.ajax({
            url: url,
            type: 'POST',
            data: {
                value: v,
                dependence: dep,
                col: col,
                row: row,
                rptIdx: reportIndex
            }
        }).then(function (result) {
            var r = WT.dealWithHtmlResult(result);
            return FR.jsonDecode(r.data);
        });
    },

    /**
     * 数据校验
     * @param cellList 校验数据列表
     *                  {
     *                      sheetIndex: 0,
     *                      cells: [{
     *                          column: 1,
     *                          row: 2,
     *                          value: 'value'
     *                      }]
     *                  }
     * @param sheetIndex {number|null} 为数字时仅校验当前页
     * @return {Promise.<TResult>}
     */
    writeVerify: function (cellList, sheetIndex) {
        var url = FR.buildServletUrl({
            op: 'fr_write_plus',
            cmd: 'write_verify',
            sessionID: FR.SessionMgr.getSessionID()
        }, true);
        var data = {
            reportXML: WT.generateReportData(cellList)
        };
        if (typeof sheetIndex === 'number') {
            data.reportIndex = sheetIndex;
        }
        return WT.ajax({
            url: url,
            type: 'POST',
            data: data
        }).then(function (result) {
            var r = WT.dealWithHtmlResult(result);
            return FR.jsonDecode(r.data);
        });
    },

    /**
     * 提交
     * @param cellList 校验数据列表
     *                  {
     *                      sheetIndex: 0,
     *                      cells: [{
     *                          column: 1,
     *                          row: 2,
     *                          value: 'value'
     *                      }]
     *                  }
     * @param sheetIndex {number|null} 为数字时仅提交当前页
     * @return {Promise.<TResult>}
     */
    submitReport: function (cellList, sheetIndex) {
        var url = FR.buildServletUrl({
            op: 'fr_write',
            cmd: 'submit_w_report',
            sessionID: FR.SessionMgr.getSessionID()
        }, true);
        var data = {
            reportXML: WT.generateReportData(cellList)
        };
        if (WT.isNumber(sheetIndex)) {
            data.reportIndex = sheetIndex;
        }
        return WT.ajax({
            url: url,
            type: 'POST',
            data: data
        }).then(function (result) {
            var r = WT.dealWithHtmlResult(result);
            return FR.jsonDecode(r.data);
        });
    },

    /**
     * 请求刷新报表
     * @param sheetIndex? {number|null} 为数字时仅刷新当前页
     * @return {Promise.<TResult>}
     */
    refreshReport: function (sheetIndex) {
        var url = FR.buildServletUrl({
            op: 'fr_write_plus',
            cmd: 'refresh_report',
            sessionID: FR.SessionMgr.getSessionID()
        }, true);
        var data = {};
        if (WT.isNumber(sheetIndex)) {
            data.reportIndex = sheetIndex;
        }
        return WT.ajax({
            url: url,
            type: 'POST',
            data: data
        }).then(function (result) {
            var r = WT.dealWithHtmlResult(result);
            return FR.jsonDecode(r.data);
        });
    },

    /**
     * 清除暂存
     * @return {Promise.<TResult>}
     */
    clearStashed: function () {
        var url = FR.buildServletUrl({
            op: 'fr_write',
            cmd: 'clearstash',
            sessionID: FR.SessionMgr.getSessionID()
        }, true);
        return WT.ajax({
            url: url,
            type: 'POST',
            data: {}
        }).then(function (result) {
            var r = WT.dealWithHtmlResult(result);
            return r.data;
        });
    },

    /**
     * excel导入
     * @param $form 表单
     * @return {*|Promise.<TResult>}
     */
    importExcel: function ($form, type) {
        var url = FR.buildServletUrl({
            op: 'fr_write',
            cmd: 'imp_w_excel_data',
            sessionID: FR.SessionMgr.getSessionID(),
            type: type
        }, true);
        return WT.ajaxSubmit($form, {
            url: url,
            type: 'POST'
        }).then(function (result) {
            var r = WT.dealWithHtmlResult(result);
            return r.data;
        });
    },
    customizeImportExcel: function ($form) {
        var url = FR.buildServletUrl({
            op: 'fr_write',
            cmd: 'c_cache_excel',
            sessionID: FR.SessionMgr.getSessionID()
        }, true);
        return WT.ajaxSubmit($form, {
            url: url,
            type: 'POST'
        });
    },

    /**
     * 保存内容（暂存时会用）
     * @param cellList 校验数据列表
     *                  {
     *                      sheetIndex: 0,
     *                      cells: [{
     *                          column: 1,
     *                          row: 2,
     *                          value: 'value'
     *                      }]
     *                  }
     * @return {Promise.<TResult>}
     */
    saveContent: function (cellList) {
        var url = FR.buildServletUrl({
            op: 'fr_write',
            cmd: 'save_w_content',
            sessionID: FR.SessionMgr.getSessionID()
        }, true);
        var data = {
            reportXML: WT.generateReportData(cellList)
        };
        return WT.ajax({
            url: url,
            type: 'POST',
            data: data
        }).then(function (result) {
            var r = WT.dealWithHtmlResult(result);
            return r.data;
        });
    },

    /**
     * 暂存
     * @return {Promise.<TResult>}
     */
    stash: function () {
        var url = FR.buildServletUrl({
            op: 'fr_write',
            cmd: 'stash',
            sessionID: FR.SessionMgr.getSessionID()
        }, true);
        return WT.ajax({
            url: url,
            type: 'POST',
            data: {}
        }).then(function (result) {
            var r = WT.dealWithHtmlResult(result);
            return r.data;
        });
    },

    /**
     * 导出pdf
     * @param extype
     */
    exportPDF: function (extype) {
        var url = FR.buildServletUrl({
            op: 'export',
            format: 'pdf',
            'fr_embedded': true,
            extype: extype,
            sessionID: FR.SessionMgr.getSessionID()
        }, true);
        WT.net.downloadExportFile(url, 'pdf');
    },

    /**
     * 导出excel
     * @param extype
     * @param sheets
     */
    exportExcel: function (extype, sheets) {
        var query = {
            op: 'export',
            format: 'excel',
            sessionID: FR.SessionMgr.getSessionID()
        };
        var sheetStr = "";
        if (sheets != null && $.isArray(sheets)) {
            sheetStr += "[";
            for (var i = 0; i < sheets.length; i++) {
                sheetStr += sheets[i] + "";
                if (i < sheets.length - 1) {
                    sheetStr += ",";
                }
            }
            sheetStr += "]";
        }
        if (sheetStr) {
            query.sheets = sheetStr;
        }
        var flag = extype.indexOf("_");
        if (flag != -1) {
            var extypeNew = extype.substring(0, flag);
            query.extype = extypeNew;
            query.isExcel2003 = true;
        } else {
            query.extype = extype;
        }
        var url = FR.buildServletUrl(query);
        WT.net.downloadExportFile(url, 'excel');
    },

    directExportToExcel: function (dsName, fileName, params, colNames) {
        var query = {
            op: 'export',
            cmd: 'direct_excel',
            sessionID: FR.SessionMgr.getSessionID(),
            dsName: dsName,
            fileName: fileName === undefined ? "" : fileName,
            params: params === undefined ? "" : params,
            colNames: colNames === undefined ? "" : colNames
        };

        var url = FR.buildServletUrl(query);
        WT.net.downloadExportFile(url, 'excel');
    },


    /**
     * 导出word
     */
    exportWORD: function () {
        var url = FR.buildServletUrl({
            op: 'export',
            format: 'word',
            sessionID: FR.SessionMgr.getSessionID()
        }, true);
        WT.net.downloadExportFile(url, 'word');
    },

    /**
     * 导出图片
     */
    exportImage: function (extype) {
        var url = FR.buildServletUrl({
            op: 'export',
            format: 'image',
            extype: extype,
            sessionID: FR.SessionMgr.getSessionID()
        }, true);
        window.location = url;
    },

    /**
     * 根据控件名请求控件，同步方式
     * @param widgetName
     * @param sheetIndex
     * @return {*}
     * @throws err
     */
    getWidgetsByNameSync: function (widgetName, sheetIndex) {
        var url = FR.buildServletUrl({
            op: 'fr_write_plus',
            cmd: 'widgets',
            sessionID: FR.SessionMgr.getSessionID()
        }, true);
        var result = WT.ajaxSync({
            url: url,
            type: 'POST',
            data: {
                widgetName: widgetName,
                sheetIndex: sheetIndex
            }
        });
        if (result.err) {
            throw result.err;
        }
        var r = WT.dealWithHtmlResult(result);
        return FR.jsonDecode(r.data);
    },

    /**
     * 根据控件所在单元格位置请求控件，同步方式
     * @param location {{column: number, row:  number, sheetIndex: number}}
     * @return {*}
     * @throws err
     */
    getWidgetByLocationSync: function (location) {
        var url = FR.buildServletUrl({
            op: 'fr_write_plus',
            cmd: 'widgets',
            sessionID: FR.SessionMgr.getSessionID()
        }, true);
        var result = WT.ajaxSync({
            url: url,
            type: 'POST',
            data: {
                cr: location.column + ',' + location.row,
                sheetIndex: location.sheetIndex
            }
        });
        if (result.err) {
            throw result.err;
        }
        var r = WT.dealWithHtmlResult(result);
        return FR.jsonDecode(r.data);
    },

    /**
     * 根据控件所在单元格位置请求控件，批量
     * @param crs Array<{{column: number, row:  number}}>
     * @param sheetIndex {number}
     * @return {*}
     * @throws err
     */
    getWidgetsByLocation: function (crs, sheetIndex) {
        var url = FR.buildServletUrl({
            op: 'fr_write_plus',
            cmd: 'widgets',
            sessionID: FR.SessionMgr.getSessionID()
        }, true);
        var cr = WT.reduce(crs, function (prev, next) {
            if (!prev) {
                return next.column + ',' + next.row;
            } else {
                return prev + ';' + next.column + ',' + next.row;
            }
        }, null);
        return WT.ajax({
            url: url,
            type: 'POST',
            data: {
                cr: cr,
                sheetIndex: sheetIndex
            }
        }).then(function (result) {
            var r = WT.dealWithHtmlResult(result);
            return FR.jsonDecode(r.data);
        });
    },

    /**
     * 导出文件
     * @param url
     * @param type
     */
    downloadExportFile: function (url, type) {
        //新导出方式插件
        if (FR.Report.Plugin.XHR2FileSaveProcessor.item && FR.Plugin.validLevel(FR.Report.Plugin.XHR2FileSaveProcessor, FR.Report.Plugin.XHR2FileSaveProcessor.item)) {
            FR.Report.Plugin.XHR2FileSaveProcessor.item.action.call(this, url, type);
            return;
        }
        FR.openUrlByForm(url);
        WT.showProgressBar(FR.SessionMgr.getSessionID(), type);
    },

    /**
     * 远程异步执行公式，返回Promise对象
     * @param formula
     * @return {*}
     */
    remoteEvaluate: function (formula, location) {
        return WT.ajax({
            url: FR.servletURL,
            type: 'POST',
            data: {
                op: 'fr_write_plus',
                cmd: 'evaluate_formula',
                sessionID: FR.SessionMgr.getSessionID(),
                expression: formula,
                location: location
            },
            timeout: 5000
        }).then(function (res) {
            var result = FR.jsonDecode(res.data);
            return result["result"];
        });
    },

    remoteEvaluateSync: function (formula, location) {
        var result = null;

        FR.ajax({
            url: FR.servletURL,
            type: 'POST',
            async: false,
            data: {
                op: 'fr_write_plus',
                cmd: 'evaluate_formula',
                sessionID: FR.SessionMgr.getSessionID(),
                expression: formula,
                location: location
            },
            timeout: 5000,
            complete: function (res, status) {
                result = FR.jsonDecode(res.responseText);
                result = result["result"];
            }
        });

        return result;
    },

    /**
     * 客户端计算公式，返回Promise对象
     * @static
     * @param {String} formula 要计算的公式
     * @param {Object} initValue 初始值
     * @param {Boolean} must  是否必须总是计算
     * @returns {Object} 计算后的值
     */
    formulaEvaluator: function (formula, initValue, must) {
        var lazyValue = initValue;
        return function (nocache) {
            return WT.createPromise(function (resolve) {
                //如果后台没有计算出公式结果initValue, 那再去remoteEval
                //这边之前写!lazyValue , 遇到lazyValue = 0时就会重复计算
                if (nocache || lazyValue == undefined || must) {
                    if (FR.SessionMgr.getSessionID()) {// kunsnat: BI在sessionID为空时, 不需要此解析公式.
                        return WT.net.remoteEvaluate(formula)['catch'](function () {
                            //说明不需要到服务器端取数
                            return lazyValue
                        });
                    }
                }
                return lazyValue;
            })
        }
    }
});