/**
 * 报表单元格
 * @extends FR.OB
 * 
 * @cfg {JSON} options 配置属性
 * @cfg {Boolean} options.isShowWidgets 直接显示控件
 * @cfg {Boolean} options.isEditor 为编辑器类控件
 * @cfg {Number} options.sheetIndex 报表sheet序号
 * @cfg {Object} options.widgetConfig 单元格所包含的控件配置
 * @cfg {Number} options.left cell在block中的左边距
 * @cfg {Number} options.top cell在block中的上边距
 * @cfg {Number} options.width cell宽度
 * @cfg {Number} options.height cell高度
 * @cfg {JSON} options.border 边框
 * @cfg {Number} options.border.left 左边框类型
 * @cfg {String} options.border.leftColor 左边框颜色
 * @cfg {Number} options.border.right 同上
 * @cfg {String} options.border.rightColor 同上
 * @cfg {Number} options.border.top 同上
 * @cfg {String} options.border.topColor 同上
 * @cfg {Number} options.border.bottom 同上
 * @cfg {String} options.border.bottomColor 同上
 * @cfg {String} options.tooltip 单元格tooltip
 * @cfg {String} options.value 单元格原始值
 * @cfg {JSON} options.display 单元格显示值信息
 * @cfg {Number} options.column 当前sheet中的列号 0开始
 * @cfg {Number} options.row 当前sheet中的行号 0开始
 */
WT.Cell = FR.extend(FR.OB, {

    _const: {
        selectBorderWidth: 2,
        editIconSize: 16
    },

	_defaultConfig: function () {
        return $.extend(WT.Cell.superclass._defaultConfig.apply(), {
            isEditor: false,
            isShowWidgets: false,
            sheetIndex: 0,
			widgetConfig: null,
            left: 0,
            top: 0,
            width: 0,
            height: 0,
            border: {
                left: 0,
                leftColor: '#000',
                right: 0,
                rightColor: '#000',
                top: 0,
                topColor: '#000',
                bottom: 0,
                bottomColor: '#000'
            },
            tooltip: '',
            value: '',
            display: null,
            column: 0,
            row: 0,
            background: '#fff',
            paddingLeft: 0,
            paddingRight: 0,
            paraProvider: null,
            sheetConfigProvider: null,
            sheetContext: null
		});
    },

	_init: function () {
		WT.Cell.superclass._init.apply(this, arguments);
		if (this.options.display && !this.options.display.color) {
            this.options.display.color = 'black';
        }
		this.selected = false;
		this.isIgnoreValueChangeEvent = false;
		this._lastWidgetValue = null;
		this._initView();
	},

	/**
	 * 初始化dom以及子控件dom
	 */
	_initView: function () {
	    var o = this.options, self = this;
        this.element = WT.dom.createJQueryElement('div', 'wt-absolute-layout wt-border-box wt-cell');
        this.element.css({
            left: o.left,
            top: o.top,
            width: o.width,
            height: o.height
        });
        this.element.attr('cellid', o.id);
        WT.setBackground(this.element, o.background);
        if (o.display && o.display.fontFamily) {
            this.element.css('font-family', o.display.fontFamily);
        }

        var borderLeft = WT.transformBorderType(o.border.left, o.border.leftColor);
        var borderRight = WT.transformBorderType(o.border.right, o.border.rightColor);
        var borderTop = WT.transformBorderType(o.border.top, o.border.topColor);
        var borderBottom = WT.transformBorderType(o.border.bottom, o.border.bottomColor);
        this.element.css({
            'border-left': borderLeft.style,
            'border-right': borderRight.style,
            'border-top': borderTop.style,
            'border-bottom': borderBottom.style
        });

        this.element.attr('title', o.tooltip);

        this.contentWidth = o.width - borderLeft.width - borderRight.width;
        this.contentHeight = o.height - borderTop.width - borderBottom.width;

        this._initContent();
	},

    /**
     * 初始化内容
     * @param left
     * @param top
     * @param width
     * @param height
     * @private
     */
    _initContent: function () {
        var o = this.options;
        if (o.widgetConfig && o.widgetConfig.xtype === 'WT.PasswordEditor') {
            this.isPassword = true;
        }
        if (o.isShowWidgets && o.widgetConfig && !o.hyperlinkGroup) {
            this._initWidget()
        }
        else {
            this._initBaseWidget();
        }
    },

    _initBaseWidget: function () {
        var o = this.options;
        var left = o.paddingLeft,
            top = 0,
            width = this.contentWidth - o.paddingLeft - o.paddingRight,
            height = this.contentHeight;
        var valueObj = this.options.display;
        if (!valueObj.displayType) {
            return;
        }
        if (valueObj.displayType === 'chart') {
            var el = WT.dom.createElement('div');
            WT.dom.appendChild(this.element, el);
            this.chart = FR.createWidget($.extend({
                renderEl: $(el)
            }, valueObj.config));
            return;
        }
        var textOpt = {
            left: left,
            top: top,
            width: width,
            height: height,
            hyperlinkGroup: o.hyperlinkGroup,
            value: o.value,
            display: o.display,
            widgetType: o.widgetConfig && o.widgetConfig.xtype
        };
        if (o.widgetConfig && o.widgetConfig.config && o.widgetConfig.config.watermark) {
            textOpt.watermark = o.widgetConfig.config.watermark;
        }
        this.baseWidget = new WT.CellLabel(textOpt);
        this.baseWidget.element.appendTo(this.element);
    },

    _initWidget: function () {
        this._applySelectStyle(false);
        var o = this.options;
        var widgetConfig = o.widgetConfig;
        if (!widgetConfig) {
            return;
        }
        var xtype = widgetConfig.xtype;
        var widget = this.widget;
        if (!widget) {
            widget = this._createWidget();
            if (!widget) {
                return;
            }
            this.widget = widget;
            WT.dom.appendChild(this.element, this.widget.element);
        } else {
            widget._detached = false;
            WT.dom.appendChild(this.element, this.widget.element);
            if (widget.setValue) {
                widget.setValue(this._convertValue4Widget(this.originValue || o.value, xtype));
                this._updateLastValue();
            }
        }
        this._partCssCopy(this.element, this.widget);
    },
    /**
     * 拷贝src部分样式到target
     * @param src
     * @param target
     * @private
     */
    _partCssCopy: function (src, target) {
        if (!target.editor) {
            return;
        }
        var o = this.options;
        var $targetEl = $(target.editor.element);
        if (!$targetEl) {
            return;
        }
        var $src = $(src);
        $.each(['fontFamily', 'fontVariant', 'fontStyle', 'fontWeight', 'backgroundColor'], function (idx, el) {
            $targetEl.css(el, $src.css(el));
        });
        if ((/transparent|rgba\(0, 0, 0, 0\)/).test($src.css('backgroundColor'))) {
            $targetEl.css('backgroundColor', 'white')
        }
        $targetEl.css('fontSize', o.display.fontSize);
        if (o.isShowWidgets && o.display.color) {
            $targetEl.css('color', o.display.color);
        }
    },
    _createWidget: function () {
        var o = this.options;
        var left = 0,
            top = 0,
            width = this.contentWidth,
            height = this.contentHeight;
        var widgetConfig = o.widgetConfig;
        var xtype = widgetConfig.xtype;
        if (!xtype) {
            return;
        }
        var cfg = $.extend(widgetConfig.config, {
            sheetContext: o.sheetContext,
            listeners: widgetConfig.events,
            left: left,
            top: top,
            width: width,
            height: height,
            location: FR.columnRow2CellStr({
                col: o.column,
                row: o.row
            }),
            reportIndex: o.sheetIndex,
            paraProvider: o.paraProvider,
            globalConfigProvider: function () {
                return window.report.globalConfig;
            },
            value: this._convertValue4Widget(this.originValue || o.value, xtype),
            format: this._convertDisplayFormat4Widget(o.display.format, xtype)
        });
        if (WT.isDataWidget(widgetConfig)) {
            cfg = $.extend({
                dataSource: new WT.DataSource({
                    dataLoader: function (oldData, args) {
                        return WT.net.getCellWidgetData({
                            sheetIndex: o.sheetIndex,
                            column: o.column,
                            row: o.row
                        }, args);
                    },
                    dataLoaderSync: function (oldData, args) {
                        return WT.net.getCellWidgetDataSync({
                            sheetIndex: o.sheetIndex,
                            column: o.column,
                            row: o.row
                        }, args);
                    }
                })
            }, widgetConfig.config);
        }
        if (WT.isRadioOrCheckBox(widgetConfig)) {
            cfg.fontSize = o.display.fontSize;
        }
        cfg.usedAsCell = true;
        var widget = WT.createWidget(xtype, cfg);
        var self = this;
        this._updateLastValue();
        var valueChanged = function () {
            self._widgetValueChange();
        };
        widget.on(FR.Events.STOPEDIT, valueChanged);
        widget.on(FR.Events.STATECHANGE, valueChanged);
        widget.on(FR.Events.CALLBACK, valueChanged);
        widget.on(WT.BasePopup.EVENT_INSIDE_STOP_EDIT, valueChanged);
        if (xtype === 'WT.TreeView') {
            widget.on(FR.Events.CLICK, valueChanged);
        }
        widget.on(WT.BaseEditor.EVENT_ENTER_PRESSED, function () {
            self.fireEvent(WT.Cell.EVENT_ENTER_PRESSED);
        });
        widget.on(WT.BaseEditor.EVENT_TAB_PRESSED, function () {
            self.fireEvent(WT.Cell.EVENT_TAB_PRESSED);
        });
        widget.on(FR.Events.BEFOREEDIT, function () {
            self.select('click');
            self._applySelectStyle(false);
            self._applyEditStyle(false);
        });
        return widget;
    },

    _widgetValueChange: function () {
        var widget = this.widget, o = this.options, self = this;
        var thisValue = widget.getOriginValue ? widget.getOriginValue() : widget.getValue();
        if (WT.isArray(thisValue) || WT.isObject(thisValue)) {
            thisValue = JSON.stringify(thisValue);
        }
        if (thisValue !== self._lastWidgetValue) {
            self._lastWidgetValue = thisValue;
            o.store && o.store.setReportModified(true);
            if (self.isIgnoreValueChangeEvent) {
                return;
            }
            self._handleWidgetValueChanged(widget.getValue(), widget.getOriginValue && widget.getOriginValue());
        }
    },

    /**
     * 根据控件类型转换值类型
     * @param value
     * @param xtype
     * @private
     */
    _convertValue4Widget: function (value, xtype) {
        if (value === null || value === undefined) {
            return value;
        }
        var o = this.options;
        var fmt = o.display.format;
        if (value instanceof Date) {
            if (xtype !== 'WT.DateEditor') {
                return FR.contentFormat(value.getTime(), fmt || 'DMM/dd/yyyy')
            }
        }
        if (value.date_milliseconds) {
            if (xtype !== 'WT.DateEditor') {
                return FR.contentFormat(value.date_milliseconds, fmt || 'DMM/dd/yyyy');
            }
        }
        if (xtype === 'WT.Button') {
            if (o.display && o.display.value !== undefined && o.display.value !== null) {
                return o.display.value;
            }
        }
        return value;
    },

    /**
     * 根据控件类型转换格式类型
     * @param fmt
     * @param xtype
     * @private
     */
    _convertDisplayFormat4Widget: function (fmt, xtype) {
        if (xtype === 'WT.DateEditor') {
            // 日期控件格式必需是日期格式，不接受额外设置的显示格式，只使用设计器中设置的格式
            // 针对excel导入日期格式的单元格时会丢失原来的日期格式问题
            return undefined;
        }
        return fmt;
    },

    /**
     * 把控件中的值取出来设置到单元格中
     * @private
     */
    _syncCellValue: function () {
        if (this.widget) {
            this.widget.stopEdit && this.widget.stopEdit(true);
            if(this.options.value !== this.widget.getValue() && this.options.isShowWidgets){
                this.options.value = this.widget.getValue();
            }
            if (this.widget.getOriginValue) {
                this.originValue = this.widget.getOriginValue();
            }
            this.widget.blur && this.widget.blur();
            this._applyValue2Label(true);
        }
    },

    _detachWidget: function () {
        if (this.widget) {
            var self = this;
            this.widget._detached = true;
            setTimeout(function () {
                self.widget && self.widget.detach && self.widget.detach();
            }, 20);
        }
    },

    _destroyWidget: function () {
        if (this.widget) {
            this._syncCellValue();
            this._detachWidget();
            this.widget.purgeListeners();
            this.widget.destroy();
            this.widget = null;
        }
    },

    _showWidget: function (noAutoSelect) {
        // REPORT-14540 直接显示控件时如果有超链接就不显示控件
        if (this.options.isShowWidgets && this.options.hyperlinkGroup) {
            return;
        }
        if (this.widget && !this.widget._detached) {
            return;
        }
        this._applyEditStyle(false);
        this._initWidget();
        this.widget && this.widget.focus && this.widget.focus(noAutoSelect);
    },

    _handleClick: function () {
        if (!this.isSelected()) {
            this.select('click');
        }
        else if (this.hasEditorWidget()) {
            this._showWidget();
        }
        this.fireEvent(FR.Events.CLICK, this)
    },

    _applySelectStyle: function (select) {
        var o = this.options;
        if (o.sheetConfigProvider && !o.sheetConfigProvider().isShowBorder) {
            WT.dom.detachFromHTML(this.selectLeft);
            WT.dom.detachFromHTML(this.selectRight);
            WT.dom.detachFromHTML(this.selectTop);
            WT.dom.detachFromHTML(this.selectBottom);
            return;
        }
        if (select && !o.isShowWidgets) {
            // TODO 若以后支持动态调整单元格大小，此处需修改
            if (!this.selectLeft) {
                this.selectLeft = WT.dom.createJQueryElement('div', 'wt-absolute-layout wt-cell-select-border').css({
                    left: 0,
                    top: 0,
                    width: this._const.selectBorderWidth,
                    height: this.contentHeight
                });
            }
            WT.dom.appendChild(this.element, this.selectLeft);

            if (!this.selectRight) {
                this.selectRight = WT.dom.createJQueryElement('div', 'wt-absolute-layout wt-cell-select-border').css({
                    left: this.contentWidth - this._const.selectBorderWidth,
                    top: 0,
                    width: this._const.selectBorderWidth,
                    height: this.contentHeight
                });
            }
            WT.dom.appendChild(this.element, this.selectRight);

            if (!this.selectTop) {
                this.selectTop = WT.dom.createJQueryElement('div', 'wt-absolute-layout wt-cell-select-border').css({
                    left: this._const.selectBorderWidth,
                    top: 0,
                    width: this.contentWidth - this._const.selectBorderWidth * 2,
                    height: this._const.selectBorderWidth
                });
            }
            WT.dom.appendChild(this.element, this.selectTop);

            if (!this.selectBottom) {
                this.selectBottom = WT.dom.createJQueryElement('div', 'wt-absolute-layout wt-cell-select-border').css({
                    left: this._const.selectBorderWidth,
                    top: this.contentHeight - this._const.selectBorderWidth,
                    width: this.contentWidth - this._const.selectBorderWidth * 2,
                    height: this._const.selectBorderWidth
                });
            }
            WT.dom.appendChild(this.element, this.selectBottom);
        }
        else {
            WT.dom.detachFromHTML(this.selectLeft);
            WT.dom.detachFromHTML(this.selectRight);
            WT.dom.detachFromHTML(this.selectTop);
            WT.dom.detachFromHTML(this.selectBottom);
        }
    },

    _applyEditStyle: function (show) {
        var widgetConfig = this.options.widgetConfig;
        if (widgetConfig && widgetConfig.xtype === 'WT.MultifileEditor') {
            //文件控件不显示
            return;
        }
        if (show && !this.options.isShowWidgets) {
            // TODO 若以后支持动态调整单元格大小，此处需修改
            if (!this.editIcon) {
                this.editIcon = WT.dom.createJQueryElement('div', 'wt-absolute-layout wt-cell-glance-editor-img').css({
                    left: this.contentWidth - this._const.editIconSize,
                    top: 0,
                    width: this._const.editIconSize,
                    height: this._const.editIconSize
                });
                var self = this;
                this.editIcon.on('click', function () {
                    self._applyEditStyle(false);
                    self._handleClick();
                });
            }
            WT.dom.appendChild(this.element, this.editIcon);
        }
        else {
            WT.dom.detachFromHTML(this.editIcon);
        }
    },

    _applyDirtyStyle: function (dirty) {
        if (dirty) {
            if (!this.dirty) {
                this.dirty = WT.dom.createJQueryElement('div', 'wt-absolute-layout wt-cell-dirty')
                    .css({
                        left: 0,
                        top: 0
                    });
            }
            WT.dom.appendChild(this.element, this.dirty);
        }
        else {
            WT.dom.detachFromHTML(this.dirty);
        }
    },

    _applyInvalidStyle: function (invalidMsg) {
        if (this.baseWidget) {
            this.baseWidget.setInvalid(invalidMsg);
        }
        else {
            if (!WT.isNullOrUndefined(invalidMsg)) {
                if (!this.warn) {
                    this.warn = WT.dom.createJQueryElement('i', 'wt-absolute-layout wt-cell-invalid-icon').attr('title', invalidMsg);
                }
                WT.dom.appendChild(this.element, this.warn);
            }
            else {
                WT.dom.detachFromHTML(this.warn);
            }
        }
    },

    _applyPasteData: WT.debounce(function (text) {
        this.setValue(text);
        this._showWidget();
    }, 100),

    _handleWidgetValueChanged: function (value, originValue) {
        this.originValue = originValue;
        this.options.value = value;
        this._applyValue2Label();
        this._applyDirtyStyle(true);
        this._synchronizeValue();
    },

    _applyValue2Label: function (presented) {
        var o = this.options, self = this;
        var value = o.value;
        var valueObj = o.display;
        if (valueObj.present && presented !== true) {
            var presentDep = valueObj.presentDep;
            var width = this.contentWidth - o.paddingLeft - o.paddingRight,
                height = this.contentHeight;
            var p = WT.createPromiseResult({});
            if (presentDep && o.paraProvider) {
                p = p.then(function (dependence) {
                    return WT.promiseReduce(presentDep, function (dependence, idx) {
                        var k = presentDep[idx];
                        return o.paraProvider.getParaAsync(k).then(function (paraVal) {
                            if (paraVal) {
                                dependence[k.toUpperCase()] = paraVal;
                            }
                            return dependence;
                        });
                    }, dependence);
                });
            }
            p.then(function (dependence) {
                return WT.net.getCellDisplayValue(value, o.column, o.row, o.sheetIndex, dependence).then(function (result) {
                    var v = '';
                    if (result.attach_id && valueObj.displayType === 'image') {
                        var img = WT.dom.createJQueryElement('img').attr('src', WT.getImageAttachURL(result.attach_id));
                        img.css({
                            width: width,
                            height: height
                        });
                        v = img;
                    } else {
                        v = result;
                    }
                    if (self.baseWidget) {
                        self.baseWidget.setValue(v);
                    } else if (self.widget && self.options.isShowWidgets) {
                        self.widget.setEditorDisplayValue && self.widget.setEditorDisplayValue(v);
                    }
                });
            })['catch'](WT.emptyFn);
        } else if (this.baseWidget) {
            if (this.isPassword) {
                value = value + '';
                var star = '';
                for (var i = 0; i < value.length; i++) {
                    star += '*';
                }
                this.baseWidget.setValue(star);
            } else {
                this.baseWidget.setValue(value);
            }
        }
    },

    _synchronizeValue: function () {
        this.fireEvent(WT.Cell.EVENT_SYNC_VALUE);
    },

    _updateLastValue: function () {
        var widget = this.widget;
        if (widget) {
            this._lastWidgetValue = widget.getOriginValue ? widget.getOriginValue() : widget.getValue();
        }
    },

    handleClick: function () {
        this._handleClick();
    },

    handleMouseIn: function () {
        if (!this.selected && this.hasEditorWidget()) {
            this._applySelectStyle(true);
            this._applyEditStyle(true);
        }
    },

    handleMouseOut: function () {
        if (!this.selected) {
            this._applySelectStyle(false);
        }
        this._applyEditStyle(false);
    },

    /**
     * 校验单元格并同步单元格值
     * @param isForceSyncValue 强制同步，忽略校验失败情况
     * @return {*}
     */
    stopEdit: function (isForceSyncValue) {
        var widget = this.widget;
        if (widget && widget.stopEdit) {
            var result = widget.stopEdit(isForceSyncValue !== true);
            if (result || isForceSyncValue === true) {
                var thisValue = widget.getOriginValue ? widget.getOriginValue() : widget.getValue();
                if (WT.isArray(thisValue) || WT.isObject(thisValue)) {
                    thisValue = JSON.stringify(thisValue);
                }
                if (thisValue !== this._lastWidgetValue) {
                    this._handleWidgetValueChanged(widget.getValue(), widget.getOriginValue && widget.getOriginValue());
                    this._lastWidgetValue = thisValue;
                }
            }
            return result;
        }
        return true;
    },

    mount: function ($parent) {
        WT.dom.appendChild($parent, this.element);
    },

    umount: function () {
        this.element.remove();
    },

    getID: function () {
        return this.options.id;
    },

    /**
     * 设置单元格值
     * @param value
     * @param isTrig 为true时触发值编辑事件
     * @param originValue
     */
    setValue: function (value, isTrig, originValue) {
        this.options.value = value;
        this.originValue = originValue;
        this._applyValue2Label();
        if (this.widget && this.widget.setValue) {
            this.widget.setValue(this._convertValue4Widget(originValue || value, this.options.widgetConfig.xtype));
            this._updateLastValue();
        }
        this._applyDirtyStyle(true);
        if (isTrig === true) {
            this._synchronizeValue();
        }
    },

    setEditorValue: function (value) {
        if (this.hasEditorWidget() && this.widget) {
            this.widget.setValue(value);
            this._updateLastValue();
        }
    },

    getValue: function () {
        return this.options.value;
    },

    /**
     * 选中该单元格
     * @param type? {'click' | 'key' | 'edit'} 触发select的操作类型
     */
    select: function (type) {
        var lastSelect = this.selected;
        this.selected = true;
        if (type !== 'focus') {
            if (this.hasEditorWidget()) {
                if (!this.widget || this.widget._detached) {
                    var globalConfig = window.report.globalConfig;
                    if (type === 'click' && globalConfig.editOnClick) {
                        this._showWidget();
                    } else if (type === 'key' && globalConfig.editOnMove) {
                        this._showWidget();
                    } else if (type === 'edit') {
                        this._showWidget();
                    } else {
                        this._applySelectStyle(true);
                    }
                } else if (this.widget && this.widget.focus) {
                    if (!this.options.isShowWidgets) {
                        this.widget.focus();
                    }
                }
            }
        }
        if (lastSelect !== true) {
            this.fireEvent(WT.Cell.EVENT_CELL_SELECT)
        }
    },

    /**
     * 取消选中该单元格
     */
    unSelect: function () {
        var oldSelected = this.selected;
        this.selected = false;
        this._applySelectStyle(false);
        if (oldSelected) {
            if (this.widget && this.widget.TYPE === 'WT.TreeCombo') {
                this._widgetValueChange();
            } else {
                this._syncCellValue();
            }
            if (this.widget && this.widget._detached !== true && !this.options.isShowWidgets) {
                this._detachWidget();
            }
        }
    },

    /**
     * 返回单元格是否选中
     */
    isSelected: function () {
        return this.selected;
    },

    edit: function (noAutoSelect) {
        if (this.selected) {
            if (this.options.isShowWidgets) {
                this.widget && this.widget.focus && this.widget.focus();
            } else {
                this._showWidget(noAutoSelect);
            }
        } else {
            this.select('edit');
        }
    },

    /**
     * 获取单元格的位置信息
     * @return {{sheetIndex: number, column: number, row: number}}
     */
    getLocation: function () {
        var o = this.options;
        return {
            sheetIndex: o.sheetIndex,
            column: o.column,
            row: o.row
        };
    },

    /**
     * 获取单元格rect
     * @return {{left: number, top: number, width: number, height: number}}
     */
    getRect: function () {
        var o = this.options;
        return {
            left: o.left,
            top: o.top,
            width: o.width,
            height: o.height
        }
    },

    /**
     * 设置是否显示单元格校验失败
     * @param invalidMsg {string | null} 为字符串时表示有错误，字符串为错误信息；为null时无错误
     */
    setInvalid: function (invalidMsg) {
        this._applyInvalidStyle(invalidMsg);
    },

    getWidgetName: function () {
        var widgetConfig = this.options.widgetConfig;
        return widgetConfig && widgetConfig.config && widgetConfig.config.widgetName;
    },

    getWidget: function () {
        if (!this.widget && this.hasWidget()) {
            this.widget = this._createWidget();
            this.widget._detached = true;
        }
        return this.widget;
    },

    /**
     * 返回单元格是否有控件
     * @return {boolean}
     */
    hasWidget: function () {
        return !!this.options.widgetConfig;
    },

    /**
     * 返回单元格是否有编辑类控件
     * @return {boolean}
     */
    hasEditorWidget: function () {
        return this.options.isEditor && this.options.widgetConfig;
    },

    getRawHtmlStrValue: function () {
        if (this.baseWidget) {
            return this.baseWidget.getRawHtmlStrValue();
        } else {
            return '';
        }
    },

    setIgnoreValueChangeEvent: function (isIgnore) {
        this.isIgnoreValueChangeEvent = !!isIgnore;
    },

    disableBorder: function () {
        this._applySelectStyle(false);
    },

    updateHyperlink: function (hy) {
        this.baseWidget && this.baseWidget.updateHyperlink(hy);
    },

	destroy: function () {
        this.editIcon && this.editIcon.off('click');
	    this.umount();
	    this.purgeListeners();
	    this._destroyWidget();
        this.baseWidget && this.baseWidget.destroy();
        this.baseWidget = null;
        this.element.off('click');
        this.element.off('mouseenter');
        this.element.off('mouseleave');
        this.element.empty();
        this.element = null;
    }

});
WT.Cell.EVENT_CELL_SELECT = 'cellselect';
WT.Cell.EVENT_SYNC_VALUE = 'EVENT_SYNC_VALUE';
WT.Cell.EVENT_ENTER_PRESSED = 'EVENT_ENTER_PRESSED';
WT.Cell.EVENT_TAB_PRESSED = 'EVENT_TAB_PRESSED';

WT.Cell.transformConfig = function (c) {
    var cfg = {
        id: c.id,
        left: c.l,
        top: c.t,
        width: c.w,
        height: c.h,
        border: {
            left: c.b.l,
            right: c.b.r,
            top: c.b.t,
            bottom: c.b.b,
            leftColor: c.b.lc || 'black',
            rightColor: c.b.rc || 'black',
            topColor: c.b.tc || 'black',
            bottomColor: c.b.bc || 'black'
        },
        value: c.v,
        tooltip: c.tp,
        column: c.c,
        row: c.r,
        background: c.bg,
        paddingLeft: c.pl,
        paddingRight: c.pr,
        modified: c.md,
        display: c.dp
    };
    if (c.isw) {
        cfg.isShowWidgets = c.isw;
    }
    if (c.wc) {
        cfg.widgetConfig = c.wc;
    }
    if (c.hl) {
        cfg.hyperlinkGroup = c.hl;
    }
    if (c.edt) {
        cfg.isEditor = c.edt;
    }
    return cfg;
};