/**
 * 参数面板容器
 * @extends FR.OB
 *
 * @cfg {JSON} options 配置属性
 * @cfg {Array} options.widgets 子控件属性列表
 */
WT.ParameterPanel = FR.extend(FR.OB, {

    _const: {
        expanderHeight: 8,
        expanderWidth: 120
    },

    _defaultConfig: function () {
        return $.extend(WT.ParameterPanel.superclass._defaultConfig.apply(), {
            background: null,
            widgets: [],
            isExpanded: true
        });
    },

    _init: function () {
        WT.ParameterPanel.superclass._init.apply(this, arguments);
        this._initView();
    },

    /**
     * 初始化dom以及子控件dom
     */
    _initView: function () {
        var o = this.options, self = this;
        this.element = $('<div class="wt-absolute-layout wt-para-panel"></div>');
        o.background && WT.setBackground(this.element, o.background);
        // 组件容器
        this.widgetContainer = $('<div class="wt-absolute-layout wt-para-widget-container"></div>');
        this.element.append(this.widgetContainer);

        this.namedWidgets = {};
        this.items = [];
        this.editingWidget = null;
        WT.each(o.widgets, function (widget) {
            var xtype = widget.xtype;
            if (!xtype) {
                return;
            }
            var events = widget.events;
            var cfg = $.extend(widget.config, {
                form: self,
                listeners: events,
                delayInit: true,
                paraProvider: WT.createParaProvider(function (key) {
                    var w = self.namedWidgets[key];
                    var type = w && w.getType && w.getType();
                    if (type === WT.ComboBox.prototype.TYPE || type === WT.ComboCheckBox.prototype.TYPE){
                        return w.getOriginValue && w.getOriginValue();
                    }
                    return w && w.getValue && w.getValue();
                })
            });
            if (WT.isDataWidget(widget)) {
                cfg = $.extend({
                    dataSource: new WT.DataSource({
                        dataLoader: function (oldData, args) {
                            return WT.net.getWidgetData(widget.config.widgetName, args);
                        },
                        dataLoaderSync: function (oldData, args) {
                            return WT.net.getWidgetDataSync(widget.config.widgetName, args);
                        }
                    })
                }, widget.config);
            }
            var item = WT.createWidget(xtype, cfg);
            var formValue = o.formData[widget.config.widgetName];
            var _setValue = function () {
                if (WT.isObject(formValue)) {
                    if (formValue.data && item.setDataArray) {
                        item.setDataArray(formValue.data);
                    }
                    if (formValue.value) {
                        item.setValue(formValue.value);
                    }
                }
                else {
                    item.setValue(formValue);
                }
            };
            if (formValue) {
                if (WT.isTreeWidget(widget)) {
                    WT.nextTick(_setValue);
                } else {
                    _setValue();
                }
            }
            self.widgetContainer.append(item.element);
            self.namedWidgets[item.getName()] = item;
            self.items.push(item);
        });

        WT.each(this.items, function (item) {
            var valueDependence = self._getDependence(item.options);
            if (valueDependence && WT.isArray(valueDependence)) {
                WT.each(valueDependence, function (d) {
                    var dependenceWidget = self._getWidgetByName(d);
                    if (dependenceWidget && dependenceWidget !== item) {
                        //获取到了本控件依赖的其他控件
                        if (!dependenceWidget.linkWidget) {
                            dependenceWidget.linkWidget = {};
                        }
                        if (FR.isEmptyObj(dependenceWidget.linkWidget)) {
                            //性能优化,依赖同一个控件,只绑定一次事件
                            dependenceWidget.on(FR.Events.STOPEDIT, function () {
                                self._handleWidgetValueChanged(dependenceWidget);
                            });
                            dependenceWidget.on(FR.Events.STATECHANGE, function () {
                                self._handleWidgetValueChanged(dependenceWidget);
                            });
                            dependenceWidget.on(FR.Events.CLICK, function () {
                                self._handleWidgetValueChanged(dependenceWidget);
                            });
                        }
                        dependenceWidget.linkWidget[item.getName()] = item;
                    }
                });
            }
            item.on(FR.Events.BEFOREEDIT, function () {
                if (self.editingWidget && self.editingWidget !== item) {
                    self.editingWidget.stopEdit(true);
                }
                self.editingWidget = item;
            })
        });

        this.originFormData = this.getParameters();

        this.expanderWrapper = $('<div class="wt-absolute-layout"></div>');
        this.expander = $('<div class="wt-absolute-layout expander"></div>');
        this.expander.on('click', function () {
            self._handleExpanderClick();
        });
        this.expanderWrapper.append(this.expander);
        this.element.append(this.expanderWrapper);
        this._setExpanderBg();

        this.element.on('click', function (e) {
            // 仅当点击自身时才触发click，点击面板中的控件时不触发
            if (e.target === e.currentTarget) {
                self.fireEvent(FR.Events.CLICK);
            }
        });

        $('body').on('click', function (e) {
            if (self.editingWidget) {
                if (!WT.isElementContains(self.editingWidget.element, e.target)) {
                    self.editingWidget.stopEdit(true);
                    self.editingWidget = null;
                }
            }
        })
    },

    _confirmEvents: function () {
        WT.nextTick(function () {
            this.fireEvent(FR.Events.AFTERINIT);
        }, this);
    },

    _getDependence: function (opt) {
        var dep = [];
        if (opt.widgetExeDependence && WT.isArray(opt.widgetExeDependence)) {
            WT.each(opt.widgetExeDependence, function (v) {
                dep.push(v);
            })
        }
        if (opt.dependence) {
            for (var k in opt.dependence) {
                var n = WT.isIntString(k) ? opt.dependence[k] : k;
                dep.push(n);
            }
        }
        return dep;
    },

    _handleExpanderClick: function () {
        this.setExpand(!this.options.isExpanded);
    },

    _setExpanderBg: function () {
        var o = this.options;
        if (this.options.isExpanded) {
            this.expander.removeClass('collapse-icon').addClass('expand-icon');
            if (o.background) {
                this.expanderWrapper.css('background', o.background);
            }
        } else {
            this.expander.removeClass('expand-icon').addClass('collapse-icon');
            this.expanderWrapper.css('background', 'transparent');
        }
    },

    /**
     * 获取表单控件中指定名字的控件
     * @param {String} name 要获取的控件的名字
     * @returns {FR.Widget} 指定名字的控件,如果没有该名字的控件则返回null
     */
    _getWidgetByName: function (name) {
        if (!name) {
            return null;
        }
        if (name.startWith("$")) {
            name = name.substring(1);
        }
        return this.namedWidgets[name.toUpperCase()];
    },

    /**
     * 控件联动触发事件
     * @param {FR.Widget} widget 控件对象
     */
    _handleWidgetValueChanged: function (widget) {
        var self = this;
        var widgetList = {};
        this._findLinkedWidget(widgetList, widget.linkWidget);
        if (FR.isEmpty(widgetList)) {
            return;
        }
        var widgetNameList = WT.keys(widgetList);
        var para = $.extend({}, this.originFormData);
        if (WT.isTreeWidget(widget)) {
            para[widget.getName()] = widget.getValue();
        } else {
            para[widget.getName()] = (widget.getOriginValue && widget.getOriginValue()) || widget.getValue();
        }
        WT.each(widgetNameList, function (name) {
            if (para[name]) {
                delete para[name];
            }
        });
        this.fireEvent(WT.ParameterPanel.EVENT_CALCULATING_WIDGET, true);
        WT.net.synchronizeFormData(para, widgetNameList).then(function (data) {
            self.setParameters(data);
            self.fireEvent(WT.ParameterPanel.EVENT_CALCULATING_WIDGET, false);
        })['catch'](function () {
            self.fireEvent(WT.ParameterPanel.EVENT_CALCULATING_WIDGET, false);
        });
    },

    _findLinkedWidget: function (widgetList, linkWidget) {
        if (!linkWidget) {
            return;
        }
        for (var w in linkWidget) {
            var wg = linkWidget[w];
            widgetList[w] = wg;
            this._findLinkedWidget(widgetList, wg.linkWidget);
        }
    },

    _setWidgetValue: function (widgetName, value) {
        var widget = this.namedWidgets[widgetName];
        if (widget) {
            if (WT.isObject(value)) {
                if (value.data && widget.setDataArray) {
                    widget.setDataArray(value.data);
                }
                if (value.value !== undefined) {
                    widget.setValue(value.value);
                }
            }
            else {
                widget.setValue(value);
            }
        }
    },

    _getWidgetsByNames: function (names) {
        var widgets = [];
        var self = this;
        WT.each(names, function (name) {
            var w = self._getWidgetByName(name);
            if (w) {
                widgets.push(w);
            }
        });
        return widgets;
    },

    /**
     * 设置参数面板中控件的值
     * @param obj
     */
    setParameters: function (obj) {
        if (!WT.isObject(obj)) {
            return;
        }
        for (var w in obj) {
            this._setWidgetValue(w, obj[w]);
        }
    },

    /**
     * 获取面板中的参数
     */
    getParameters: function () {
        var para = {};
        this.items && WT.each(this.items, function (widget) {
            if (widget.options.needSubmit === true) {
                var val;
                if (widget.TYPE === '') {
                    val = widget.getQuickEditValue();
                } else if (WT.isTreeWidget(widget)) {
                    val = widget.getValue();
                } else {
                    val = WT.isFunction(widget.getOriginValue) ? widget.getOriginValue() : widget.getValue();
                }
                para[widget.getName()] = val;
            }
        });
        return para;
    },

    /**
     * 设置大小
     * @param x {Number} 左边距
     * @param y {Number} 上边距
     * @param width {Number} 宽度
     * @param height {Number} 高度
     */
    resize: function (x, y, width, height) {
        this.element.css({
            left: x,
            top: y,
            width: width,
            height: height
        });
        this.expanderWrapper.css({
            left: 0,
            top: height - this._const.expanderHeight,
            width: width,
            height: this._const.expanderHeight
        });
        this.expander.css({
            left: (width - this._const.expanderWidth) / 2,
            top: 0,
            width: this._const.expanderWidth,
            height: this._const.expanderHeight
        });
    },

    /**
     * 获取预定义的高度，给父组件布局使用
     */
    getHeight: function () {
        return this.options.isExpanded ? this.options.height : this._const.expanderHeight;
    },

    getRootElement: function () {
        return this.element;
    },

    setExpand: function (expand) {
        expand = !!expand;
        if (this.options.isExpanded !== expand) {
            this.options.isExpanded = expand;
            this._setExpanderBg();
            if (expand) {
                this.widgetContainer.css("display", "block");
                this.fireEvent(WT.ParameterPanel.EVENT_EXPANDED);
            }
            else {
                this.widgetContainer.css("display", "none");
                this.fireEvent(WT.ParameterPanel.EVENT_COLLAPSED);
            }
        }
    },

    isExpanded: function () {
        return this.options.isExpanded;
    },

    verify: function () {
        if (this.items) {
            for (var i = 0; i < this.items.length; i++) {
                var item = this.items[i];
                if (item.verify && item.options.needSubmit === true) {
                    if (!item.verify()) {
                        FR.Msg.toast(item.errMsg);
                        return false;
                    }
                }
            }
        }
        return true;
    },

    invisible: function (widgetNames) {
        WT.each(this._getWidgetsByNames(widgetNames), function (w) {
            w.invisible();
        });
    },

    visible: function (widgetNames) {
        WT.each(this._getWidgetsByNames(widgetNames), function (w) {
            w.visible();
        });
    },

    enable: function (widgetNames) {
        WT.each(this._getWidgetsByNames(widgetNames), function (w) {
            w.enable();
        });
    },

    disable: function (widgetNames) {
        WT.each(this._getWidgetsByNames(widgetNames), function (w) {
            w.disable();
        });
    },

    getWidgetByName: function (name) {
        return this._getWidgetByName(name);
    },

    /**
     * 提交入库
     * @param {JSON} o 配置属性
     * @param {String} o.xmlconf xml配置
     * @param {Function} o.callback 回调函数
     * @param {Object} o.feedbackMap 回调对象
     * @param {JSON} target 目标对象
     */
    dbCommit: function (o, target) {
        if (_g().saveReport) {
            _g().saveReport((function () {
                this.formSubmit({
                    url: FR.servletURL + "?op=dbcommit",
                    data: {
                        location: target
                            ? target.options.location
                            : null,
                        sheetNum: _g().selectedIndex,
                        xmlconf: o.xmlconf,
                        jsConfId: o.jsConfId,
                        callbackConfId: o.callbackConfId,
                        callback: o.callback,
                        feedbackMap: o.feedbackMap,
                        sessionID: _g().currentSessionID || _g().sessionID
                    }
                });
            }).createDelegate(this));
        } else {
            this.formSubmit({
                url: FR.servletURL + "?op=dbcommit",
                data: {
                    xmlconf: o.xmlconf,
                    jsConfId: o.jsConfId,
                    callbackConfId: o.callbackConfId,
                    callback: o.callback,
                    feedbackMap: o.feedbackMap,
                    sessionID: _g().currentSessionID || _g().sessionID
                }
            });
        }
    },

    formSubmit: function (o) {
        var o = o || {};
        var self = this;
        if (!this.verify()) {
            if (this.QueryBtn) {
                this.QueryBtn.enable();
            }
            return;
        }
        if (_g().isDirtyPage() && _g().options.warnWhenLeave) {
            if (!confirm(FR.i18nText('Fine-Engine_Report_Unload_Check_Msg') + "!\n" + FR.i18nText('Fine-Engine_Report_SQLWizard_Would_You_Like_To_Continue') + "？")) {
                if (this.QueryBtn) {
                    this.QueryBtn.enable();
                }
                return;
            }
        }
        if (!o.url) {
            return;
        }
        FR.dealWithSessionIDParam(o);
        _g().waitForCalculation().then(function () {
            var para = self.getParameters();
            if (o.data) {
                $.extend(para, o.data);
            }

            FR.ajax({
                url: o.url,
                type: "POST",
                headers: o.headers,
                data: {
                    __parameters__: FR.jsonEncode(para)
                },
                complete: function (res, status) {
                    var response = FR.jsonDecode(res.responseText);
                    if (o.callback) {
                        o.callback.call(self, res, status);
                    }
                    if (response.callback) {
                        res.responseText = response.oldres;
                        response.callback.call(self, res, status);
                    }
                    if (status === 'success') {
                        self.fireEvent(FR.Events.SS);
                    } else {
                        self.fireEvent(FR.Events.SF);
                    }
                    self.fireEvent(FR.Events.SC);
                }
            });
        });
    },

    resolveVariable: function (val) {
        var widget = this._getWidgetByName(val);
        if (widget && widget.getValue) {
            return widget.getValue();
        }
    }

});

WT.ParameterPanel.EVENT_COLLAPSED = 'EVENT_COLLAPSED';
WT.ParameterPanel.EVENT_EXPANDED = 'EVENT_EXPANDED';
WT.ParameterPanel.EVENT_CALCULATING_WIDGET = 'EVENT_CALCULATING_WIDGET';