/**
 * 自定义滚动条
 * @extends FR.OB
 */
WT.ScrollBar = FR.extend(FR.OB, {

    _defaultConfig: function () {
        return $.extend(WT.ReportBlock.superclass._defaultConfig.apply(), {
            verticalDirection: true,
            scrollBarSliderSize: 6,
            position: 0
        });
    },

    _init: function () {
        WT.ScrollBar.superclass._init.apply(this, arguments);
        this.position = this.options.position;
        this.dragging = false;

        this.handleDragStart = this._handleDragStart.bind(this);
        this.handleDragMove = this._handleDragMove.bind(this);
        this.handleDragStop = this._handleDragStop.bind(this);
        this.handleBarClick = this._handleBarClick.bind(this);

        this.element = $('<div class="wt-absolute-layout wt-custom-scrollbar"></div>');
        this.slider = $('<div class="wt-absolute-layout slider"></div>');
        this.element.append(this.slider);

        this.slider.on('mousedown', this.handleDragStart);
        $(document).on('mousemove', this.handleDragMove);
        $(document).on('mouseup', this.handleDragStop);
        this.element.on('click', this.handleBarClick);
    },

    _handleDragStart: function (e) {
        this.dragging = true;
        this.dragPosition = -1;
        e.stopPropagation();
        e.preventDefault();
        this.fireEvent(WT.ScrollBar.EVENT_DRAG_START);
    },

    _handleDragMove: function (e) {
        if (this.dragging) {
            var o = this.options;
            if (this.dragPosition !== -1) {
                var offset, lastOffset = (this.scrollingLength - this.sliderLength) * this.position;
                offset = o.verticalDirection
                    ? lastOffset + e.clientY - this.dragPosition
                    : lastOffset + e.clientX - this.dragPosition;
                if (offset < 0) {
                    offset = 0;
                } else if (offset > this.scrollingLength - this.sliderLength) {
                    offset = this.scrollingLength - this.sliderLength;
                }
                if (offset !== lastOffset) {
                    this.position = offset / (this.scrollingLength - this.sliderLength);
                    this._applyScrollPosition();
                    this.fireEvent(WT.ScrollBar.EVENT_POSITION_CHANGE, this.position, 'drag');
                }
            }
            this.dragPosition = o.verticalDirection
                ? e.clientY
                : e.clientX;
            e.stopPropagation();
            e.preventDefault();
        }
    },

    _handleDragStop: function (e) {
        if (this.dragging) {
            this.dragging = false;
            e.stopPropagation();
            e.preventDefault();
            this.fireEvent(WT.ScrollBar.EVENT_DRAG_STOP);
        }
    },

    _handleBarClick: function (e) {
        if (e.target !== e.currentTarget) {
            return;
        }
        var o = this.options;
        this.position = o.verticalDirection ? e.offsetY / this.size.height : e.offsetX / this.size.width;
        this._applyScrollPosition();
        this.fireEvent(WT.ScrollBar.EVENT_POSITION_CHANGE, this.position, 'click');
        e.stopPropagation();
        e.preventDefault();
    },

    _applyScrollPosition: function () {
        var o = this.options;
        var offset = Math.floor((this.scrollingLength - this.sliderLength) * this.position);
        if (o.verticalDirection) {
            this.slider.css('top', offset + this.spaceBetween);
        } else {
            this.slider.css('left', offset + this.spaceBetween);
        }
    },

    /**
     *
     * @param x
     * @param y
     * @param width
     * @param height
     * @param scale 滚动内容大小与容器大小的比例，大于1时表示有滚动
     */
    resize: function (x, y, width, height, scale) {
        var o = this.options;
        this.size = {
            x: x,
            y: y,
            width: width,
            height: height
        };
        this.element.css({
            left: x,
            top: y,
            width: width,
            height: height,
            'border-radius': o.verticalDirection ? Math.floor(width / 2) + 'px' : Math.floor(height / 2) + 'px'
        });

        this.sliderLength = o.verticalDirection ? Math.floor(height / scale) : Math.floor(width / scale);
        if (this.sliderLength < 22) {
            this.sliderLength = 22;
        }
        this.slider.css('border-radius', Math.floor(o.scrollBarSliderSize / 2) + 'px');
        if (o.verticalDirection) {
            this.slider.css({
                left: Math.floor((width - o.scrollBarSliderSize) / 2),
                width: o.scrollBarSliderSize,
                height: this.sliderLength
            });
        } else {
            this.slider.css({
                top: Math.floor((height - o.scrollBarSliderSize) / 2),
                height: o.scrollBarSliderSize,
                width: this.sliderLength
            });
        }

        this.spaceBetween = o.verticalDirection
            ? Math.floor((this.size.width - o.scrollBarSliderSize) / 2)
            : Math.floor((this.size.height - o.scrollBarSliderSize) / 2);
        this.scrollingLength = o.verticalDirection
            ? this.size.height - this.spaceBetween * 2
            : this.size.width - this.spaceBetween * 2;

        this._applyScrollPosition();
    },

    /**
     * 设置滚动位置，不触发滚动位置改变事件
     * @param position {number} 滚动范围的百分比，如滚动到一般位置，值为0.5
     */
    setPosition: function (position) {
        this.position = position;
        this._applyScrollPosition();
    },

    getPosition: function () {
        return this.position;
    },

    isDragging: function () {
        return this.dragging;
    },

    destroy: function () {
        this.slider.off('mousedown', this.handleDragStart);
        $(document).off('mousemove', this.handleDragMove);
        $(document).off('mouseup', this.handleDragStop);
        this.element.off('click', this.handleBarClick);
        this.purgeListeners();
        this.element.remove();
        this.element = null;
    }

});
WT.ScrollBar.EVENT_POSITION_CHANGE = 'EVENT_POSITION_CHANGE';
WT.ScrollBar.EVENT_DRAG_START = 'EVENT_DRAG_START';
WT.ScrollBar.EVENT_DRAG_STOP = 'EVENT_DRAG_STOP';
