/**
 * 滚动遮罩
 * @extends FR.OB
 */
WT.ScrollingCover = FR.extend(FR.OB, {

    _const: {
        indicatorWidth: 54,
        indicatorHeight: 30,
        margin: 20,
        marginSpace: 80
    },

    _init: function () {
        WT.ScrollingCover.superclass._init.apply(this, arguments);

        this.px = 0;
        this.py = 0;

        this.element = $('<div class="wt-absolute-layout wt-scrolling-cover"></div>');

        this.horizontalIndicator = $('<div class="wt-absolute-layout indicator"></div>');
        this.horizontalIndicator.css({
            width: this._const.indicatorWidth,
            height: this._const.indicatorHeight,
            'line-height': this._const.indicatorHeight + 'px'
        });
        this.horizontalIndicator.appendTo(this.element);
        this.verticalIndicator = $('<div class="wt-absolute-layout indicator"></div>');
        this.verticalIndicator.css({
            width: this._const.indicatorWidth,
            height: this._const.indicatorHeight,
            'line-height': this._const.indicatorHeight + 'px'
        });
        this.verticalIndicator.appendTo(this.element);

        this.element.hide();
        this.visible = false;
    },

    _applyIndicator: function () {
        if (!this.visible) {
            return;
        }
        var hAreaSize = this.size.width - this._const.margin - this._const.marginSpace - this._const.indicatorWidth;
        var vAreaSize = this.size.height - this._const.margin - this._const.marginSpace - this._const.indicatorHeight;
        var hOffset = Math.floor(hAreaSize * this.px);
        var vOffset = Math.floor(vAreaSize * this.py);
        this.horizontalIndicator.css('left', hOffset + this._const.margin);
        this.verticalIndicator.css('top', vOffset + this._const.margin);

        this.horizontalIndicator.html(Math.floor(this.px * 100) + ' %');
        this.verticalIndicator.html(Math.floor(this.py * 100) + ' %');
    },

    setPosition: function (x, y) {
        this.px = x;
        this.py = y;
        this._applyIndicator();
    },

    resize: function (x, y, width, height) {
        this.size = {
            left: x,
            top: y,
            width: width,
            height: height
        };
        this.element.css(this.size);

        this.horizontalIndicator.css('top', height - this._const.indicatorHeight - this._const.margin);
        this.verticalIndicator.css('left', width - this._const.indicatorWidth - this._const.margin);

        this._applyIndicator();
    },

    setVisible: function (visible) {
        this.visible = visible;
        if (visible) {
            this.element.show();
            this._applyIndicator();
        } else {
            this.element.hide();
        }
    },

    isVisible: function () {
        return this.visible;
    },

    destroy: function () {
        this.purgeListeners();
        this.element.remove();
        this.element = null;
    }

});
