/**
 * 报表sheet
 * @extends FR.OB
 *
 * @cfg {JSON} options 配置属性
 * @cfg {String} options.name sheet名称
 * @cfg {Boolean} options.isAlignLeft 报表显示位置是否靠左
 * @cfg {Boolean} options.visible sheet可见性
 * @cfg {Boolean} options.isShowWidgets 直接显示控件
 */
WT.Sheet = FR.extend(FR.OB, {

    _const: {
        scrollBarSize: 10,
        scrollBarSliderSize: 6,
        allSheetSwitch: true   // 暂时保留在屏幕范围内切换单元格的功能，用该变量切换
    },

    _defaultConfig: function () {
        return $.extend(WT.Sheet.superclass._defaultConfig.apply(), {
            name: 'sheet',
            isAlignLeft: true,
            visible: false,
            sheetIndex: 0,
            isShowWidgets: false
        });
    },

    _init: function () {
        WT.Sheet.superclass._init.apply(this, arguments);
        this.loaded = false;
        this.selectedCellLocation = null;
        this.needToSelectCell = null;

        //值变过的单元格的值表，key为单元格id，值为：{column, row, value}
        this.cellDirtyMap = {};

        //校验失败的单元格表，key为单元格id，值为错误信息
        this.invalidCellMap = {};

        //当前sheet内的控件名：控件值对
        this.sheetPara = {};

        this.centerScrollPosition = {
            left: 0,
            top: 0
        };

        this.sheetConfig = {
            isShowBorder: true
        };

        this.verticalScrollBar = null;
        this.horizontalScrollBar = null;

        /**
         * 正在加载的动作数
         * @type {number}
         */
        this.loadingCoverActionCount = 0;

        this.loadingBlockCount = 0;

        this.outOfRectStatusCount = 0;
        this.sheetContext = new WT.SheetContext();
        this._initView();
    },

    /**
     * 初始化dom以及子控件dom
     */
    _initView: function () {
        var o = this.options;
        this.element = $('<div class="wt-absolute-layout"></div>');
        this.setVisible(o.visible);

        this._initCellEvents();
        this._initScrollEvents();
    },

    _initCellEvents: function () {
        var o = this.options, self = this;
        var hoverCellId = null;
        var lastTarget = null;

        var handleMouseAction = function (e) {
            var target = e.target;
            if (target === lastTarget) {
                return;
            }
            lastTarget = target;
            var cellId = self._findCellElement(e.target);
            if (hoverCellId !== cellId) {
                var cell;
                if (hoverCellId) {
                    cell = self.getCellById(hoverCellId);
                    cell && cell.handleMouseOut();
                }
                if (cellId) {
                    cell = self.getCellById(cellId);
                    cell && cell.handleMouseIn();
                }

                hoverCellId = cellId;
            }
        };
        this.element.on('mouseenter', handleMouseAction);
        this.element.on('mousemove', handleMouseAction);
        this.element.on('mouseleave', function () {
            lastTarget = null;
            if (hoverCellId) {
                var cell = self.getCellById(hoverCellId);
                cell && cell.handleMouseOut();
                hoverCellId = null;
            }
        });

        var editEvent = o.isShowWidgets ? 'mousedown' : 'click';
        //直接显示控件的时候试一下mousedown的时候进入编辑状态来解决快速编辑丢失编辑值的问题
        this.element.on(editEvent, function (e) {
            var cellId = self._findCellElement(e.target);
            if (cellId) {
                var cell = self.getCellById(cellId);
                cell && cell.handleClick();
            }
        });
    },

    _initScrollEvents: function () {
        var self = this;
        this.scrollingCover = new WT.ScrollingCover();
        this.scrollingCover.element.appendTo(this.element);

        this.element.on('mousewheel', function (e) {
            e.preventDefault();
            if (!self.loaded) {
                return;
            }
            var lastX = self.centerScrollPosition.left;
            var lastY = self.centerScrollPosition.top;
            var thisX = lastX + e.deltaX * e.deltaFactor;
            var thisY = lastY - e.deltaY * e.deltaFactor;
            if (thisX < 0) {
                thisX = 0;
            }
            if (thisY < 0) {
                thisY = 0;
            }
            if (self.center.getContentWidth() > self.centerWidth) {
                if (thisX > self.center.getContentWidth() - self.centerWidth) {
                    thisX = self.center.getContentWidth() - self.centerWidth;
                }
            } else {
                thisX = 0;
            }
            if (self.center.getContentHeight() > self.centerHeight) {
                if (thisY > self.center.getContentHeight() - self.centerHeight) {
                    thisY = self.center.getContentHeight() - self.centerHeight;
                }
            } else {
                thisY = 0;
            }
            if (thisX !== lastX || thisY !== lastY) {
                self.fireEvent(WT.Sheet.EVENT_INSIDE_SCROLL, thisX, thisY, 'wheel');
            }
        });

        this.on(WT.Sheet.EVENT_INSIDE_SCROLL, function (offsetX, offsetY, type) {
            if (!self.loaded) {
                return;
            }
            self.centerScrollPosition.left = offsetX;
            self.centerScrollPosition.top = offsetY;

            if (self.north) {
                self.north.scrollTo(offsetX, 0, type);
            }
            if (self.west) {
                self.west.scrollTo(0, offsetY, type);
            }
            if (self.center) {
                self.center.scrollTo(offsetX, offsetY, type);
            }

            var px = offsetX / (self.center.getContentWidth() - self.centerWidth);
            var py = offsetY / (self.center.getContentHeight() - self.centerHeight);

            this.scrollingCover.setPosition(px, py);

            if (type !== 'click' && type !== 'drag') {
                if (self.horizontalScrollBar) {
                    self.horizontalScrollBar.setPosition(px);
                }
                if (self.verticalScrollBar) {
                    self.verticalScrollBar.setPosition(py);
                }
            }
        });
    },

    _initBlock: function (blocks) {
        var o = this.options, self = this;
        if (blocks.corner) {
            this.corner = new WT.ReportBlock($.extend({
                sheet: this,
                sheetIndex: o.sheetIndex,
                isShowWidgets: o.isShowWidgets,
                type: 'corner',
                store: o.store,
                sheetContext: this.sheetContext
            }, blocks.corner));
            this.corner.on(WT.Cell.EVENT_CELL_SELECT, function (cell) {
                self._handleCellSelect(cell);
            });
            this.corner.on(WT.ReportBlock.EVENT_CELL_UNSELECT, function () {
                self._handleCellUnSelect();
            });
            this.corner.on(WT.Cell.EVENT_SYNC_VALUE, function (cell) {
                self.calculateCellValue(cell);
            });
            this.corner.on(WT.ReportBlock.EVENT_LOADING_COVER, function (show) {
                self._handleLoadingCover(show);
            });
            this.corner.on(WT.ReportBlock.EVENT_FIRST_LOADING_COMPLETE, function () {
                self._handleBlockFirstLoadComplete();
            });
            this.corner.on(WT.ReportBlock.EVENT_LOADING_STATUS, function (isLoading) {
                self._handleLoading(isLoading);
            });
            this.element.append(this.corner.element);
        }
        if (blocks.north) {
            this.north = new WT.ReportBlock($.extend({
                sheet: this,
                sheetIndex: o.sheetIndex,
                isShowWidgets: o.isShowWidgets,
                type: 'north',
                store: o.store,
                sheetContext: this.sheetContext
            }, blocks.north));
            this.north.on(WT.Cell.EVENT_CELL_SELECT, function (cell) {
                self._handleCellSelect(cell);
            });
            this.north.on(WT.ReportBlock.EVENT_CELL_UNSELECT, function () {
                self._handleCellUnSelect();
            });
            this.north.on(WT.Cell.EVENT_SYNC_VALUE, function (cell) {
                self.calculateCellValue(cell);
            });
            this.north.on(WT.ReportBlock.EVENT_LOADING_COVER, function (show) {
                self._handleLoadingCover(show);
            });
            this.north.on(WT.ReportBlock.EVENT_FIRST_LOADING_COMPLETE, function () {
                self._handleBlockFirstLoadComplete();
            });
            this.north.on(WT.ReportBlock.EVENT_SCROLL_OUT_OF_RECT_STATUS, function (isOutOfRect) {
                self._handleScrollOutOfRectStatus(isOutOfRect);
            });
            this.north.on(WT.ReportBlock.EVENT_LOADING_STATUS, function (isLoading) {
                self._handleLoading(isLoading);
            });
            this.element.append(this.north.element);
        }
        if (blocks.west) {
            this.west = new WT.ReportBlock($.extend({
                sheet: this,
                sheetIndex: o.sheetIndex,
                isShowWidgets: o.isShowWidgets,
                type: 'west',
                store: o.store,
                sheetContext: this.sheetContext
            }, blocks.west));
            this.west.on(WT.Cell.EVENT_CELL_SELECT, function (cell) {
                self._handleCellSelect(cell);
            });
            this.west.on(WT.ReportBlock.EVENT_CELL_UNSELECT, function () {
                self._handleCellUnSelect();
            });
            this.west.on(WT.Cell.EVENT_SYNC_VALUE, function (cell) {
                self.calculateCellValue(cell);
            });
            this.west.on(WT.ReportBlock.EVENT_LOADING_COVER, function (show) {
                self._handleLoadingCover(show);
            });
            this.west.on(WT.ReportBlock.EVENT_FIRST_LOADING_COMPLETE, function () {
                self._handleBlockFirstLoadComplete();
            });
            this.west.on(WT.ReportBlock.EVENT_SCROLL_OUT_OF_RECT_STATUS, function (isOutOfRect) {
                self._handleScrollOutOfRectStatus(isOutOfRect);
            });
            this.west.on(WT.ReportBlock.EVENT_LOADING_STATUS, function (isLoading) {
                self._handleLoading(isLoading);
            });
            this.element.append(this.west.element);
        }
        if (blocks.center) {
            this.center = new WT.ReportBlock($.extend({
                sheet: this,
                sheetIndex: o.sheetIndex,
                isShowWidgets: o.isShowWidgets,
                type: 'center',
                store: o.store,
                sheetContext: this.sheetContext
            }, blocks.center));
            this.center.on(WT.ReportBlock.EVENT_REQUEST_RESIZE, function () {
                self._resizeBlock();
            });
            this.center.on(WT.Cell.EVENT_CELL_SELECT, function (cell) {
                self._handleCellSelect(cell);
            });
            this.center.on(WT.ReportBlock.EVENT_CELL_UNSELECT, function () {
                self._handleCellUnSelect();
            });
            this.center.on(WT.Cell.EVENT_SYNC_VALUE, function (cell) {
                self.calculateCellValue(cell);
            });
            this.center.on(WT.ReportBlock.EVENT_LOADING_COVER, function (show) {
                self._handleLoadingCover(show);
            });
            this.center.on(WT.ReportBlock.EVENT_FIRST_LOADING_COMPLETE, function () {
                self._handleBlockFirstLoadComplete();
            });
            this.center.on(WT.ReportBlock.EVENT_SCROLL_OUT_OF_RECT_STATUS, function (isOutOfRect) {
                self._handleScrollOutOfRectStatus(isOutOfRect);
            });
            this.center.on(WT.ReportBlock.EVENT_LOADING_STATUS, function (isLoading) {
                self._handleLoading(isLoading);
            });
            this.element.append(this.center.element);
            WT.nextTick(function () {
                this.fireEvent(WT.Sheet.EVENT_INSIDE_SCROLL, this.centerScrollPosition.left, this.centerScrollPosition.top);
                WT.nextTick(function () {
                    this.center.refreshCellData();
                    this.west && this.west.refreshCellData();
                    this.north && this.north.refreshCellData();
                    this.corner && this.corner.refreshCellData();
                }, this);
            }, this);
        }

        this._resizeBlock();
        this.loaded = true;
    },

    _clearBlock: function () {
        this._markBlockToClear();
        this.loaded = false;
    },

    _markBlockToClear: function () {
        this._markedBlockObj = {
            corner: this.corner,
            north: this.north,
            west: this.west,
            center: this.center,
            horizontalScrollBar: this.horizontalScrollBar,
            verticalScrollBar: this.verticalScrollBar
        };
        this.corner = null;
        this.north = null;
        this.west = null;
        this.center = null;
        this.horizontalScrollBar = null;
        this.verticalScrollBar = null;
    },

    _clearMarkedBlock: function () {
        if (this._markedBlockObj) {
            var b = this._markedBlockObj;
            if (b.corner) {
                b.corner.destroy();
            }
            if (b.north) {
                b.north.destroy();
            }
            if (b.west) {
                b.west.destroy();
            }
            if (b.center) {
                b.center.destroy();
            }
            if (b.horizontalScrollBar) {
                b.horizontalScrollBar.destroy();
            }
            if (b.verticalScrollBar) {
                b.verticalScrollBar.destroy();
            }
            this._markedBlockObj = undefined;
        }
    },

    _resizeBlock: function () {
        var self = this;
        if (!this.size) {
            return;
        }
        var cornerWidth = 0;
        var cornerHeight = 0;
        if (this.north) {
            cornerHeight = this.north.getContentHeight();
        }
        if (this.west) {
            cornerWidth = this.west.getContentWidth();
        }

        var centerWidth = this.size.width - cornerWidth;
        var centerHeight = this.size.height - cornerHeight;
        if (this.center && this.center.getContentWidth() > this.size.width - cornerWidth) {
            centerHeight -= this._const.scrollBarSize;
            if (!this.horizontalScrollBar) {
                this.horizontalScrollBar = new WT.ScrollBar({
                    verticalDirection: false,
                    scrollBarSliderSize: this._const.scrollBarSliderSize
                });
                this.horizontalScrollBar.element.appendTo(this.element);
                this.horizontalScrollBar.on(WT.ScrollBar.EVENT_POSITION_CHANGE, function (position, type) {
                    self._handleScrollBarScroll(position, false, type);
                });
                this.horizontalScrollBar.on(WT.ScrollBar.EVENT_DRAG_START, function () {
                    self._handleScrollBarDragStart(false);
                });
                this.horizontalScrollBar.on(WT.ScrollBar.EVENT_DRAG_STOP, function () {
                    self._handleScrollBarDragStop(false);
                });
            }
        } else {
            if (this.horizontalScrollBar) {
                this.horizontalScrollBar.destroy();
                this.horizontalScrollBar = null;
            }
        }
        if (this.center && this.center.getContentHeight() > this.size.height - cornerHeight) {
            centerWidth -= this._const.scrollBarSize;
            if (!this.verticalScrollBar) {
                this.verticalScrollBar = new WT.ScrollBar({
                    verticalDirection: true,
                    scrollBarSliderSize: this._const.scrollBarSliderSize
                });
                this.verticalScrollBar.element.appendTo(this.element);
                this.verticalScrollBar.on(WT.ScrollBar.EVENT_POSITION_CHANGE, function (position, type) {
                    self._handleScrollBarScroll(position, true, type);
                });
                this.verticalScrollBar.on(WT.ScrollBar.EVENT_DRAG_START, function () {
                    self._handleScrollBarDragStart(true);
                });
                this.verticalScrollBar.on(WT.ScrollBar.EVENT_DRAG_STOP, function () {
                    self._handleScrollBarDragStop(true);
                });
            }
        } else {
            if (this.verticalScrollBar) {
                this.verticalScrollBar.destroy();
                this.verticalScrollBar = null;
            }
        }

        if (this.corner) {
            this.corner.resize(0, 0, cornerWidth, cornerHeight);
        }
        if (this.north) {
            this.north.resize(cornerWidth, 0, centerWidth, cornerHeight);
        }
        if (this.west) {
            this.west.resize(0, cornerHeight, cornerWidth, centerHeight);
        }

        //判断居中显示的情况
        if (this.center
            && !this.west
            && !this.north
            && !this.corner
            && !this.options.isAlignLeft
            && this.size.width > this.center.getContentWidth()) {
            //居中展示
            var leftMargin = Math.floor((this.size.width - this.center.getContentWidth()) / 2);
            this.center.resize(leftMargin, 0, this.size.width - leftMargin, this.size.height);
        } else {
            //默认的靠左展示
            if (this.center) {
                this.center.resize(cornerWidth, cornerHeight, centerWidth, centerHeight);
            }
        }

        if (this.horizontalScrollBar) {
            this.horizontalScrollBar.resize(0, this.size.height - this._const.scrollBarSize,
                this.size.width - this._const.scrollBarSize, this._const.scrollBarSize,
                this.center.getContentWidth() / centerWidth);
        }

        if (this.verticalScrollBar) {
            this.verticalScrollBar.resize(this.size.width - this._const.scrollBarSize, 0,
                this._const.scrollBarSize, this.size.height - this._const.scrollBarSize,
                this.center.getContentHeight() / centerHeight);
        }

        this.cornerWidth = cornerWidth;
        this.cornerHeight = cornerHeight;
        this.centerWidth = centerWidth;
        this.centerHeight = centerHeight;
    },

    _applySheetBackground: function (bg) {
        this.sheetBackground = bg;
    },

    _handleCellSelect: function (cell, noTrig) {
        this.selectedCellLocation = cell.getLocation();
        var cells = this.getCells();
        WT.each(cells, function (c) {
            if (c !== cell) {
                c.unSelect();
            }
        }, this);
        if (noTrig !== true) {
            this.fireEvent(WT.Cell.EVENT_CELL_SELECT, cell);
        }
    },

    _handleCellUnSelect: function () {
        this.selectedCellLocation = null;
        var cells = this.getCells();
        WT.each(cells, function (c) {
            c.unSelect();
        }, this);
    },

    _handleBlockFirstLoadComplete: function () {
        // 冻结不支持背景，所以只要考虑center块
        if (this.center && this.center.isLoadCompleted()) {
            this.center.setBackground(this.sheetBackground);
            // 暂且跟背景放一起，有需要再调整
            FR.showWatermark(this.center.placeholder);
        }

        if (this.corner && !this.corner.isLoadCompleted()) {
            return;
        }
        if (this.north && !this.north.isLoadCompleted()) {
            return;
        }
        if (this.west && !this.west.isLoadCompleted()) {
            return;
        }
        if (this.center && !this.center.isLoadCompleted()) {
            return;
        }
        this._clearMarkedBlock();
        this.fireEvent(FR.Events.AFTERLOAD);
        if (this.selectedCellLocation) {
            this.selectCell(this.selectedCellLocation.column, this.selectedCellLocation.row);
        } else {
            this.selectFirstCell();
        }
    },

    _handleScrollBarScroll: function (position, isVertical, type) {
        var offsetX, offsetY;
        if (isVertical) {
            offsetX = this.horizontalScrollBar
                ? Math.floor(this.horizontalScrollBar.getPosition() * (this.center.getContentWidth() - this.centerWidth))
                : 0;
            offsetY = Math.floor(position * (this.center.getContentHeight() - this.centerHeight));
        } else {
            offsetX = Math.floor(position * (this.center.getContentWidth() - this.centerWidth));
            offsetY = this.verticalScrollBar
                ? Math.floor(this.verticalScrollBar.getPosition() * (this.center.getContentHeight() - this.centerHeight))
                : 0;
        }
        var popup = this.sheetContext.currentPopup();
        if (popup) {
            popup.hideView && popup.hideView();
        }
        this.fireEvent(WT.Sheet.EVENT_INSIDE_SCROLL, offsetX, offsetY, type);
    },

    _handleScrollBarDragStart: function (isVertical) {
        if (this.north) {
            this.north.onScrollBarDragStart(isVertical);
        }
        if (this.west) {
            this.west.onScrollBarDragStart(isVertical);
        }
        if (this.center) {
            this.center.onScrollBarDragStart(isVertical);
        }
    },

    _handleScrollBarDragStop: function (isVertical) {
        if (this.north) {
            this.north.onScrollBarDragStop(isVertical);
        }
        if (this.west) {
            this.west.onScrollBarDragStop(isVertical);
        }
        if (this.center) {
            this.center.onScrollBarDragStop(isVertical);
        }
    },

    /**
     * 把新值合并到 this.cellDirtyMap 中
     * @param dirtyValue {object | array} {id, column, row, value} | [{id, column, row, value}]
     * @private
     */
    _mergeDirtyValue: function (dirtyValue) {
        var list = WT.isArray(dirtyValue) ? dirtyValue : [dirtyValue];
        WT.each(list, function (item) {
            if (item.depended === true) {
                return;
            }
            this.cellDirtyMap[item.id] = item;
            if (item.widgetName) {
                this.sheetPara[item.widgetName.toUpperCase()] = item.value;
            }
        }, this);
    },

    _handleLoading: function (isLoading) {
        var oldCount = this.loadingBlockCount;
        if (isLoading) {
            this.loadingBlockCount++;
        } else {
            this.loadingBlockCount--;
            if (this.loadingBlockCount < 0) {
                this.loadingBlockCount = 0;
            }
        }
        if (!this.loadingBlockCount && !!this.loadingBlockCount !== !!oldCount) {
            this.corner && this.corner.applyFetchedData();
            this.north && this.north.applyFetchedData();
            this.west && this.west.applyFetchedData();
            this.center && this.center.applyFetchedData();
            if (this.needToSelectCell) {
                var c = this.needToSelectCell;
                this.needToSelectCell = null;
                this.selectCell(c.column, c.row, c.type);
            }
        }
    },

    _handleLoadingCover: function (show) {
        var oldCount = this.loadingCoverActionCount;
        if (show) {
            this.loadingCoverActionCount++;
        } else {
            this.loadingCoverActionCount--;
            if (this.loadingCoverActionCount < 0) {
                this.loadingCoverActionCount = 0;
            }
        }
        if (!!this.loadingCoverActionCount !== !!oldCount) {
            this.fireEvent(WT.ReportBlock.EVENT_LOADING_COVER, !!this.loadingCoverActionCount);
        }
    },

    _handleScrollOutOfRectStatus: function (isOutOfRect) {
        var oldCount = this.outOfRectStatusCount;
        if (isOutOfRect) {
            this.outOfRectStatusCount++;
        } else {
            this.outOfRectStatusCount--;
            if (this.outOfRectStatusCount < 0) {
                this.outOfRectStatusCount = 0;
            }
        }
        if (!!this.outOfRectStatusCount !== !!oldCount) {
            this._applyScrollingCover(!!this.outOfRectStatusCount);
        }
    },

    _applyScrollingCover: function (show) {
        this.scrollingCover.setVisible(show);
        /*
        this.north && this.north.scrollToNewPosition();
            this.west && this.west.scrollToNewPosition();
            this.center && this.center.scrollToNewPosition();
         */
    },

    /**
     * 查找元素所在的单元格元素的cellid
     * @param el 被查找元素
     * @private
     * @return {string | null} 若el为某个单元格或其子元素，返回该单元格id，否则返回null
     */
    _findCellElement: function (el) {
        if (!el) {
            return null;
        }
        var cellId = el.getAttribute && el.getAttribute('cellid');
        if (cellId) {
            return cellId;
        }
        var p = el.parentNode;
        while (p && p !== this.element[0]) {
            cellId = p.getAttribute && p.getAttribute('cellid');
            if (cellId) {
                return cellId;
            }
            p = p.parentNode;
        }
        return null;
    },

    /**
     * 指定方向选择一个可选中的单元格，当前屏幕显示范围内
     * @param direction {number} 1 - right, 2 - down, 3 - left, 4 - top
     * @private
     */
    _selectNextSelectableCellByDirection: function (direction) {
        var c1 = null, c2 = null, iteratorFn;
        var cells = this.getCellsInScreen();
        var currentLoc = this.selectedCellLocation || {column: 0, row: 0};
        if (direction === 1) {
            iteratorFn = function (cell) {
                var loc = cell.getLocation();
                if (loc.row > currentLoc.row || loc.row === currentLoc.row && loc.column > currentLoc.column) {
                    if (!c1) {
                        c1 = cell;
                    } else {
                        var lc1 = c1.getLocation();
                        if (loc.row < lc1.row || loc.row === lc1.row && loc.column < lc1.column) {
                            c1 = cell;
                        }
                    }
                } else {
                    if (!c2) {
                        c2 = cell;
                    } else {
                        var lc2 = c2.getLocation();
                        if (loc.row < lc2.row || loc.row === lc2.row && loc.column < lc2.column) {
                            c2 = cell;
                        }
                    }
                }
            };
        } else if (direction === 2) {
            iteratorFn = function (cell) {
                var loc = cell.getLocation();
                if (loc.column > currentLoc.column || loc.column === currentLoc.column && loc.row > currentLoc.row) {
                    if (!c1) {
                        c1 = cell;
                    } else {
                        var lc1 = c1.getLocation();
                        if (loc.column < lc1.column || loc.column === lc1.column && loc.row < lc1.row) {
                            c1 = cell;
                        }
                    }
                } else {
                    if (!c2) {
                        c2 = cell;
                    } else {
                        var lc2 = c2.getLocation();
                        if (loc.column < lc2.column || loc.column === lc2.column && loc.row < lc2.row) {
                            c2 = cell;
                        }
                    }
                }
            };
        } else if (direction === 3) {
            iteratorFn = function (cell) {
                var loc = cell.getLocation();
                if (loc.row < currentLoc.row || loc.row === currentLoc.row && loc.column < currentLoc.column) {
                    if (!c1) {
                        c1 = cell;
                    } else {
                        var lc1 = c1.getLocation();
                        if (loc.row > lc1.row || loc.row === lc1.row && loc.column > lc1.column) {
                            c1 = cell;
                        }
                    }
                } else {
                    if (!c2) {
                        c2 = cell;
                    } else {
                        var lc2 = c2.getLocation();
                        if (loc.row > lc2.row || loc.row === lc2.row && loc.column > lc2.column) {
                            c2 = cell;
                        }
                    }
                }
            };
        } else if (direction === 4) {
            iteratorFn = function (cell) {
                var loc = cell.getLocation();
                if (loc.column < currentLoc.column || loc.column === currentLoc.column && loc.row < currentLoc.row) {
                    if (!c1) {
                        c1 = cell;
                    } else {
                        var lc1 = c1.getLocation();
                        if (loc.column > lc1.column || loc.column === lc1.column && loc.row > lc1.row) {
                            c1 = cell;
                        }
                    }
                } else {
                    if (!c2) {
                        c2 = cell;
                    } else {
                        var lc2 = c2.getLocation();
                        if (loc.column > lc2.column || loc.column === lc2.column && loc.row > lc2.row) {
                            c2 = cell;
                        }
                    }
                }
            };
        } else {
            return null;
        }
        WT.each(cells, function (cell) {
            if (!cell.hasEditorWidget()) {
                return;
            }
            iteratorFn(cell);
        });
        var c = c1 || c2;
        if (c) {
            c.select('key');
        }
    },

    /**
     * 指定方向选择一个可选中的单元格，整个sheet范围内
     * @param direction {number} 1 - right, 2 - down, 3 - left, 4 - top
     * @private
     */
    _selectNextSelectableCellByDirectionAll: function (direction) {
        if (this.scrollingCover.isVisible()) {
            return;
        }
        var currLoc = this.selectedCellLocation;
        if (currLoc == undefined) {//null or undefined
            return;
        }
        var finderFn = function (item) {
            return item.column === currLoc.column && item.row === currLoc.row;
        };
        var currIndex;
        if (direction === 1 || direction === 3) {
            currIndex = WT.findIndex(this.selectableCellInfo.h, finderFn);
        } else if (direction === 2 || direction === 4) {
            currIndex = WT.findIndex(this.selectableCellInfo.v, finderFn);
        } else {
            return;
        }
        var self = this;
        var toNextFn = function (currIndex) {
            var nextIndex, targetCellInfo;
            if (direction === 1) {
                nextIndex = currIndex === self.selectableCellInfo.h.length - 1 ? 0 : currIndex + 1;
                targetCellInfo = self.selectableCellInfo.h[nextIndex];
            } else if (direction === 2) {
                nextIndex = currIndex === self.selectableCellInfo.v.length - 1 ? 0 : currIndex + 1;
                targetCellInfo = self.selectableCellInfo.v[nextIndex];
            } else if (direction === 3) {
                nextIndex = currIndex === 0 ? self.selectableCellInfo.h.length - 1 : currIndex - 1;
                targetCellInfo = self.selectableCellInfo.h[nextIndex];
            } else if (direction === 4) {
                nextIndex = currIndex === 0 ? self.selectableCellInfo.v.length - 1 : currIndex - 1;
                targetCellInfo = self.selectableCellInfo.v[nextIndex];
            } else {
                return;
            }
            if (!targetCellInfo) {
                return;
            }
            var rect = targetCellInfo.rect;
            if (rect.width === 0 || rect.height === 0) {
                return toNextFn(nextIndex);//隐藏状态的格子不能被选中，需要自动跳到下一个

            }
            return targetCellInfo;
        }
        var targetCell = toNextFn(currIndex);
        if (!targetCell) {
            return;
        }
        if (targetCell.column === currLoc.column && targetCell.row === currLoc.row) {
            //计算出下一个聚焦的格子跟当前聚焦的是同一个格子,那么结束编辑状态,切成纯选中状态,stopEditAndReselectCell 这个动作有点大,可能引入问题
            //REPORT-43308 这边如果不失焦的话,新填报只有一个编辑控件的时候,输入enter键无法触发保存
            this.stopEditAndReselectCell(targetCell.column, targetCell.row);
        } else {
            this.selectCell(targetCell.column, targetCell.row);
            this.scrollIntoView(targetCell.rect, targetCell.blockType, 'wheel');
        }
    },

    _transformPasteValue: function (value) {
        //换行符统一替换为\n
        value = value.replace(/\r\n/g, '\n').replace(/\r/g, '\n');
        //移除首部的换行符
        if (value.startWith('\n')) {
            value = value.substring(1, value.length);
        }
        //移除尾部的换行符
        if (value.endWith('\n')) {
            value = value.substring(0, value.length - 1);
        }
        var rows = [], start_index = 0, flag = false;
        for (var i = 0; i < value.length; i++) {
            var cr = value.charAt(i);
            if (cr === '"') {
                flag = !flag;
            } else if (cr === '\n' && !flag) {
                //'\n\r' 作为中介
                rows.push(value.substring(start_index, i)
                    .replaceAll('""', '\n\r')
                    .replaceAll('"', '')
                    .replaceAll('\n\r', '"'));
                start_index = i + 1;
            }
        }
        //由于移除了尾部的换行符，因此需要将最后一行追加上来
        if (start_index < value.length) {
            rows.push(value.substring(start_index, value.length)
                .replaceAll('""', '\n\r')
                .replaceAll('"', '')
                .replaceAll('\n\r', '"'));
        }
        for (var i = 0; i < rows.length; i++) {
            if (rows[i].endWith('\t')) {
                rows[i] = rows[i].substring(0, rows[i].length - 1);
            }
            rows[i] = rows[i].split('\t');
        }
        return rows;
    },

    _getCellsWithFetchAsync: function (columnRows) {
        var sheetIndex = this.options.sheetIndex;
        var self = this;
        var cells = this.getCells();
        var resultCells = [];

        var cellsCopy = WT.copyArray(cells), columnRowsCopy = WT.copyArray(columnRows);
        var i = 0, j = 0;
        while (i < columnRowsCopy.length) {
            var cr = columnRowsCopy[i];
            var found = false;
            for (j = 0; j < cellsCopy.length; j++) {
                var cell = cellsCopy[j];
                var loc = cell.getLocation();
                if (loc.column === cr.column && loc.row === cr.row) {
                    resultCells.push(cell);
                    cellsCopy.splice(j, 1);
                    found = true;
                    break;
                }
            }
            if (found) {
                columnRowsCopy.splice(i, 1);
            } else {
                i++;
            }
        }
        var p = WT.createPromiseResult(resultCells);
        if (columnRowsCopy.length > 0) {
            p = p.then(function (resultCells) {
                return WT.net.getWidgetsByLocation(columnRowsCopy, sheetIndex).then(function (result) {
                    self.appendNewCells(result);
                    cells = self.getCells();
                    cellsCopy = WT.copyArray(cells);
                    i = 0;
                    while (i < columnRowsCopy.length) {
                        var cr = columnRowsCopy[i];
                        var found = false;
                        for (j = 0; j < cellsCopy.length; j++) {
                            var cell = cellsCopy[j];
                            var loc = cell.getLocation();
                            if (loc.column === cr.column && loc.row === cr.row) {
                                resultCells.push(cell);
                                cellsCopy.splice(j, 1);
                                found = true;
                                break;
                            }
                        }
                        if (found) {
                            columnRowsCopy.splice(i, 1);
                        } else {
                            i++;
                        }
                    }
                    return resultCells;
                });
            });
        }
        return p;
    },


    _formatValue2Number: function(value) {
        // 如果是数字控件，则将复制的值转化成数字
        // 考虑带千位符逗号的情况,去掉
        value = $.trim(value);
        value = value.replaceAll(",", "");
        // parseFloat会把"4.00"变成4  "4.0400"变成4.04
        if (value.indexOf('.') !== -1) {
            value = value.replace(/[0]+$/, '');
            if (value.endWith('.')) {
                value = value.substring(0, value.length - 1);
            }
        } else if (value.match(/^[0]+$/)) {
            value = '0';
        } else {
            value = value.replace(/^[0]+/, '');
        }
        var ori = value;
        value = parseFloat(value, 10);
        // 粘贴10%、22sdf 这种数据，清除掉，前后有空格的保留
        // ie9之前的版本不支持string的trim方法
        if (value && value.toString() !== $.trim(ori)) {
            value = null;
        }
        if (isNaN(value)) {
            value = null;
        }
        return value;
    },

    _getColumnRows: function (rows, firstLocation) {
        var crs = [];
        for (var i = 0; i < rows.length; i++) {
            var row = rows[i];
            var r = firstLocation.row + i;
            for (var j = 0; j < row.length; j++) {
                var c = firstLocation.column + j;
                crs.push({
                    column: c,
                    row: r
                });
            }
        }
        return crs;
    },

    handlePaste: function (value) {
        if (!value) {
            return;
        }
        var cell = this.getSelectedCell();
        if (!cell) {
            return;
        }
        var rows = this._transformPasteValue(value);

        if (rows.length === 1 && rows[0].length === 1) {
            // 数字控件处理一下 REPORT-36622
            if (cell.getWidget().getType() === 'WT.NumberEditor') {
                rows[0][0] = this._formatValue2Number(rows[0][0]);
            }
            if (cell.hasEditorWidget()) {
                cell.edit();
                setTimeout(function () {
                    cell.setValue(rows[0][0], true);
                }, 10);
            }
            return;
        }

        var self = this;
        var firstLocation = cell.getLocation();
        var crs = this._getColumnRows(rows, firstLocation);
        this.fireEvent(WT.ReportBlock.EVENT_LOADING_COVER, true);
        this._getCellsWithFetchAsync(crs).then(function (cells) {
            return WT.filter(cells, function (c) {
                if (!c.hasEditorWidget()) {
                    return false;
                }
                var editor = c.getWidget();
                return editor.isEnable()
                    && editor.isVisible()
                    && !(editor.options.directEdit === false || editor.options.allowCustomValue === false)
            });
        }).then(function (cells) {
            var lastLoc = null;
            var editedCRs = [];
            WT.each(cells, function (cell) {
                var loc = cell.getLocation();
                if (!lastLoc) {
                    lastLoc = loc;
                } else {
                    if (loc.row > lastLoc.row || loc.row === lastLoc.row && loc.column > lastLoc.column) {
                        lastLoc = loc;
                    }
                }
                var offsetCol = loc.column - firstLocation.column;
                var offsetRow = loc.row - firstLocation.row;
                var value = rows[offsetRow][offsetCol];
                if (value) {
                    var fm = null;
                    if (WT.isString(value) && value.startWith('=')) {
                        fm = value;
                        value = null;
                    } else if (cell.getWidget().getType() === 'WT.NumberEditor') {
                       value = self._formatValue2Number(value);
                    }

                    if (value || value === 0 || fm) {
                        cell.setValue(value);
                        editedCRs.push({
                            col: loc.column,
                            row: loc.row
                        });
                        self._mergeDirtyValue({
                            id: cell.getID(),
                            column: loc.column,
                            row: loc.row,
                            value: value,
                            formula: fm
                        });

                        var w = cell.getWidget();
                        cell.setIgnoreValueChangeEvent(true);
                        w.fireEvent(FR.Events.AFTEREDIT);
                        w.fireEvent(FR.Events.STOPEDIT);
                        cell.setIgnoreValueChangeEvent(false);
                    }
                }
            });

            var dirtyCellList = [{
                sheetIndex: self.options.sheetIndex,
                cells: self.getDirtyCells()
            }];
            return WT.net.calculateCellsValue(editedCRs, self.options.sheetIndex, dirtyCellList).then(function (cellInfoList) {
                self.fireEvent(WT.ReportBlock.EVENT_CELLS_RELOAD, cellInfoList);
            }).then(function () {
                self.selectCell(lastLoc.column, lastLoc.row);
            });
        })['catch'](WT.toastError).then(function () {
            self.fireEvent(WT.ReportBlock.EVENT_LOADING_COVER, false);
        });
    },

    getWidgetByCell: function (col, row) {
        var cell = this.getCell(col, row);
        if (!cell) {
            try {
                var cellConfig = WT.net.getWidgetByLocationSync({
                    column: col,
                    row: row,
                    sheetIndex: this.getSheetIndex()
                });
                this.appendNewCells(cellConfig);
                cell = this.getCell(col, row);
            } catch (ignore) {
            }
        }
        return cell ? cell.getWidget() : null;
    },

    /**
     * 设置单元格新值
     * @param newValue {object | array} {id, column, row, value} | [{id, column, row, value}]
     */
    applyNewValue: function (newValue) {
        this._mergeDirtyValue(newValue);
        this.corner && this.corner.applyNewValue(newValue);
        this.north && this.north.applyNewValue(newValue);
        this.west && this.west.applyNewValue(newValue);
        this.center && this.center.applyNewValue(newValue);
    },

    /**
     * 计算该cell的相关联的其他格子的值
     * @param cell
     */
    calculateCellValue: function (cell) {
        var loc = cell.getLocation();
        var opts = {
            id: cell.getID(),
            column: loc.column,
            row: loc.row,
            value: WT.transformValueType(cell.getValue()),
            widgetName: cell.getWidgetName()
        };
        if ((!cell.widget || !WT.isTreeWidget(cell.widget)) && cell.originValue) {
            opts.originValue = cell.originValue;
        }
        this._mergeDirtyValue(opts);
        this.fireEvent(WT.Cell.EVENT_SYNC_VALUE, cell);
    },

    /**
     * 获取该sheet下所有单元格
     * @return {Array}
     */
    getCells: function () {
        var cells = [];
        if (this.corner) {
            WT.each(this.corner.getCellMap(), function (cell) {
                cells.push(cell);
            }, this);
        }
        if (this.north) {
            WT.each(this.north.getCellMap(), function (cell) {
                cells.push(cell);
            }, this);
        }
        if (this.west) {
            WT.each(this.west.getCellMap(), function (cell) {
                cells.push(cell);
            }, this);
        }
        if (this.center) {
            WT.each(this.center.getCellMap(), function (cell) {
                cells.push(cell);
            }, this);
        }
        return cells;
    },

    getCellsInScreen: function () {
        var cells = [], scrollRect;
        if (this.corner) {
            scrollRect = this.corner.getScrollRect();
            WT.each(this.corner.getCellMap(), function (cell) {
                if (WT.rectContains(scrollRect, cell.getRect(), true)) {
                    cells.push(cell);
                }
            }, this);
        }
        if (this.north) {
            scrollRect = this.north.getScrollRect();
            WT.each(this.north.getCellMap(), function (cell) {
                if (WT.rectContains(scrollRect, cell.getRect(), true)) {
                    cells.push(cell);
                }
            }, this);
        }
        if (this.west) {
            scrollRect = this.west.getScrollRect();
            WT.each(this.west.getCellMap(), function (cell) {
                if (WT.rectContains(scrollRect, cell.getRect(), true)) {
                    cells.push(cell);
                }
            }, this);
        }
        if (this.center) {
            scrollRect = this.center.getScrollRect();
            WT.each(this.center.getCellMap(), function (cell) {
                if (WT.rectContains(scrollRect, cell.getRect(), true)) {
                    cells.push(cell);
                }
            }, this);
        }
        return cells;
    },

    /**
     * 获取该sheet下指定位置的单元格
     * @param column
     * @param row
     * @return {*}
     */
    getCell: function (column, row) {
        var cell = null;
        if (!cell && this.corner) {
            cell = WT.find(this.corner.getCellMap(), function (cell) {
                var l = cell.getLocation();
                return l.column === column && l.row === row;
            }, this);
        }
        if (!cell && this.north) {
            cell = WT.find(this.north.getCellMap(), function (cell) {
                var l = cell.getLocation();
                return l.column === column && l.row === row;
            }, this);
        }
        if (!cell && this.west) {
            cell = WT.find(this.west.getCellMap(), function (cell) {
                var l = cell.getLocation();
                return l.column === column && l.row === row;
            }, this);
        }
        if (!cell && this.center) {
            cell = WT.find(this.center.getCellMap(), function (cell) {
                var l = cell.getLocation();
                return l.column === column && l.row === row;
            }, this);
        }
        return cell;
    },

    getCellById: function (cellId) {
        if (!cellId) {
            return null;
        }
        var cell = null;
        if (this.corner) {
            cell = this.corner.getCellMap()[cellId];
            if (cell) {
                return cell;
            }
        }
        if (this.north) {
            cell = this.north.getCellMap()[cellId];
            if (cell) {
                return cell;
            }
        }
        if (this.west) {
            cell = this.west.getCellMap()[cellId];
            if (cell) {
                return cell;
            }
        }
        if (this.center) {
            cell = this.center.getCellMap()[cellId];
            if (cell) {
                return cell;
            }
        }
        return null;
    },

    /**
     * 选中指定单元格
     * @param column 单元格column
     * @param row 单元格row
     * @param type
     */
    selectCell: function (column, row, type) {
        var cell = this.getCell(column, row);
        if (cell) {
            cell.select(type);
        } else {
            this.needToSelectCell = {
                column: column,
                row: row,
                type: type
            };
        }
    },
    /**
     * 结束编辑当前格子,不切换选中框
     * @param column 单元格column
     * @param row 单元格row
     * @param type
     */
    stopEditAndReselectCell: function (column, row) {
        var cell = this.getCell(column, row);
        if (cell) {
            cell.unSelect();
            cell.select();
        }
    },

    selectFirstCell: function () {
        var firstCommonCell = null, cell = null;
        var iteratorFn = function (c) {
            var l = c.getLocation();
            if (!firstCommonCell) {
                firstCommonCell = c;
            } else {
                var fl = firstCommonCell.getLocation();
                if (l.row < fl.row || l.row === fl.row && l.column < fl.column) {
                    firstCommonCell = c;
                }
            }
            if (c.hasEditorWidget()) {
                if (!cell) {
                    cell = c;
                } else {
                    var cl = cell.getLocation();
                    if (l.row < cl.row || l.row === cl.row && l.column < cl.column) {
                        cell = c;
                    }
                }
            }
        };
        // 先找第一个控件格子
        if (!cell && this.corner) {
            WT.each(this.corner.getCellMap(), iteratorFn);
        }
        if (!cell && this.north) {
            WT.each(this.north.getCellMap(), iteratorFn);
        }
        if (!cell && this.west) {
            WT.each(this.west.getCellMap(), iteratorFn);
        }
        if (!cell && this.center) {
            WT.each(this.center.getCellMap(), iteratorFn);
        }

        if (cell) {
            cell.select();
        } else if (firstCommonCell) {
            firstCommonCell.select();
        }
    },

    /**
     * 选择下一个单元格（右边，按tab或方向右）
     */
    selectNext: function () {
        this._const.allSheetSwitch
            ? this._selectNextSelectableCellByDirectionAll(1)
            : this._selectNextSelectableCellByDirection(1);
    },

    /**
     * 选择上一个单元格（左边，按方向左）
     */
    selectPrev: function () {
        this._const.allSheetSwitch
            ? this._selectNextSelectableCellByDirectionAll(3)
            : this._selectNextSelectableCellByDirection(3);
    },

    /**
     * 选择纵向下一个单元格（下边，按enter或方向下）
     */
    selectDown: function () {
        this._const.allSheetSwitch
            ? this._selectNextSelectableCellByDirectionAll(2)
            : this._selectNextSelectableCellByDirection(2);
    },

    /**
     * 选择纵向上一个单元格（右边，按方向上）
     */
    selectUp: function () {
        this._const.allSheetSwitch
            ? this._selectNextSelectableCellByDirectionAll(4)
            : this._selectNextSelectableCellByDirection(4);
    },

    /**
     * 编辑单元格
     */
    editCell: function () {
        var cell = this.getSelectedCell();
        if (cell) {
            cell.edit();
        }
    },

    /**
     * 清除单元格值
     */
    clearCellValue: function () {
        var cell = this.getSelectedCell();
        if (cell) {
            cell.setValue('', true);
        }
    },

    getSelectedCell: function () {
        if (!this.selectedCellLocation) {
            return null;
        }
        return this.getCell(this.selectedCellLocation.column, this.selectedCellLocation.row);
    },

    getDirtyCells: function () {
        var list = [];
        WT.each(this.cellDirtyMap, function (cellValue) {
            list.push(cellValue);
        });
        return list;
    },

    getDirtyCellMap: function () {
        return this.cellDirtyMap;
    },

    clearDirtyCells: function () {
        this.cellDirtyMap = {};
    },

    /**
     * 滚动指定范围到屏幕中
     * @param rect {{left, top, width, height}}
     * @param blockType {'corner', 'west', 'north', 'center'}
     * @param scrollType?
     */
    scrollIntoView: function (rect, blockType, scrollType) {
        if (blockType === 'corner') {
            //corner无法滚动
            return;
        }
        if (!this.center) {
            return;
        }
        var scrollRect = this.center.getScrollRect();
        var scrollTop = scrollRect.top, scrollLeft = scrollRect.left;
        if (blockType === 'west' || blockType === 'center') {
            if (scrollRect.top > rect.top) {
                scrollTop = rect.top;
            } else if (scrollRect.top + scrollRect.height < rect.top + rect.height) {
                scrollTop = rect.top + rect.height - scrollRect.height;
            }
        }
        if (blockType === 'north' || blockType === 'center') {
            if (scrollRect.left > rect.left) {
                scrollLeft = rect.left;
            } else if (scrollRect.left + scrollRect.width < rect.left + rect.width) {
                scrollLeft = rect.left + rect.width - scrollRect.width;
            }
        }
        this.fireEvent(WT.Sheet.EVENT_INSIDE_SCROLL, scrollLeft, scrollTop, scrollType);
    },

    /**
     * 设置校验失败的单元格信息
     * @param cellInfoList [{id, msg}]
     */
    setInvalidCellInfo: function (cellInfoList) {
        WT.each(cellInfoList, function (info) {
            this.invalidCellMap[info.id] = info.msg;
        }, this);
        WT.each(this.getCells(), function (cell) {
            var msg = this.invalidCellMap[cell.getID()];
            if (msg !== undefined) {
                cell.setInvalid(msg);
            }
        }, this);
    },

    getInvalidCellMap: function () {
        return this.invalidCellMap;
    },

    /**
     * 清空校验失败的单元格信息
     */
    clearInvalidCellInfo: function () {
        WT.each(this.getCells(), function (cell) {
            if (this.invalidCellMap[cell.getID()]) {
                cell.setInvalid(null);
            }
        }, this);
        this.invalidCellMap = {};
    },

    /**
     * 设置大小
     * @param x {Number} 左边距
     * @param y {Number} 上边距
     * @param width {Number} 宽度
     * @param height {Number} 高度
     */
    resize: function (x, y, width, height) {
        this.size = {
            left: x,
            top: y,
            width: width,
            height: height
        };
        this.element.css({
            left: x,
            top: y,
            width: width,
            height: height
        });
        this.scrollingCover.resize(x, y, width, height);
        if (!this.loaded) {
            return;
        }
        this._resizeBlock();
    },

    /**
     * 通知报表开始加载数据
     */
    loadSheetData: function () {
        var self = this;
        if (this.loaded) {
            return;
        }
        this.loaded = true;
        this.fireEvent(FR.Events.STARTLOAD);
        WT.net.getSheetInfo(this.options.sheetIndex, this.element).then(function (data) {
            var h = WT.map(data.selectableCell.h, function (item) {
                var s = item.split(',');
                var blockType = '';
                if (s[7] === 'r') {
                    blockType = 'corner';
                } else if (s[7] === 'n') {
                    blockType = 'north';
                } else if (s[7] === 'w') {
                    blockType = 'west';
                } else if (s[7] === 'c') {
                    blockType = 'center';
                }
                return {
                    id: s[0],
                    column: parseInt(s[1]),
                    row: parseInt(s[2]),
                    rect: {
                        left: parseInt(s[3]),
                        top: parseInt(s[4]),
                        width: parseInt(s[5]),
                        height: parseInt(s[6])
                    },
                    blockType: blockType
                }
            });
            var v = [];
            WT.each(data.selectableCell.v, function (n) {
                v.push(h[n]);
            });
            self.selectableCellInfo = {
                h: h,
                v: v
            };
            self._applySheetBackground(data.background);
            self._initBlock(data.block);
            FR.loadWatermark(data.watermark);
            if (report && report.sheetContainer && report.sheetContainer.tabBar && report.sheetContainer.tabBar.tabBtns) {
                $.each(data.sheets, function (index, sheet) {
                    report.sheetContainer.tabBar.tabBtns[index].reload(sheet.title)
                });
            }
            if (data.copyright) {
                FR.loadCopyright(data.copyright);
            }
        })['catch'](function (err) {
            self.fireEvent(FR.Events.AFTERLOAD);
        });
    },

    loadSheetDataAfterAppendOrDelete: function (location) {
        if (this.selectedCellLocation) {
            this.selectedCellLocation.column = location.column;
            this.selectedCellLocation.row = location.row;
        }
        this.loadSheetData();
    },

    /**
     * 加载新的单元格
     * @param cellConfig {{corner?: any[], north?: any[], west?: any[], center?: any[]}} 单元格config
     */
    appendNewCells: function (cellConfig) {
        if (cellConfig.corner && this.corner) {
            this.corner.appendNewCells(cellConfig.corner);
        }
        if (cellConfig.north && this.north) {
            this.north.appendNewCells(cellConfig.north);
        }
        if (cellConfig.west && this.west) {
            this.west.appendNewCells(cellConfig.west);
        }
        if (cellConfig.center && this.center) {
            this.center.appendNewCells(cellConfig.center);
        }
    },

    clearSheetData: function () {
        this._clearBlock();
        this.loaded = false;
    },

    isLoaded: function () {
        return this.loaded;
    },

    isFrozen: function () {
        return this.corner || this.north || this.west;
    },

    /**
     * 设置sheet可见性，用于tab切换
     * @param visible 可见性
     */
    setVisible: function (visible) {
        this.options.visible = visible;
        if (visible) {
            this.element.show();
        } else {
            this.element.hide();
        }
    },

    isVisible: function () {
        return this.options.visible;
    },

    getName: function () {
        return this.options.name;
    },

    disableBorder: function () {
        this.sheetConfig.isShowBorder = false;
        var cell = this.getSelectedCell();
        if (cell) {
            cell.disableBorder();
        }
    },

    /**
     * 锁住窗口滚动
     * @param lock {Boolean} 是否锁住
     */
    lockScroll: function (lock) {
        //corner不用管，本来就不能滚动
        this.north && this.north.lockScroll(lock);
        this.west && this.west.lockScroll(lock);
        this.center && this.center.lockScroll(lock);
    },
    
    /**
     * 清除整个sheet的dom以及子组件
     */
    clearSheetAll: function(){
        this.purgeListeners();
        this.element.empty();
        this.loaded = false;
    }

});
WT.Sheet.EVENT_INSIDE_SCROLL = 'EVENT_INSIDE_SCROLL';

WT.SheetContext = FR.extend(FR.OB, {

    _defaultConfig: function () {
        return $.extend(WT.SheetContext.superclass._defaultConfig.apply(), {
            currentPopup: null
        });
    },

    currentPopup: function (popup) {
        if (popup !== undefined) {
            this.options.currentPopup = popup;
        } else {
            return this.options.currentPopup;
        }
    }

});
