/**
 * 多sheet时的Tab控件
 * @extends FR.OB
 *
 * @cfg {JSON} options 配置属性
 * @cfg {Array<String>} options.items 选择项
 */
WT.TabBar = FR.extend(FR.OB, {

    _defaultConfig: function () {
        return $.extend(WT.TabBar.superclass._defaultConfig.apply(), {
            baseCls: "fr-tabpane",
            items: [],
            tabHeight: 22,
            defaultActiveIndex: 0,
            noControl: false,
            tabPosition: 'bottom'
        });
    },

    _init: function () {
        WT.TabBar.superclass._init.apply(this, arguments);
        this._initView();
    },

    /**
     * 初始化dom以及子控件dom
     */
    _initView: function () {
        var opts = this.options;
        this.activeTabIndex = opts.defaultActiveIndex; //当前被激活tab的位置索引
        this.wrapWidth = 0; //记录所有标签总宽度
        this.element = $('<div class="wt-absolute-layout"></div>');
        //生成配置信息
        var borderLayoutConfig = this._createConfig4TabControl();
        //整体布局
        this.borderLayout = new FR.BorderLayout(borderLayoutConfig);
        this.isMoving = false;
        this.tabsContent.parent().addClass(opts.baseCls + '-tabscontent-bg');
        WT.nextTick(function () {
            //生成所有的tab标签
            this._initTabs();
            //选中默认tab
            if (this.tabBtns[opts.defaultActiveIndex]) {
                this.tabBtns[opts.defaultActiveIndex].setSelected();
            }
        }, this);
    },

    /**
     * 生成标签和控制按钮的配置信息
     * @private
     */
    _createConfig4TabControl: function () {
        var self = this, opts = this.options;
        this.tabsContent = $('<div/>').addClass(opts.baseCls + '-tabscontent')
            .addClass(opts.baseCls + '-tabscontent-' + opts.tabPosition);
        var items = [
            {
                region: "center",
                height: opts.tabHeight,
                el: $('<div/>').append(this.tabsContent)
            }
        ];
        //不需要tab控制按钮就不生成
        if (!this.options.noControl) {
            items.unshift({
                region: "west",
                el: {
                    type: 'horizontal',
                    widgetName: 'controlbuttons',
                    baseCls: 'fr-tabpane-controlbuttons',
                    alignment: 'center',
                    hgap: 3,
                    vgap: 2,
                    items: [
                        {
                            el: this._createTabControlBtn('arrow1', function () {
                                if(self.isMoving){
                                    return;
                                }
                                self._moveViewToPos('start'); //移动到第一个tab
                            }),
                            width: 13
                        },
                        {
                            el: this._createTabControlBtn('arrow2', function () {
                                if(self.isMoving){
                                    return;
                                }
                                var distance = self.stepDis;
                                var realDis = self.tabsContent.offset().left - self.tabsWrap.offset().left;
                                distance = Math.min(distance,realDis);
                                self._moveViewToPos(distance);
                            }),
                            width: 13
                        },
                        {
                            el: this._createTabControlBtn('arrow3', function () {
                                if(self.isMoving){
                                    return;
                                }
                                var distance = self.stepDis;
                                var realDis = self.tabsWrap.offset().left + self.wrapWidth
                                    - self.tabsContent.offset().left - self.tabsContent.width();
                                distance = Math.min(distance,realDis);
                                distance = distance < 0?0:distance;
                                self._moveViewToPos(- distance);
                            }),
                            width: 13
                        },
                        {
                            el: this._createTabControlBtn('arrow4', function () {
                                if(self.isMoving){
                                    return;
                                }
                                self._moveViewToPos('end'); //移动到最后一个tab
                            }),
                            width: 13
                        }
                    ]
                },
                width: 72,
                height: opts.tabHeight
            });
        }
        return {
            type: "border",
            renderEl: this.element,
            items: items
        };
    },

    /**
     * 生成tab控制按钮
     * @param {String} name 样式类名
     * @param {Function} handler 点击事件
     * @returns {jQuery} 返回按钮的DOM对象
     * @private
     */
    _createTabControlBtn: function (name, handler) {
        var iconbtn = new FR.IconButton({
            type: "iconbutton",
            baseClass: 'fr-edit-sheetcontrol-icon-box',
            handler: handler
        });
        $('<div>').addClass('fr-edit-sheetcontrol-icon-' + name).appendTo(iconbtn.element);
        return iconbtn.element;
    },

    /**
     * 初始化所有的tab
     * @private
     */
    _initTabs: function () {
        var opts = this.options;
        this.tabBtns = [];
        //用来装载所有的tab标签并用于平移
        this.tabsWrap = $('<ul/>').addClass(opts.baseCls + '-tabswrap').appendTo(this.tabsContent);
        if (!opts.items) {
            return;
        }
        if (!opts.tabs) {
            opts.tabs = [];
        }
        for (var i = 0; i < opts.items.length; i++) {
            this._addTabTag(i, opts.items[i]);
        }
    },

    /**
     * 添加一个tab的标签
     * @param {Number} index 位置索引
     * @param {jQuery} item tab对象
     * @private
     */
    _addTabTag: function (index, item) {
        var self = this, items = this.options.items;
        //生成一个tab标签
        var config = {
            tabPane: this,
            name: item,
            index: index,
            isFirst: index === 0,
            isLast: index === items.length - 1,
            renderEl: $('<li/>').appendTo(this.tabsWrap),
            tabBtns: this.tabBtns,
            handler: function () {
                self.activeTabIndex = index;
                self._showSheetContent(self.activeTabIndex);
                self._moveViewToShow(self.activeTabIndex);
            }
        };
        var sheetBtn = new FR.SheetButton(config);
        this.tabBtns.push(sheetBtn);
        this.wrapWidth += sheetBtn.getWidth();
        this.stepDis = this.wrapWidth/this.tabBtns.length; //取平均数作为固定移动步长
    },

    /**
     * 使指定位置的tab移动到可完整显示
     * @param {Number} tabIndex tab的位置索引
     * @private
     */
    _moveViewToShow: function (tabIndex) {
        var tabObj = this.tabBtns[tabIndex].element; //当前选中DOM
        var view = this.tabsWrap;  //滑动主体DOM
        var bounds = this.tabsContent; //外边界DOM
        var boundsLeft = bounds.offset().left;
        var boundsRight = boundsLeft + bounds.width();
        var viewLeft = view.offset().left;
        var viewRight = viewLeft + this.wrapWidth;
        var tabLeft = tabObj.offset().left;
        var tabRight = tabLeft + tabObj.width();
        var dis = tabObj.width(); //理应移动的距离
        var realDis = 0; //实际可移动的距离
        if (tabLeft < boundsLeft) {
            //左出界,右移 +
            realDis = boundsLeft - viewLeft;
            realDis = Math.min(realDis, dis);
        } else if (tabRight > boundsRight) {
            //右出界,左移 -
            realDis = boundsRight - viewRight;
            realDis = Math.max(realDis, -dis);
        } else {
            return;
        }
        this._moveViewToPos(realDis);
    },

    /**
     * 所有tab移动一段距离或移动到某个位置
     * @param {Number/String/'start'/'end'} distance 距离,'start'表示移动到第一个tab标签,'end'表示移动到最后一个tab标签
     * @private
     */
    _moveViewToPos: function (distance) {
        var self = this;
        this.isMoving = true;
        if (distance === 'start') {
            //表示回到原点
            this.tabsWrap.animate({
                'left': '0'
            }, 'fast', function(){
                self.isMoving = false;
            });
        } else if (distance === 'end') {
            var pos = this.tabsContent.width() - this.wrapWidth;
            if(pos > 0){
                pos = 0;
            }
            this.tabsWrap.animate({
                'left': pos
            }, 'fast', function(){
                self.isMoving = false;
            });
        }
        else {
            this.tabsWrap.animate({
                'left': '+=' + distance
            }, 'fast', function(){
                self.isMoving = false;
            });
        }
    },
    /**
     * 显示指定位置索引的sheet内容
     * @param {Number} position 位置索引
     */
    _showSheetContent: function (position) {
        // this.fireEvent(FR.Events.TABCHANGESTART, this, position);
        this.fireEvent(FR.Events.TABCHANGE, position);
        // this.fireEvent(FR.Events.SCROLLCHANGE, this, position);
    },

    /**
     * 切换到指定tab
     * @param index
     */
    selectTab: function (index) {
        if (this.tabBtns[index]) {
            this.tabBtns[index].setSelected();
        }
    },

    /**
     * 获取预定义的高度，给父组件布局使用
     */
    getHeight: function () {
        return this.options.tabHeight;
    },

    /**
     * 设置大小
     * @param x {Number} 左边距
     * @param y {Number} 上边距
     * @param width {Number} 宽度
     * @param height {Number} 高度
     */
    resize: function (x, y, width, height) {
        this.element.css({
            left: x,
            top: y,
            width: width,
            height: height
        });
        this.borderLayout.doLayout();
    },

    setInvalidTab: function (index) {
        var btn = this.tabBtns[index];
        btn && btn.setValidState(false);
    },

    clearInvalidTab: function () {
        WT.each(this.tabBtns, function (btn) {
            btn.setValidState(true);
        });
    }

});