/**
 * 工具栏
 * @extends FR.OB
 * 
 * @cfg {JSON} options 配置属性
 * @cfg {Boolean} options.enable 工具栏可用性
 * @cfg {Array} options.widgets 子控件属性列表
 */
WT.ToolBar = FR.extend(FR.OB, {

	_defaultConfig: function () {
        return $.extend(WT.ToolBar.superclass._defaultConfig.apply(), {
			enable: false,
            widgets: []
		});
    },

	_init: function () {
		WT.ToolBar.superclass._init.apply(this, arguments);
		this._initView();
	},

	/**
	 * 初始化工具栏dom以及子控件dom
	 */
	_initView: function () {
	    var o = this.options, self = this;
		this.element = $('<div class="wt-absolute-layout wt-border-box wt-toolbar"></div>');
        o.background && WT.setBackground(this.element, o.background);
		this.items = [];
		WT.each(o.widgets, function (widget, idx) {
		    var xtype = widget.xtype;
		    if (!xtype) {
		        return;
            }
			var cfg = widget.config;
		    cfg.listeners = widget.events;
            var item = WT.createWidget(xtype, cfg);
            item.element.addClass('item');
            if (idx > 0) {
                self.element.append(self._createSeparator());
            }
            self.element.append(item.element);
            self.items.push(item);
        });
	},

    _createSeparator: function () {
	    return $('<span class="sep"/>');
    },

	/**
	 * 设置大小
	 * @param x {Number} 左边距
	 * @param y {Number} 上边距
	 * @param width {Number} 宽度
	 * @param height {Number} 高度
	 */
	resize: function (x, y, width, height) {
		this.element.css({
		    left: x,
            top: y,
            width: width,
            height: height
        });
	},

	/**
	 * 获取预定义的高度，给父组件布局使用
	 */
	getHeight: function () {
		return this.options.height;
	},

	isEnable: function () {
		return this.options.enable;
	},

	setEnable: function (enable) {
		if (this.options.enable === enable) {
		    return;
        }
        this.options.enable = enable;
		if (!this.items) {
		    return;
        }
		if (enable) {
            WT.each(this.items, function (item) {
                item.setEnable(true);
            });
        }
        else {
            WT.each(this.items, function (item) {
                item.setEnable(false);
            });
        }
	},

    getRootElement: function () {
        return this.element;
    }

});