/**
 * 新填报下按钮的基本类
 * @class WT.BaseButton
 * @extends WT.Widget
 *
 * @cfg {JSON} options 配置属性
 * @cfg {String} options.text 显示文字
 * @cfg {String} options.fontFamily 字体
 * @cfg {String} options.fontWeight 字体粗细
 * @cfg {String} options.fontStyle 字体样式
 * @cfg {String} options.fontSize 字体大小
 * @cfg {String} options.decoration 字体修饰（下划线等样式）
 * @cfg {String} options.color 字体颜色
 * @cfg {String} options.bgNormal 普通状态背景，作为background的样式属性添加
 * @cfg {String} options.bgHover 鼠标悬浮状态背景，作为background的样式属性添加
 * @cfg {String} options.bgClick 点击状态背景，作为background的样式属性添加
 * @cfg {String} options.shortcutKey 快捷键
 * @cfg {String} options.icon 按钮图标，作为background的样式属性添加
 * @cfg {String} options.iconOnly 仅支持图标选项，icon不为空时有效
 * @cfg {Boolean} options.wrap 宽度不够时文字能否多行显示
 */
WT.BaseButton = FR.extend(WT.Widget, {

    TYPE: 'WT.BaseButton',

    _const: {
        iconSize: 16
    },

    forceCustomFontConstant: false,

    _defaultConfig: function () {
        return $.extend(WT.BaseButton.superclass._defaultConfig.apply(), {
            width: 'auto',
            text: '',
            textAlign: 'center',
            color: null,
            fontFamily: null,
            fontWeight: null,
            fontStyle: null,
            fontSize: 12,
            decoration: null,
            bgNormal: null,
            bgHover: null,
            bgClick: null,
            shortcutKey: null,
            icon: null,
            iconOnly: false,
            wrap: false,
            horizontalPadding: 0
        });
    },

    /**
     * 初始化其他变量
     * @protected
     */
    _initVariable: function () {
        WT.BaseButton.superclass._initVariable.apply(this);
    },

    /**
     * 初始化dom
     */
    _initView: function () {
        var o = this.options, self = this;
        this.button = new BI.BasicButton({
            forceNotSelected: true,
            handler: function () {
                self.fireEvent(FR.Events.CLICK);
            }
        });
        this.button.setEnable = function (enable) {
            if (enable === true) {
                this.options.disabled = false;
            } else if (enable === false) {
                this.options.disabled = true;
            }
        }.bind(this.button);
        this.button._isMounted = true;
        this.element = WT.dom.createJQueryElement('div', 'wt-absolute-layout');
        this.element.css({
            left: o.left,
            top: o.top,
            height: o.height,
            width: o.width
        });
        WT.dom.appendChild(this.element, this.button.element);
        WT.setBackground(this.element, o.bgNormal || '');
        var items = [];
        var text = o.text;
        if (!text && this.options.freeButton && o.value) {
            text = o.value;
        }
        if (!WT.isString(text)) {
            text = WT.object2String(text);
        }
        if (WT.isKey(o.icon)) {
            this.icon = BI.createWidget({
                type: "bi.icon",
                width: this._const.iconSize,
                height: this._const.iconSize
            });
            (WT.isString(o.icon) && o.icon.indexOf('css:') === 0)
                ? this.icon.element.addClass(o.icon.substr(4))
                : this.icon.element.css('background', o.icon);
            items.push(this.icon);
        }
        if (!o.iconOnly) {
            this.text = BI.createWidget({
                type: "bi.label",
                lgap: 2,
                text: text,
                height: o.height,
                whiteSpace: o.wrap ? "normal" : "nowrap"
            });
            this.text.element.attr('unselectable', 'on');
            this.text.element.addClass('wt-no-select');
            this.text.element.css('font-size', o.fontSize);
            o.color && this.text.element.css('color', o.color);
            if (o.fontFamily) {
                this.text.element.css('font-family', o.fontFamily);
            }
            if (o.fontWeight) {
                this.text.element.css('font-weight', o.fontWeight);
            }
            if (o.fontStyle) {
                this.text.element.css('font-style', o.fontStyle);
            }
            if (o.decoration) {
                this.text.element.css('text-decoration', o.decoration);
            }
            if (!this.forceCustomFontConstant && !o.freeButton) {
                this.text.element.addClass('wt-default-font');
            }
            items.push(this.text);
        }
        this._createAdditionalItem(items);
        this.autoHorizontal = BI.createWidget({
            type: o.textAlign !== 'center' ? "bi.horizontal" : "bi.horizontal_auto",
            element: this.button,
            scrollx: false,
            lgap: o.horizontalPadding,
            rgap: o.horizontalPadding,
            items: [{
                type: "bi.horizontal",
                items: items
            }]
        });
        if (o.textAlign === 'center') {
            WT.each(items, function (item) {
                var el = item.element;
                if (el instanceof $) {
                    el.addClass('wt-base-button-flex-fix');
                }
            })
        }
        if (o.iconOnly) {
            this.icon && this.icon.element.css('margin-top', (o.height - this._const.iconSize) / 2)
        }
        if (o.textAlign === 'center') {
            this.autoHorizontal._children[0].element.css('justify-content', 'center');
        }
        if (o.disabled) {
            this.element.addClass('wt-disable');
        }
        this.setEnable(!o.disabled);
        this.setVisible(!o.invisible);
        this._bindEvents();
    },

    /**
     * 绑定事件
     * @private
     */
    _bindEvents: function () {
        var self = this, o = this.options;
        var isHover = false;
        this.element.hover(function () {
            //enter
            if (o.disabled) {
                return;
            }
            if (o.bgHover) {
                WT.setBackground(self.element, o.bgHover);
            }
            isHover = true;
        }, function () {
            //leave
            if (o.disabled) {
                return;
            }
            WT.setBackground(self.element, o.bgNormal || '');
            isHover = false;
        });
        this.element.mousedown(function () {
            if (o.disabled) {
                return;
            }
            if (o.bgClick) {
                WT.setBackground(self.element, o.bgClick);
            }
        });
        this.element.mouseup(function () {
            if (o.disabled) {
                return;
            }
            if (o.bgHover) {
                WT.setBackground(self.element, isHover ? o.bgHover : o.bgNormal || '');
            }
            else {
                WT.setBackground(self.element, o.bgNormal || '');
            }
        });
        if (o.shortcutKey) {
            var handleShortcutKey = function (e) {
                if (($(document).codeToString(e) === o.shortcutKey) && self.isEnable()) {
                    var bht = FR.CookieInfor.getCookieByName("buttonHotKeyTimer");
                    var nt = (new Date()).getTime();
                    if (FR.isEmpty(bht) || (nt - bht > 1000)) {
                        if (self.isEnable()) {
                            self.fireEvent(FR.Events.CLICK, e);
                            if (o.bgClick) {
                                WT.setBackground(self.element, o.bgClick);
                                setTimeout(function () {
                                    WT.setBackground(self.element, o.bgNormal || '');
                                }, 200);
                            }
                            document.cookie = "buttonHotKeyTimer=" + nt;
                            e.preventDefault();
                            e.stopPropagation();
                        }
                    }
                }
            };
            WT.Keys.reg(handleShortcutKey);
            self.unRegShortcutKeyEvent = function () {
                WT.Keys.unreg(handleShortcutKey);
            }
        }
    },

    /**
     * 创建额外子元素（一般按钮有图标元素和文字元素，可以在items中push额外元素）
     * @param items 子元素数组
     */
    _createAdditionalItem: function(items) {
    },

    setEnable: function (enable) {
        WT.BaseButton.superclass.setEnable.apply(this, arguments);
        this.button.setEnable(enable);
    },

    /**
     * 设置按钮标签文本
     * @param text
     */
    setText: function (text) {
        if (this.options.iconOnly) {
            return;
        }
        this.options.text = text;
        this.text.setText(text);
    },

    getText: function () {
        return this.options.text;
    },

    setValue: function (v) {
        if (this.options.freeButton && v) {
            this.setText(v);
        }
    },

    getValue: function () {
        return this.getText();
    },

    destroy: function () {
        WT.BaseButton.superclass.destroy.apply(this);
        //注销监听的全局快捷键
        this.unRegShortcutKeyEvent && this.unRegShortcutKeyEvent();
    }

});