/**
 * 新填报下所有带弹出层控件的基类
 * @class WT.BasePopup
 * @extends WT.Widget
 *
 *
 * @cfg {JSON} options 配置属性
 * @cfg {Boolean} options.allowBlank 允许空值
 * @cfg {String} options.errorMsg4Blank 空值时提示信息
 */
WT.BasePopup = FR.extend(WT.Widget, {

    TYPE: 'WT.BasePopup',

    _const: {
        popupMaxHeight: 200,
        popupMaxWidth: 300,
        popupMinHeight: 20,
        iconWidth: 19,
        iconCls: 'css:wt-popup-default-icon'
    },

    _defaultConfig: function () {
        return $.extend(WT.BasePopup.superclass._defaultConfig.apply(), {
            sheetContext: null,
            allowBlank: true,
            errorMsg4Blank: FR.i18nText('Fine-Engine_Write_Enhanc_Not_Support_Null')
        });
    },

    _initVariable: function () {
        WT.BasePopup.superclass._initVariable.apply(this);
        var self = this;
        this.on(WT.BasePopup.EVENT_INSIDE_VALUE_CHANGED, function (value, sender) {
            if (value === undefined) {
                value = '';
            }
            self.options.value = value;
            if (!sender) {
                self._lastValue4StopEditEvent = value;
            }
        });
        this._lastValue4StopEditEvent = undefined;
    },

    _initView: function () {
        var o = this.options, self = this;
        this.wrapper = BI.createWidget({
            type: 'bi.layout'
        });
        this.popup = BI.createWidget({
            type: 'bi.vertical'
        });
        this.popup.element.addClass('wt-scrollbar');
        this.combo = BI.createWidget({
            type: 'bi.combo',
            toggle: true,
            isNeedAdjustHeight: false,
            isNeedAdjustWidth: false,
            adjustLength: 1,
            el: this.wrapper,
            popup: {
                el: this.popup,
                maxHeight: this._const.popupMaxHeight,
                maxWidth: this._const.popupMaxWidth,
                minHeight: this._const.popupMinHeight,
                stopPropagation: false
            }
        });
        this.combo.on(BI.Combo.EVENT_BEFORE_POPUPVIEW, function () {
            o.sheetContext && o.sheetContext.currentPopup(self.combo);
            self._createPopupView();
            self._beforePopupShow();
        });
        this.combo.on(BI.Combo.EVENT_AFTER_HIDEVIEW, function () {
            o.sheetContext && o.sheetContext.currentPopup(null);
            self._afterPopupHide();
            self._clearPopupView();
        });
        this.combo._isMounted = true;

        this.element = this.combo.element;
        this.element.addClass('wt-absolute-layout');
        this.element.css({
            left: o.left,
            top: o.top,
            height: o.height,
            width: o.width
        });

        this.wrapper.element.css({
            height: o.height,
            width: o.width
        });
        WT.dom.appendChild(this.element, this.wrapper.element);

        this.editor = this._createEditor();
        WT.dom.appendChild(this.wrapper.element, this.editor.element);

        this.trigger = new WT.BaseButton({
            height: o.height - 2,
            width: this._const.iconWidth - 1,
            left: o.width - this._const.iconWidth,
            icon: this._const.iconCls,
            iconOnly: true
        });
        this.trigger.element.addClass('wt-trigger-button');
        WT.dom.appendChild(this.wrapper.element, this.trigger.element);

        this.setEnable(!o.disabled);
        this.setVisible(!o.invisible);
        this.editor._blurCheck = this._blurCheck.bind(this.editor, this);

    },

    _blurCheck: function (self) {
        if (self.shouldBlur === true) {
            return true;
        }
        return !(self.isPopupShow && self.isPopupShow());
    },

    _createEditor: function () {
        var o = this.options, self = this;
        var editor = new WT.TextEditor({
            usedAsCell: o.usedAsCell,
            height: o.height,
            width: o.width - this._const.iconWidth,
            watermark: o.watermark,
            allowBlank: o.allowBlank,
            errorMsg4Blank: o.errorMsg4Blank,
            regex: o.regex,
            errorMsg4Regex: o.errorMsg4Regex,
            disabled: o.disabled,
            fontSize: o.fontSize
        });
        editor.on(FR.Events.BEFOREEDIT, function () {
            self.fireEvent(FR.Events.BEFOREEDIT);
        });
        editor.on(FR.Events.AFTEREDIT, function () {
            self.fireEvent(FR.Events.AFTEREDIT);
            self.fireEvent(WT.BasePopup.EVENT_INSIDE_AFTER_EDIT);
        });
        editor.on(FR.Events.STOPEDIT, function () {
            self.fireEvent(WT.BasePopup.EVENT_INSIDE_STOP_EDIT);
            self.fireEvent(FR.Events.STOPEDIT);
            if (editor.isEditManually()) {
                editor.clearEditFlag();
                self.fireEvent(WT.BasePopup.EVENT_INSIDE_VALUE_CHANGED,
                    self._convertEditorValue2Value(editor.getValue()), editor);
            }
        });
        editor.on(WT.BaseEditor.EVENT_ENTER_PRESSED, function () {
            self.fireEvent(WT.BaseEditor.EVENT_ENTER_PRESSED);
        });
        editor.on(WT.BaseEditor.EVENT_TAB_PRESSED, function () {
            self.fireEvent(WT.BaseEditor.EVENT_TAB_PRESSED);
        });
        this.on(WT.BasePopup.EVENT_INSIDE_VALUE_CHANGED, function (value, sender) {
            if (sender === editor) {
                //屏蔽自己发出的事件
                return;
            }
            editor.setValue(value === undefined ? '' : self._convertValue2EditorValue(value));
        });
        return editor;
    },

    /**
     * 创建弹出层内容
     * @protected
     * @return {jQuery}
     */
    _createPopupView: function () {
    },

    /**
     * 清除弹出层内容
     * @protected
     * @return {jQuery}
     */
    _clearPopupView: function () {
    },

    /**
     * 从UI组件的状态生成值
     * @return {null}
     */
    _generateValue: function () {
        return this.editor.getValue();
    },

    /**
     * popup即将显示出来时调用
     * @protected
     */
    _beforePopupShow: function () {
        this.editor.focus();
    },

    /**
     * popup关闭后调用
     * @protected
     */
    _afterPopupHide: function () {
        this.editor.focus();
    },

    _showPopup: function () {
        this.combo.showView();
    },

    _hidePopup: function () {
        this.combo.hideView();
    },

    /**
     * 将中间值转换为编辑器显示值
     * @param val
     * @return {*}
     */
    _convertValue2EditorValue: function(val) {
        return val;
    },

    /**
     * 将编辑器显示值转换为中间值
     * @param val
     * @return {*}
     */
    _convertEditorValue2Value: function(val) {
        return val;
    },

    setValue: function (v) {
        this.fireEvent(WT.BasePopup.EVENT_INSIDE_VALUE_CHANGED, v);
    },

    setEditingValue: function (v) {
        this.editor.setEditingValue && this.editor.setEditingValue(v);
    },

    getValue: function () {
        this.options.value = this._convertEditorValue2Value(this.editor.getValue());
        return this.options.value;
    },


    getText: function () {
        return this.editor.getValue();
    },

    setEnable: function (enable) {
        this.options.disabled = !enable;
        this.editor.setEnable(enable);
        this.trigger.setEnable(enable);
        this.combo.setEnable(enable);
    },

    focus: function (noAutoSelect) {
        this.editor.focus(noAutoSelect);
    },

    blur: function () {
        this.shouldBlur = true;
        this.editor.blur();
    },

    stopEdit: function (isToastError) {
        var r = this.editor.stopEdit(isToastError);
        if (this.options.value !== this._lastValue4StopEditEvent) {
            this._lastValue4StopEditEvent = this.options.value;
            this.fireEvent(FR.Events.STOPEDIT);
        }
        return r;
    },

    couldUsedAsEditor: function () {
        return true;
    },

    setEditorDisplayValue: function (val) {
        this.editor.setEditorDisplayValue(val);
    },
    isPopupShow: function () {
        return this.combo && this.combo.isViewVisible();
    },
    verify: function (cValue) {
        return this._verify(this, cValue);
    },
    _verify: function (self, cValue) {

    }

});
WT.BasePopup.EVENT_INSIDE_VALUE_CHANGED = 'INSIDE_VALUE_CHANGED';
WT.BasePopup.EVENT_INSIDE_AFTER_EDIT = 'EVENT_INSIDE_AFTER_EDIT';
WT.BasePopup.EVENT_INSIDE_STOP_EDIT = 'EVENT_INSIDE_STOP_EDIT';