/**
 * 新填报下的复选框
 * @class WT.CheckBox
 * @extends WT.BaseButton
 *
 * @cfg {JSON} options 配置属性
 * @cfg {Boolean} options.value 控件值
 * @cfg {String} options.text 文本
 */
WT.CheckBox = FR.extend(WT.BaseButton, {

    TYPE: 'WT.CheckBox',

    forceCustomFontConstant: true,

    _defaultConfig: function () {
        return $.extend(WT.CheckBox.superclass._defaultConfig.apply(), {
            value: false,
            icon: 'transparent'
        });
    },

    /**
     * 初始化其他变量
     * @protected
     */
    _initVariable: function () {
        WT.CheckBox.superclass._initVariable.apply(this);
        this.isHalf = false;
    },

    _initView: function () {
        WT.CheckBox.superclass._initView.apply(this);
        var o = this.options, self = this;
        this._setIcon();
        this.on(FR.Events.CLICK, function () {
            if (self.isHalf) {
                o.value = true;
                self.isHalf = false;
            }
            else {
                o.value = !o.value;
            }
            self._setIcon();
            self.fireEvent(FR.Events.STATECHANGE, o.value);
        });
    },

    _setIcon: function () {
        this.icon.element.removeClass('wt-check-box-icon-half');
        this._setIconWithState(this._parseBoolean(this.options.value));
    },
    _parseBoolean: function (state) {
        if (typeof state == 'boolean') {
            return state;
        } else if (state == 'true') {
            return true;
        } else {
            return false;
        }
    },
    _setIconWithState: function (selected) {
        selected = typeof selected == 'boolean'
            ? selected
            : (selected != 'false');
        if (selected) {
            this.icon.element.removeClass('wt-check-box-icon').addClass('wt-check-box-icon-check');
        } else {
            this.icon.element.removeClass('wt-check-box-icon-check').addClass('wt-check-box-icon');
        }
    },

    /**
     * 设置值，默认触发VALUECHANGE事件
     * @param v {Boolean}
     * @param notTrig {Boolean} 为true时不触发VALUECHANGE事件
     */
    setValue: function (v, notTrig) {
        var oldValue = this.options.value;
        v = this._parseBoolean(v);
        this.options.value = v;
        this.isHalf = false;
        this._setIcon();
        if (v !== oldValue) {
            if (notTrig !== true) {
                this.fireEvent(FR.Events.STATECHANGE, v);
            }
        }
    },
    
    getValue: function () {
        return this.options.value;
    },

    selected: function () {
        return this.icon.element.is('.wt-check-box-icon-check');
    },

    /**
     * 设置显示为中间状态，一般用于全选按钮
     */
    setStateHalf: function () {
        this.icon.element.removeClass('wt-check-box-icon');
        this.icon.element.removeClass('wt-check-box-icon-check');
        this.icon.element.removeClass('wt-check-box-icon-half');
        this.icon.element.addClass('wt-check-box-icon-half');
        this.isHalf = true;
    }

});