/**
 * 新填报下下拉框组件
 * @class WT.ComboBox
 * @extends WT.BasePopup
 *
 * @cfg {JSON} options 配置属性
 * @cfg {Boolean} options.allowEdit 允许直接编辑
 * @cfg {Boolean} options.allowCustomValue 允许自定义值
 * @cfg {Number} options.searchTime 搜索延迟时间
 * @cfg {Object} options.dataSource 数据加载器
 */
WT.ComboBox = FR.extend(WT.BasePopup, {

    TYPE: 'WT.ComboBox',

    _const: {
        countPerPage: 100,
        popupMaxWidth: 'auto'
    },

    _defaultConfig: function () {
        return $.extend(WT.ComboBox.superclass._defaultConfig.apply(), {
            allowEdit: true,
            allowCustomValue: true,
            searchTime: 200,
            dataSource: null,
            paraProvider: null
        });
    },

    _initVariable: function () {
        WT.ComboBox.superclass._initVariable.apply(this);
        var o = this.options, self = this;
        this._const = $.extend({}, WT.DateEditor.superclass._const, this._const);
        this.data = null;
        this.cachedDataMap = {};
        this.cachedTextMap = {};
        this.dataSource = o.dataSource || WT.DataSource.EMPTY;
        this.dataSource.on(WT.DataSource.EVENT_DATA_UPDATED, function (newData) {
            if (self.combo.isViewVisible()) {
                //popup在弹出状态
                self._hidePopup();
                self._showPopup();
            }
        });
        this.dataSource.on(WT.DataSource.EVENT_DATA_APPEND, function (dataChunk) {
            self._appendItems(dataChunk);
        });
        this.dataSource.on(WT.DataSource.EVENT_DATA_UPDATE_ERROR, function (err) {
            if (self.combo.isViewVisible()) {
                //popup在弹出状态
                self._hidePopup();
                self._showPopup();
            }
        });
        this.items = [];
    },

    _initView: function () {
        WT.ComboBox.superclass._initView.apply(this);
        var self = this, o = this.options;
        if (!this.options.allowEdit) {
            this.editor.editor.element.attr('readonly', 'readonly');
        }
        //这是一个非常坑的操作,editor 调用_verify的时候强行塞了self进来...因为要用到self的校验属性
        this.editor._verify = this._verify.bind(this.editor, this);
        this.popup.element.addClass('wt-popup');
        this.fireEvent(WT.BasePopup.EVENT_INSIDE_VALUE_CHANGED, this.options.value);

        this.search = '';
        var dealSearch = WT.debounce(function () {
            self.clearCombo();
            self._showPopup();
        }, o.searchTime);
        this.on(WT.BasePopup.EVENT_INSIDE_AFTER_EDIT, function () {
            if (o.allowEdit) {
                dealSearch();
            }
        })
    },
    clearCombo: function () {
        var self = this;
        var editingValue = self.editor.getEditingValue();
        self.search = editingValue;
        self.dataSource.clearData();
        if (self.combo.isViewVisible()) {
            //popup在弹出状态
            self._hidePopup();
        }
    },

    _createPopupView: function () {
        var o = this.options, self = this;
        var loadArgs = {
            widgetName: o.widgetName,
            search: this.search
        };
        if (WT.isNotEmptyObject(o.dependence) && o.paraProvider) {
            var dependence = {};
            for (var k in o.dependence) {
                var paraVal = o.paraProvider.getPara(o.dependence[k]);
                if (!FR.isEmpty(paraVal)) {
                    dependence[WT.isIntString(k) ? o.dependence[k].toUpperCase() : k.toUpperCase()] = paraVal;
                }
            }
            if (!WT.isEmpty(dependence)) {
                loadArgs.dependence = dependence;
            }
        }
        if (!this.dataSource.hasData(JSON.stringify(loadArgs))) {
            this._loadFirstPageData(loadArgs);
            var loadingWrap = WT.dom.createElement('div', 'wt-loading');
            loadingWrap.innerHTML = '<span>' + FR.i18nText('Fine-Engine_Platform_Loading') + '</span>';
            WT.dom.appendChild(this.popup.element, loadingWrap);
            return;
        }
        this.data = WT.copyArray(this.dataSource.getData());
        this.hasChecked = false;   //做个标志变量，数据源有相同值时只允许一个选中
        this.rendingItemsData = this.data;
        if (o.allowBlank) {
            var wrapper = WT.dom.createElement('div');
            var noSelectItem = new WT.ComboButton({
                text: FR.i18nText('Fine-Engine_Write_Enhanc_No_Select'),
                width: '100%',
                textCls: 'wt-combo-box-deselect-text'
            });
            noSelectItem.element.css('position', 'relative');
            noSelectItem.on(FR.Events.CLICK, function () {
                self._hidePopup();
                self.fireEvent(WT.BasePopup.EVENT_INSIDE_VALUE_CHANGED, undefined, noSelectItem);
                self.fireEvent(FR.Events.AFTEREDIT);
                self.clearCombo();
                self.editor.verify4Style();
            });
            WT.dom.appendChild(wrapper, noSelectItem.element);
            WT.dom.appendChild(this.popup.element, wrapper);
        }
        this._createItemsElement(this.rendingItemsData);
        //下拉框最小宽度为editor的宽度
        this.combo.popupView.element.css('min-width',this.options.width);
        this.popup.element.on('scroll', WT.debounce(function () {
            var contentHeight = self.popup.element[0].scrollHeight;
            var wrapperHeight = self.popup.element.height();
            var scrollTop = self.popup.element.scrollTop();
            if (contentHeight - wrapperHeight - scrollTop < 20) {
                self._loadNextPageData(loadArgs);
            }
        }, 200));
    },

    _appendItems: function (chunkData) {
        if (!this.combo.isViewVisible()) {
            return;
        }
        var self = this;
        var lastDataLen = self.data.length;
        this._createItemsElement(chunkData, lastDataLen);
        if (chunkData.length < this._const.countPerPage) {
            this.pageLoadEnd = true;
        }
    },

    _createItemsElement: function (itemsData, lastDataLen) {
        lastDataLen = lastDataLen || 0;
        var self = this, o = this.options;
        var findCheckIdx = function (dataProp) {
            var idx = -1;
            for (var i = 0; i < itemsData.length; i++) {
                var d = itemsData[i];
                var isSel = (d[dataProp] + '' === o.value + '');
                if (isSel) {
                    idx = i;
                    break;
                }
            }
            return idx;
        };
        var checkIdx = findCheckIdx('value');
        if (checkIdx === -1) {
            checkIdx = findCheckIdx('text');
        }
        WT.each(itemsData, function (d, idx) {
            var wrapper = WT.dom.createElement('div');
            var checked = false;
            if (!self.hasChecked && checkIdx === idx) {
                checked = true;
                self.hasChecked = true;
            }
            var item = self._createItemComp(d, checked);
            item.element.css('position', 'relative');
            item.on(FR.Events.CLICK, function () {
                self._onItemClick(idx + lastDataLen, item);
            });
            WT.dom.appendChild(wrapper, item.element);
            WT.dom.appendChild(self.popup.element, wrapper);
            self.items.push(item);
        });
    },

    _loadFirstPageData: function (args) {
        this.pageLoadEnd = false;
        this.pageCount = 0;
        var hash = JSON.stringify(args);
        args = $.extend({
            start: 0,
            limit: this._const.countPerPage
        }, args);
        this.dataSource.loadData(args, hash);
    },

    _loadNextPageData: function (args) {
        if (this.pageLoadEnd) {
            return;
        }
        this.pageCount = this.pageCount || 0;
        this.pageCount++;
        var hash = JSON.stringify(args);
        args = $.extend({
            start: this.pageCount * this._const.countPerPage,
            limit: this._const.countPerPage
        }, args);
        this.dataSource.loadData(args, hash);
    },

    _createItemComp: function (data, checked) {
        var item = new WT.ComboButton({
            text: data.text,
            textCls: checked ? 'wt-combo-box-selected-text' : null,
            width: '100%'
        });
        return item;
    },

    _clearPopupView: function () {
        this.items && WT.each(this.items, function (item) {
            item && item.purgeListeners && item.purgeListeners();
        });
        this.items = [];
        this.popup.element.empty();
        this.popup.element.off('scroll');
    },

    _onItemClick: function (idx, item) {
        this._hidePopup();
        var d = this.data[idx];
        if (d !== undefined && d !== null) {
            this.fireEvent(WT.BasePopup.EVENT_INSIDE_VALUE_CHANGED, d.value, item);
            this.fireEvent(FR.Events.AFTEREDIT);
            this.editor.verify4Style();
        }
        this.fireEvent(FR.Events.CLICK);
    },

    _verify: function (self, cValue) {
        var dv = cValue ? cValue : this.editor.getValue();
        if (FR.isEmpty(dv)) {
            if (self.options.allowBlank === false) {
                this.errMsg = self.options.errorMsg4Blank || FR.i18nText('Fine-Engine_Base_NOT_NULL_Des');
                return false;
            } else {
                return true;
            }
        }
        if (self.options.allowCustomValue === false) {
            if (self.data) {
                for (var i = 0; i < self.data.length; i++) {
                    var d = self.data[i];
                    if (d.text === dv) {
                        return true;
                    }
                    if (d.value === dv) {
                        return true;
                    }
                }
            } else {
                //REPORT-33016 当不存在数据字典时以及不允许自定义输入值,输入数据时,应该不通过校验,原来的直接返回true了
                this.errMsg = FR.i18nText('Fine-Engine_Report_Basic_Not_In_List');
                return false;
            }
            this.errMsg = FR.i18nText('Fine-Engine_Report_Basic_Not_In_List');
            return false;
        }
        return true;
    },

    _convertValue2EditorValue: function(val) {
        return this._getTextByValue(val);
    },

    _convertEditorValue2Value: function(val) {
        return this._getValueByText(val);
    },

    _getTextByValue: function (value) {
        if (this.data) {
            for (var i = 0; i < this.data.length; i++) {
                // 都转换为字符串再比较，比如 551 == '551'，防止这种情况 0 == ''
                if (this.data[i].value + '' === value + '') {
                    return this.data[i].text;
                }
            }
        }
        if (this.cachedDataMap[value] !== undefined) {
            return this.cachedDataMap[value];
        }
        var dic = this.dataSource.loadRawDataSync({
            search: value
        });
        if (dic && dic.length > 0) {
            for (var i = 0; i < dic.length; i++) {
                var t = dic[i];
                if (t.value + '' === value + '') {
                    var r = t.text;
                    this.cachedDataMap[value] = r;
                    this.cachedTextMap[r] = value;
                    return r;
                }
            }
        }
        return value;
    },

    _getValueByText: function (text) {
        if (this.data) {
            for (var i = 0; i < this.data.length; i++) {
                // 都转换为字符串再比较，比如 551 == '551'，防止这种情况 0 == ''
                if (this.data[i].text + '' === text + '') {
                    return this.data[i].value;
                }
            }
        }
        return text;
    },

    setDataArray: function (data) {
        if (WT.isArray(data) && data.length > 0) {
            var first = data[0];
            if (first.text !== undefined && first.value !== undefined) {
                this.data = data;
            }
        }
    },

    getOriginValue: function () {
        return this.options.value;
    },

    getValue: function () {
        return this._getTextByValue(this.options.value);
    },

    setValue: function (v) {
        var val = null;
        if (this.data) {
            for (var i = 0; i < this.data.length; i++) {
                if (this.data[i].value + '' === v + '') {
                    val = v;
                    break;
                }
            }
            if (val === null) {
                val = this._getValueByText(v);
            }
        }
        if (val === null) {
            val = v;
        }
        this.fireEvent(WT.BasePopup.EVENT_INSIDE_VALUE_CHANGED, val);
    }

});
