/**
 * 新填报下下拉复选框组件
 * @class WT.ComboCheckBox
 * @extends WT.BasePopup
 *
 * @cfg {JSON} options 配置属性
 * @cfg {Boolean} options.allowEdit 允许直接编辑
 * @cfg {Boolean} options.allowCustomValue 允许自定义值
 * @cfg {String} options.returnDataType 返回类型 'string' or 'array'
 * @cfg {String} options.returnDataSplit 返回值分隔符 仅返回类型为array时生效
 * @cfg {String} options.returnDataPrefix 返回值起始符 仅返回类型为array时生效
 * @cfg {String} options.returnDataSuffix 返回值结束符 仅返回类型为array时生效
 * @cfg {Boolean} options.showAsTag 标签形式显示
 * @cfg {Number} options.searchTime 搜索延迟时间
 * @cfg {Object} options.dataSource 数据加载器
 */
WT.ComboCheckBox = FR.extend(WT.BasePopup, {

    TYPE: 'WT.ComboCheckBox',

    _const: {
        popupMaxHeight: 200,
        countPerPage: 100,
        popupMaxWidth: 'auto'
    },

    _defaultConfig: function () {
        return $.extend(WT.ComboCheckBox.superclass._defaultConfig.apply(), {
            allowEdit: true,
            allowCustomValue: true,
            returnDataType: 'array',
            returnDataSplit: ',',
            returnDataPrefix: '',
            returnDataSuffix: '',
            showAsTag: true,
            searchTime: 200,
            dataSource: null,
            paraProvider: null
        });
    },

    _initVariable: function () {
        WT.ComboCheckBox.superclass._initVariable.apply(this);
        var o = this.options, self = this;
        this._const = $.extend({}, WT.DateEditor.superclass._const, this._const);
        this.data = null;
        this.cachedDataMap = {};
        this.cachedTextMap = {};
        this.dataSource = o.dataSource || WT.DataSource.EMPTY;
        this.dataSource.on(WT.DataSource.EVENT_DATA_UPDATED, function (newData) {
            if (self.combo.isViewVisible()) {
                //popup在弹出状态
                self._hidePopup();
                self._showPopup();
            }
        });
        this.dataSource.on(WT.DataSource.EVENT_DATA_APPEND, function (dataChunk, args) {
            self._appendItems(dataChunk, args);
        });
        this.dataSource.on(WT.DataSource.EVENT_DATA_UPDATE_ERROR, function (err) {
            if (self.combo.isViewVisible()) {
                //popup在弹出状态
                self._hidePopup();
                self._showPopup();
            }
        });
        this.items = [];
        this.originValue = [];
        this.quickEditValue = this.options.value;
        this.on(WT.BasePopup.EVENT_INSIDE_VALUE_CHANGED, function (value) {
            self.quickEditValue = value;
        });
    },

    _initView: function () {
        WT.ComboCheckBox.superclass._initView.apply(this);
        var self = this, o = this.options;
        if (!this.options.allowEdit) {
            this.editor.editor.element.attr('readonly', 'readonly');
        }
        this.editor._verify = this._verify.bind(this.editor, this);
        this.popup.element.addClass('wt-popup');
        if (!FR.isEmpty(this.options.value)) {
            this.fireEvent(WT.BasePopup.EVENT_INSIDE_VALUE_CHANGED, this._texts2Values(this._assertValueIsArray(this.options.value)));
        }

        this.search = '';
        var dealSearch = WT.debounce(function () {
            var editingValue = self.editor.getEditingValue();
            self.search = editingValue;
            self.dataSource.clearData();
            if (self.combo.isViewVisible()) {
                //popup在弹出状态
                self._hidePopup();
            }
            self._showPopup();
        }, o.searchTime);
        this.on(WT.BasePopup.EVENT_INSIDE_AFTER_EDIT, function () {
            if (o.allowEdit) {
                dealSearch();
            }
        });
    },

    _createEditor: function () {
        var o = this.options, self = this;
        if (o.showAsTag) {
            //需要以标签形式显示
            var editor = new WT.TagEditor({
                usedAsCell: o.usedAsCell,
                height: o.height,
                width: o.width - this._const.iconWidth,
                watermark: o.watermark,
                allowBlank: o.allowBlank,
                errorMsg4Blank: o.errorMsg4Blank,
                regex: o.regex,
                errorMsg4Regex: o.errorMsg4Regex,
                disabled: o.disabled,
                fontSize: o.fontSize
            });
            editor.on(FR.Events.BEFOREEDIT, function () {
                self.fireEvent(FR.Events.BEFOREEDIT);
            });
            editor.on(FR.Events.AFTEREDIT, function () {
                self.fireEvent(FR.Events.AFTEREDIT);
                self.fireEvent(WT.BasePopup.EVENT_INSIDE_AFTER_EDIT);
            });
            editor.on(WT.TagEditor.EVENT_TAG_DELETE, function () {
                var deleted = editor.getDeletedTagIndexes();
                if (deleted.length > 0) {
                    self._handleDeletedTag(deleted);
                }
            });
            editor.on(WT.BaseEditor.EVENT_ENTER_PRESSED, function () {
                self._hidePopup();
                self.fireEvent(WT.BaseEditor.EVENT_ENTER_PRESSED);
            });
            editor.on(WT.BaseEditor.EVENT_TAB_PRESSED, function () {
                self._hidePopup();
                self.fireEvent(WT.BaseEditor.EVENT_TAB_PRESSED);
            });
            this.on(WT.BasePopup.EVENT_INSIDE_VALUE_CHANGED, function (value, sender) {
                if (sender === editor || sender === 'tag') {
                    //屏蔽自己发出的事件
                    return;
                }
                editor.editor.setValue('');
                editor.setTagValue(self._values2Texts(value));
            });
            return editor;
        }
        else {
            var editor = new WT.TextEditor({
                usedAsCell: o.usedAsCell,
                height: o.height,
                width: o.width - this._const.iconWidth,
                watermark: o.watermark,
                allowBlank: o.allowBlank,
                errorMsg4Blank: o.errorMsg4Blank,
                regex: o.regex,
                errorMsg4Regex: o.errorMsg4Regex,
                disabled: o.disabled,
                fontSize: o.fontSize
            });
            editor.on(FR.Events.BEFOREEDIT, function () {
                self.fireEvent(FR.Events.BEFOREEDIT);
            });
            editor.on(FR.Events.AFTEREDIT, function () {
                self.fireEvent(FR.Events.AFTEREDIT);
                self.fireEvent(WT.BasePopup.EVENT_INSIDE_AFTER_EDIT);
            });
            editor.on(FR.Events.STOPEDIT, function () {
                if (editor.isEditManually()) {
                    editor.clearEditFlag();
                    //把value由 'a,b,c,d' 转为 ['a', 'b', 'c', 'd']
                    var v = WT.map(editor.getValue().split(','), function (str) {
                        return WT.trim(str);
                    });
                    var textUniq = WT.uniq(v);
                    if (v.length !== textUniq.length) {
                        editor.setValue(textUniq.join(','));
                    }
                    self.fireEvent(WT.BasePopup.EVENT_INSIDE_VALUE_CHANGED, self._texts2Values(textUniq), editor);
                }
            });
            this.on(WT.BasePopup.EVENT_INSIDE_VALUE_CHANGED, function (value, sender) {
                if (sender === editor) {
                    //屏蔽自己发出的事件
                    return;
                }
                //把value由 ['a', 'b', 'c', 'd'] 转为 'a,b,c,d'
                editor.setValue(WT.reduce(self._values2Texts(value), function (prev, curr) {
                    if (prev) {
                        return prev + ',' + curr;
                    }
                    return curr;
                }, null));
            });
            editor.on(WT.BaseEditor.EVENT_ENTER_PRESSED, function () {
                self._hidePopup();
                self.fireEvent(WT.BaseEditor.EVENT_ENTER_PRESSED);
            });
            editor.on(WT.BaseEditor.EVENT_TAB_PRESSED, function () {
                self._hidePopup();
                self.fireEvent(WT.BaseEditor.EVENT_TAB_PRESSED);
            });
            return editor;
        }
    },

    _createPopupView: function () {
        var o = this.options, self = this;
        var loadArgs = {
            widgetName: o.widgetName,
            search: this.search
        };
        if (WT.isNotEmptyObject(o.dependence) && o.paraProvider) {
            var dependence = {};
            for (var k in o.dependence) {
                var paraVal = o.paraProvider.getPara(o.dependence[k]);
                if (!FR.isEmpty(paraVal)) {
                    dependence[WT.isIntString(k) ? o.dependence[k].toUpperCase() : k.toUpperCase()] = paraVal;
                }
            }
            if (!WT.isEmpty(dependence)) {
                loadArgs.dependence = dependence;
            }
        }
        if (!this.dataSource.hasData(JSON.stringify(loadArgs))) {
            this._loadFirstPageData(loadArgs);
            var loadingWrap = WT.dom.createElement('div', 'wt-loading');
            loadingWrap.innerHTML = '<span>' + FR.i18nText('Fine-Engine_Platform_Loading') + '</span>';
            WT.dom.appendChild(this.popup.element, loadingWrap);
            return;
        }
        this.data = WT.copyArray(this.dataSource.getData());
        var wrapper = WT.dom.createElement('div');
        var selectAllItem = this._createItemComp({
            text: FR.i18nText('Fine-Engine_Write_Enhanc_Select_All'),
            cls: 'wt-combo-check-box-item-select-all'
        }, false);
        selectAllItem.element.css('position', 'relative');
        selectAllItem.on(FR.Events.STATECHANGE, function (checked) {
            var fn = function () {
                for (var i = 0; i < self.items.length; i++) {
                    var item = self.items[i];
                    item.setValue(checked, true);
                }
                self.fireEvent(WT.BasePopup.EVENT_INSIDE_VALUE_CHANGED, self._getItemsValue(), selectAllItem);
                self.fireEvent(FR.Events.CLICK);
                self.fireEvent(FR.Events.AFTEREDIT);
                self.fireEvent(WT.BasePopup.EVENT_INSIDE_AFTER_EDIT);
                self.editor.verify4Style();
            };
            if (!self.pageLoadEnd) {
                // 没有加载完数据的话就先全部加载
                self.appendAfterHook = fn;
                self._loadAllPageData(loadArgs);
            } else {
                fn();
            }
        });
        WT.dom.appendChild(wrapper, selectAllItem.element);
        WT.dom.appendChild(this.popup.element, wrapper);
        this.selectAllItem = selectAllItem;
        this.rendingItemsData = this.data;
        this._createItemsElement(this.rendingItemsData);
        //下拉框最小宽度为editor的宽度
        this.combo.popupView.element.css('min-width',this.options.width);
        this._setSelectAllItemState();
        this.popup.element.on('scroll', WT.debounce(function () {
            var contentHeight = self.popup.element[0].scrollHeight;
            var wrapperHeight = self.popup.element.height();
            var scrollTop = self.popup.element.scrollTop();
            if (contentHeight - wrapperHeight - scrollTop < 20) {
                self._loadNextPageData(loadArgs);
            }
        }, 200));
    },

    _appendItems: function (chunkData, args) {
        if (!this.combo.isViewVisible()) {
            return;
        }
        var lastDataLen = this.data.length;
        this._createItemsElement(chunkData, lastDataLen);
        this.data = this.data.concat(chunkData);
        if (chunkData.length < args.limit) {
            this.pageLoadEnd = true;
        }
        if (this.appendAfterHook) {
            WT.nextTick(function () {
                this.appendAfterHook && this.appendAfterHook.call(this);
                this.appendAfterHook = undefined
            }, this);
        }
    },

    _createItemsElement: function (itemsData, lastDataLen) {
        lastDataLen = lastDataLen || 0;
        var self = this, o = this.options;
        var findCheckIdx = function (dataProp, selectAll) {
            var idxes = [];
            for (var i = 0; i < itemsData.length; i++) {
                var d = itemsData[i];
                var checked = WT.arrayContains(o.value, d[dataProp]);
                if (checked) {
                    idxes.push(i);
                    if (!selectAll) {
                        break;
                    }
                }
            }
            return idxes;
        };
        var checkIdxes = findCheckIdx('value', true);
        if (checkIdxes.length === 0) {
            checkIdxes = findCheckIdx('text');
        }
        WT.each(itemsData, function (d, idx) {
            var wrapper = WT.dom.createElement('div');
            var checked = checkIdxes.indexOf(idx) >= 0;
            var item = self._createItemComp(d, checked);
            item.element.css('position', 'relative');
            item.on(FR.Events.STATECHANGE, function (checked) {
                self._onItemClick(idx + lastDataLen, checked, item);
            });
            WT.dom.appendChild(wrapper, item.element);
            WT.dom.appendChild(self.popup.element, wrapper);
            self.items.push(item);
        });
    },

    _loadFirstPageData: function (args) {
        this.pageLoadEnd = false;
        this.pageCount = 0;
        var hash = JSON.stringify(args);
        args = $.extend({
            start: 0,
            limit: this._const.countPerPage
        }, args);
        this.dataSource.loadData(args, hash);
    },

    _loadNextPageData: function (args) {
        if (this.pageLoadEnd) {
            return;
        }
        this.pageCount = this.pageCount || 0;
        this.pageCount++;
        var hash = JSON.stringify(args);
        args = $.extend({
            start: this.pageCount * this._const.countPerPage,
            limit: this._const.countPerPage
        }, args);
        this.dataSource.loadData(args, hash);
    },

    _loadAllPageData: function (args) {
        if (this.pageLoadEnd) {
            return;
        }
        this.pageCount = this.pageCount || 0;
        this.pageCount++;
        var hash = JSON.stringify(args);
        args = $.extend({
            start: this.pageCount * this._const.countPerPage,
            limit: 1 << 30
        }, args);
        this.dataSource.loadData(args, hash);
    },

    _createItemComp: function (opts, checked) {
        var item = new WT.CheckBox({
            text: opts.text,
            value: checked,
            width: '100%',
            textAlign: 'left',
            horizontalPadding: 5
        });
        opts.cls && item.element.addClass(opts.cls);
        return item;
    },

    _clearPopupView: function () {
        this.items && WT.each(this.items, function (item) {
            item && item.purgeListeners && item.purgeListeners();
        });
        this.items = [];
        this.selectAllItem && this.selectAllItem.purgeListeners();
        this.selectAllItem = null;
        this.popup.element.empty();
        this.popup.element.off('scroll');
    },

    _onItemClick: function (idx, checked, item) {
        this._setSelectAllItemState();
        var newVal = null;
        if (!this.dataSource.getArgs().search || !this.options.showAsTag) {
            newVal = this._getItemsValue();
        } else {
            newVal = WT.copyArray(this.options.value);
            var v = this.data[idx];
            if (checked) {
                if (v) {
                    newVal.push(v.value);
                }
            } else {
                var rIdx = newVal.indexOf(v.value);
                if (rIdx !== -1) {
                    WT.arrayPull(newVal, rIdx);
                }
            }
        }
        this.fireEvent(WT.BasePopup.EVENT_INSIDE_VALUE_CHANGED, newVal, item);
        this.fireEvent(FR.Events.CLICK);
        this.fireEvent(FR.Events.AFTEREDIT);
        this.focus();
    },

    _getItemsValue: function () {
        if (!this.data) {
            return [];
        }
        var value = [];
        for (var i = 0; i < this.items.length; i++) {
            var item = this.items[i];
            if (item.getValue()) {
                value.push(this.data[i].value);
            }
        }
        return value;
    },

    _setSelectAllItemState: function () {
        if (this._isAllChecked()) {
            this.selectAllItem.setValue(true, true);
        }
        else if (this._isAllUnckecked()) {
            this.selectAllItem.setValue(false, true);
        }
        else {
            this.selectAllItem.setStateHalf();
        }
    },

    _isAllChecked: function () {
        for (var i = 0; i < this.items.length; i++) {
            var item = this.items[i];
            if (!item.getValue()) {
                return false;
            }
        }
        return true;
    },

    _isAllUnckecked: function () {
        for (var i = 0; i < this.items.length; i++) {
            var item = this.items[i];
            if (item.getValue()) {
                return false;
            }
        }
        return true;
    },

    _texts2Values: function (texts) {
        var self = this;
        var map = {};
        if (this.data) {
            map = this._generateKeyIndexMap(this.data, 'text');
        }
        return WT.map(texts, function (text) {
            var idxes = map[text + ''];
            var v = text;
            if (idxes) {
                v = self.data[idxes[0]].value;
                self.cachedTextMap[text] = v;
            } else {
                if (self.cachedTextMap[text] !== undefined) {
                    v = self.cachedTextMap[text];
                } else {
                    var dic = self.dataSource.loadRawDataSync({
                        search: text
                    });
                    if (dic && dic.length > 0) {
                        for (var i = 0; i < dic.length; i++) {
                            var t = dic[i];
                            if (t.text + '' === text + '') {
                                v = t.value;
                                self.cachedTextMap[text] = v;
                                break;
                            }
                        }
                    }
                }
            }
            return v;
        });
    },

    _values2Texts: function (values) {
        var self = this;
        var map = {};
        if (this.data) {
            map = this._generateKeyIndexMap(this.data, 'value');
        }
        return WT.map(values, function (value) {
            var idxes = map[value + ''];
            var r = value;
            if (idxes) {
                r = self.data[idxes[0]].text;
                self.cachedDataMap[value] = r;
                self.cachedTextMap[r] = value;
            } else {
                if (self.cachedDataMap[value] !== undefined) {
                    r = self.cachedDataMap[value];
                } else {
                    var dic = self.dataSource.loadRawDataSync({
                        search: value
                    });
                    if (dic && dic.length > 0) {
                        for (var i = 0; i < dic.length; i++) {
                            var t = dic[i];
                            if (t.value + '' === value + '') {
                                r = t.text;
                                self.cachedDataMap[value] = r;
                                self.cachedTextMap[r] = value;
                                break;
                            }
                        }
                    }
                }
            }
            return r;
        });
    },

    _verify: function (self, cValue) {
        var verifyValue = cValue == null ? this.editor.editor.getValue() : cValue;
        //dv: 结果的text集合
        var dv = verifyValue ? self._assertValueIsArray(verifyValue) : self._values2Texts(self.options.value);
        if (dv.length === 0) {
            if (self.options.allowBlank === false) {
                this.errMsg = self.options.errorMsg4Blank || FR.i18nText('Fine-Engine_Base_NOT_NULL_Des');
                return false;
            } else {
                return true;
            }
        }
        if (self.options.allowCustomValue === false) {
            if (self.data) {
                var dataTexts = WT.map(self.data, function (d) {
                    return d.text + '';   // 统一转为字符串比较
                });
                //判断当前值如果有任何一个不在datasource中，就校验失败
                for (var i = 0; i < dv.length; i++) {
                    var dvi = dv[i] + '';   // 统一转为字符串比较
                    if (!WT.arrayContains(dataTexts, dvi) && self.cachedTextMap[dv[i]] === undefined) {
                        this.errMsg = FR.i18nText('Fine-Engine_Report_Basic_Not_In_List');
                        return false;
                    }
                }
            }
            else {
                //REPORT-33016 当不存在数据字典时以及不允许自定义输入值,输入数据时,应该不通过校验,下拉复选框没有做这类判断,会直接返回true
                this.errMsg = FR.i18nText('Fine-Engine_Report_Basic_Not_In_List');
                return false;
            }
        }
        return true;
    },

    _assertValueIsArray: function (value) {
        var o = this.options;
        if (WT.isArray(value)) {
            return value;
        }
        if (WT.isNumber(value)) {
            value = value + '';
        }
        if (WT.isString(value) && value.length > 0) {
            if (value.indexOf(o.returnDataPrefix) === 0) {
                //去掉前缀
                value = value.substr(o.returnDataPrefix.length);
            }
            if (value.indexOf(o.returnDataSuffix) === value.length - o.returnDataSuffix.length) {
                //去掉后缀
                value = value.substr(0, value.length - o.returnDataSuffix.length);
            }
            //分割成数组
            return WT.map(value.split(o.returnDataSplit), function (v) {
                return WT.trim(v);
            });
        }
        return [];
    },

    _generateKeyIndexMap: function (dataList, prop) {
        var m = {};
        WT.each(dataList, function (item, idx) {
            var p = item[prop];
            if (p) {
                var ps = p + '';
                if (m[ps]) {
                    m[ps].push(idx);
                } else {
                    m[ps] = [idx];
                }
            }
        });
        return m;
    },

    _handleDeletedTag: function (deletedTagIndexes) {
        var arrVal = WT.copyArray(this.options.value);
        WT.each(deletedTagIndexes, function (idx) {
            WT.arrayPull(arrVal, idx);
        });
        this.fireEvent(WT.BasePopup.EVENT_INSIDE_VALUE_CHANGED, arrVal, 'tag');
    },

    _afterPopupHide: function () {
    },

    setDataArray: function (data) {
        if (WT.isArray(data) && data.length > 0) {
            var first = data[0];
            if (first.text !== undefined && first.value !== undefined) {
                this.data = data;
            }
        }
    },

    setValue: function (v) {
        var arrVal = this._assertValueIsArray(v);
        var self = this;
        if (this.data) {
            var valueMap = this._generateKeyIndexMap(this.data, 'value');
            var textMap = this._generateKeyIndexMap(this.data, 'text');
            arrVal = WT.map(arrVal, function (itemVal) {
                var rVal = null;
                var valueIdxes = valueMap[itemVal + ''];
                if (valueIdxes) {
                    rVal = itemVal;
                }
                if (rVal === null) {
                    var textIdxes = textMap[itemVal + ''];
                    if (textIdxes) {
                        rVal = self.data[textIdxes[0]].value;
                    }
                }
                if (rVal === null) {
                    rVal = itemVal;
                }
                return rVal;
            })
        }
        this.fireEvent(WT.BasePopup.EVENT_INSIDE_VALUE_CHANGED, arrVal);
    },

    getOriginValue: function () {
        var o = this.options;
        var returnData;
        if (o.returnDataType === 'array') {
            returnData = WT.copyArray(o.value);
        }
        else {
            returnData = '' + o.returnDataPrefix;
            var valueList = WT.copyArray(o.value);
            returnData = WT.reduce(valueList, function (prev, curr, idx) {
                var sp = '';
                if (idx > 0) {
                    sp = o.returnDataSplit;
                }
                return prev + sp + curr;
            }, returnData);
            returnData += o.returnDataSuffix;
        }
        return returnData;
    },

    getValue: function () {
        return this.getText();
    },

    getText: function () {
        var o = this.options;
        var returnData;
        if (o.returnDataType === 'array') {
            returnData = this._values2Texts(WT.copyArray(o.value));
        }
        else {
            returnData = '' + o.returnDataPrefix;
            var valueList = this._values2Texts(WT.copyArray(o.value));
            returnData = WT.reduce(valueList, function (prev, curr, idx) {
                var sp = '';
                if (idx > 0) {
                    sp = o.returnDataSplit;
                }
                return prev + sp + curr;
            }, returnData);
            returnData += o.returnDataSuffix;
        }
        return returnData;
    },

    getQuickEditValue: function () {
        return this.quickEditValue;
    },

    stopEdit: function (isToastError) {
        var o = this.options;
        if (o.showAsTag) {
            if(!this.editor.stopEdit(isToastError)) {
                return false;
            }
            var editorValue = this.editor.editor.getValue();
            if (editorValue) {
                var v = WT.copyArray(this.options.value);
                v.push(this._texts2Values([editorValue])[0]);
                this.fireEvent(WT.BasePopup.EVENT_INSIDE_VALUE_CHANGED, v, 'stopedit');
            }
        }
        this.fireEvent(FR.Events.STOPEDIT);
        return true;
    }

});