/**
 * 日期控件
 * @class WT.DateEditor
 *
 * @cfg {JSON} options 配置属性
 * @cfg {String} options.format 格式
 * @cfg {Boolean} options.customFormat 自定义格式
 * @cfg {String} options.startDate 开始日期
 * @cfg {String} options.endDate 结束日期
 * @cfg {Boolean} options.allowEdit 是否支持手动输入
 * @cfg {Boolean} options.returnDate 是否返回日期
 */
WT.DateEditor = FR.extend(WT.BasePopup, {

    TYPE: 'WT.DateEditor',

    _const: {
        popupMaxHeight: 400,
        iconCls: 'css:wt-date-editor-icon'
    },

    _defaultConfig: function () {
        return $.extend(WT.DateEditor.superclass._defaultConfig.apply(), {
            format: 'yyyy-MM-dd',
            customFormat: false,
            startDate: null,
            endDate: null,
            allowEdit: true,
            returnDate: false
        });
    },

    /**
     * 初始化其他变量
     * @protected
     */
    _initVariable: function () {
        WT.DateEditor.superclass._initVariable.apply(this);
        var o = this.options, self = this;
        this._const = $.extend({}, WT.DateEditor.superclass._const, this._const);
        this.viewMode = this._initViewMode();
        this.std = this._createStartDate(o.startDate, o.format, this.viewMode);
        this.edd = this._createEndDate(o.endDate, o.format, this.viewMode);
        this.data = null;
        this.dataSource = o.dataSource || WT.DataSource.EMPTY;
        this.dataSource.on(WT.DataSource.EVENT_DATA_UPDATED, function (newData) {
            if (self.combo.isViewVisible()) {
                //popup在弹出状态
                self._hidePopup();
                self._showPopup();
            }
        });
    },

    _initView: function () {
        WT.DateEditor.superclass._initView.apply(this);
        var o = this.options;
        if (!this.options.allowEdit) {
            this.editor.editor.element.attr('readonly', 'readonly');
        }
        this.editor._verify = this._verify.bind(this.editor, this);
        var val = this._convertValue(this.options.value);
        this.fireEvent(WT.BasePopup.EVENT_INSIDE_VALUE_CHANGED, val);
    },
    _createPopupView: function () {
        var self = this, o = this.options;
        //这些都是为了在加载日期的时候获取依赖的参数
        var loadArgs = {
            widgetName: o.widgetName
        };
        if (WT.isNotEmptyObject(o.dependence) && o.paraProvider) {
            var dependence = {};
            for (var k in o.dependence) {
                var paraVal = o.paraProvider.getPara(o.dependence[k]);
                if (!FR.isEmpty(paraVal)) {
                    dependence[WT.isIntString(k) ? o.dependence[k].toUpperCase() : k.toUpperCase()] = paraVal;
                }
            }
            if (!WT.isEmpty(dependence)) {
                loadArgs.dependence = dependence;
            }
        }
        if (!this.dataSource.hasData(JSON.stringify(loadArgs))) {
            this._loadDateData(loadArgs);
            var loadingWrap = WT.dom.createElement('div', 'wt-loading');
            loadingWrap.innerHTML = '<span>' + FR.i18nText('Fine-Engine_Platform_Loading') + '</span>';
            WT.dom.appendChild(this.popup.element, loadingWrap);
            return;
        }
        this.data = WT.copyArray(this.dataSource.getData());
        //根据参数依赖重新设置起始时间和终止时间.
        if (this.data[0]) {
            if (this.data[0].startDate) {
                var sd = new Date(this.data[0].startDate);
                if (!FR.isInvalidDate(sd)) {
                    if(!FR.isEmpty(o.format)){
                        o.startDate = FR.date2Str(sd, o.format);
                    }else {
                        o.startDate = FR.date2Str(sd, 'yyyy-MM-dd');
                    }
                    this.std = this._createStartDate(o.startDate, o.format, this.viewMode);
                }
            }
            if (this.data[0].endDate) {
                var ed = new Date(this.data[0].endDate);
                if (!FR.isInvalidDate(ed)) {
                    if(!FR.isEmpty(o.format)){
                        o.endDate = FR.date2Str(ed, o.format);
                    } else {
                        o.endDate = FR.date2Str(ed, 'yyyy-MM-dd');
                    }
                    this.edd = this._createEndDate(o.endDate, o.format, this.viewMode);
                }
            }
        }
        this.datepicker = new WT.DatePicker({
            renderEl: this.popup.element,
            viewMode: this.viewMode,
            date: FR.str2Date(this.editor.getValue(), o.format),
            dateFormat: o.format,
            startDate: this.std,
            endDate: this.edd,
            onDateUpdate:function(){
                self.fireEvent(WT.BasePopup.EVENT_INSIDE_VALUE_CHANGED, FR.date2Str(this.getValue(), o.format), self.datepicker);
                self.fireEvent(FR.Events.AFTEREDIT);
            },
            onClear: function () {
                self._hidePopup();
            },
            onOK: function () {
                self._hidePopup();
            },
            onClose: function () {
                self._hidePopup();
            },
            onToday: function () {
                self._hidePopup();
            }
        });
    },

    _clearPopupView: function () {
        this.popup.element.empty();
    },

    _loadDateData: function (args) {
        var hash = JSON.stringify(args);
        this.dataSource.loadData(args, hash);
    },

    _initViewMode: function(){
        var o = this.options;
        var viewMode = 1;//年月日
        if (o.format.match(/[YyDd]/) && o.format.match(/[Hh]/)) {
            viewMode = 3;//年月日时分秒
        } else if (o.format.match(/[M]/) && !o.format.match(/[Dd]/) && !o.format.match(/[Hh]/)) {
            viewMode = 0;//年月
        } else if (!o.format.match(/[YyMDd]/)) {
            viewMode = 2;//时分秒
        }
        return viewMode;
    },

    _createStartDate: function(st, fmt, viewMode){
        if(FR.isEmpty(st)){
            return null;
        }
        var std = FR.str2Date(st, fmt);
        switch(viewMode){
            case 0:
                return new Date(std.getFullYear(), std.getMonth(), 1, 0, 0, 0);

            case 1:
                return new Date(std.getFullYear(), std.getMonth(), std.getDate(), 0, 0, 0);
            case 2:
                var today = new Date();
                today.setHours(0);
                today.setMinutes(0);
                today.setSeconds(0);
                return today;
            case 3:
                return new Date(std.getFullYear(), std.getMonth(), std.getDate(), 0, 0, 0);
            default:
                return std;
        }
    },

    _createEndDate: function(ed, fmt, viewMode){
        if(FR.isEmpty(ed)){
            return null;
        }
        var edd = FR.str2Date(ed, fmt);
        switch(viewMode){
            case 0:
                return new Date(edd.getFullYear(), edd.getMonth(), edd.getMonthDays(), 23, 59, 59);
            case 1:
                return new Date(edd.getFullYear(), edd.getMonth(), edd.getDate(), 23, 59, 59);
            case 2:
                var today = new Date();
                today.setHours(23);
                today.setMinutes(59);
                today.setSeconds(59);
                return today;
            case 3:
                return new Date(edd.getFullYear(), edd.getMonth(), edd.getDate(), 23, 59, 59);
            default:
                return edd;
        }
    },
    _verify: function (self, cValue) {
        var format = self.options.format;
        var startDate = self.std;
        var endDate = self.edd;
        var matchFormat = false;
        var custom = self.options.customFormat;
        var dv = cValue != null ? cValue : this.editor.editor.getValue();
        if (self.options.allowBlank !== false && FR.isEmpty(dv)) {
            matchFormat = true;
            return matchFormat;
        }
        var dt = FR.matchDateFormat(dv, format);
        if (!custom && !dt) {
            this.errMsg = FR.i18nText("Fine-Engine_Please_Input_With_Right_Format") + ":" + format;
        } else if (startDate && !FR.isInvalidDate(new Date(startDate)) && dt < new Date(startDate)) {
            this.errMsg = FR.i18nText("Fine-Engine_Report_Err_The_Number_Is_Less_Than_The_Minimum_Value") + FR.date2Str(startDate, 'yyyy-MM-dd');
        } else if (endDate && !FR.isInvalidDate(new Date(endDate)) && dt > new Date(endDate)) {
            this.errMsg = FR.i18nText("Fine-Engine_Report_Err_The_Number_Is_Larger_Than_The_Maximum_Value") + FR.date2Str(endDate, 'yyyy-MM-dd');
        } else if (dt > new Date(2999,11,31) || dt < new Date(1900,0,1)) {
            this.errMsg = FR.i18nText("Fine-Engine_Report_Err_The_Time_Out_Of_Range");
        } else {
            matchFormat = true;
        }
        return matchFormat;
    },

    _convertValue: function (val) {
        var v = val;
        if (v instanceof Date) {
            v = FR.date2Str(v, this.options.format);
        } else if (WT.isObject(v) && v.date_milliseconds) {
            v = FR.date2Str(new Date(v.date_milliseconds), this.options.format);
        } else {
            if (WT.isNumber(v)) {
                v = v + '';
            }
            if (WT.isString(v) && !WT.isEmpty(v)) {
                var valid = this._verify(this, val);
                if (valid) {
                    v = FR.str2Date(v, this.options.format) || '';
                    if (v) {
                        v = FR.date2Str(v, this.options.format);
                    }
                }
            }
        }
        return v;
    },

    setValue: function (v) {
        this.fireEvent(WT.BasePopup.EVENT_INSIDE_VALUE_CHANGED, this._convertValue(v));
    },

    getValue: function () {
        var format = this.options.format;
        var value = this.options.value;
        var date = FR.str2Date(value, format);
        if (!this.options.returnDate) {
            return !date ? "" : value;
        }
        return !date ? "" : date;
    },

    getText: function () {
        var format = this.options.format;
        var value = this.options.value;
        var date = FR.str2Date(value, format);
        return !date ? "" : value;
    }
});

WT.DatePicker = FR.extend(FR.DatePicker, {

    /**
     * 生成年月选择部分
     * @private
     */
    _createMonthPicker: function () {
        var table = WT.DatePicker.superclass._createMonthPicker.apply(this, arguments);
        var o = this.options;
        if (o.viewMode === this.CONSTS.VIEWMODE.YM) {
            var _hide = table.hide;
            table.hide = function () {
                var res = _hide.apply(table, arguments);
                o.onOK();
                return res;
            };
        }
        return table;
    }

});
