/**
 * 新填报下的标签控件
 * @class WT.Label
 * @extends WT.Widget
 *
 * @cfg {JSON} options 配置属性
 * @cfg {Boolean} options.autoWrap 自动换行
 * @cfg {Boolean} options.centerInVertical 垂直居中
 * @cfg {String} options.vAlign 垂直对齐方式 top/center/bottom
 * @cfg {String} options.align 对齐方式 left/center/right
 * @cfg {String} options.fontFamily 字体
 * @cfg {String} options.fontWeight 字体粗细
 * @cfg {String} options.fontStyle 字体样式
 * @cfg {Number} options.fontSize 字体大小
 * @cfg {Boolean} options.fontShadow 字体阴影
 * @cfg {String} options.decoration 字体修饰（下划线等样式）
 * @cfg {String} options.color 字体颜色
 * @cfg {Boolean} options.fontSubscript 字体下标
 * @cfg {Boolean} options.fontSuperscript 字体上标
 */
WT.Label = FR.extend(WT.Widget, {

    TYPE: 'WT.Label',

    _defaultConfig: function () {
        return $.extend(WT.Label.superclass._defaultConfig.apply(), {
            value: '',
            autoWrap: true,
            centerInVertical: true,
            vAlign: null,
            align: 'center',
            fontFamily: null,
            fontWeight: null,
            fontStyle: null,
            fontSize: 12,
            fontShadow: false,
            decoration: null,
            color: 'black',
            fontSubscript: false,
            fontSuperscript: false
        });
    },

    _initView: function () {
        WT.Label.superclass._initView.apply(this);
        var o = this.options, self = this;
        this.element = WT.dom.createJQueryElement('div', 'wt-absolute-layout');
        this.element.css({
            left: o.left,
            top: o.top,
            height: o.height,
            width: o.width,
            overflow: 'hidden'
        });

        this.wrapper = WT.dom.createJQueryElement('table').attr('cellpadding', '0').attr('cellspacing', '0');
        this.$tr = WT.dom.createJQueryElement('tr');
        this.$td = WT.dom.createJQueryElement('td');
        this.wrapper.css({
            'table-layout': 'fixed',
            height: o.height,
            width: o.width,
            '-webkit-text-size-adjust': 'none'
        });
        WT.dom.appendChild(this.element, this.wrapper);

        this.text = WT.dom.createJQueryElement('pre', 'wt-label-pre');
        this.text.css({
            'text-align': o.align,
            'font-size': o.fontSize,
            color: o.color
        });
        if (o.fontFamily) {
            this.text.css('font-family', o.fontFamily);
        }
        if (o.fontWeight) {
            this.text.css('font-weight', o.fontWeight);
        }
        if (o.fontStyle) {
            this.text.css('font-style', o.fontStyle);
        }
        if (o.decoration) {
            this.text.css('text-decoration', o.decoration);
        }
        if (o.fontShadow) {
            this.text.addClass('shadow');
        }
        if (o.autoWrap) {
            this.text.css('word-wrap', 'break-word').css(
                'white-space', 'pre-wrap');
        } else {
            if (!FR.Browser.isIE()) {
                this.text.css('white-space', 'pre');
            } else {
                this.text.css('white-space', 'pre-wrap').css(
                    'word-break', "keep-all");
            }
        }
        if (o.vAlign) {
            if (o.vAlign === 'top') {
                this.$td.css('vertical-align', 'top');
            }
            else if (o.vAlign === 'bottom') {
                this.$td.css('vertical-align', 'bottom');
            }
            else {
                this.$td.css('vertical-align', 'middle');
            }
        }
        else if (o.centerInVertical) {
            this.$td.css('vertical-align', 'middle');
        }
        WT.dom.appendChild(this.$td, this.text);
        WT.dom.appendChild(this.$tr, this.$td);
        WT.dom.appendChild(this.wrapper, this.$tr);
        var value = o.value;
        if (o.fontSuperscript) {
            this.text.addClass('super');
            value = '<sup>' + value + '</sup>';
        }
        if (o.fontSubscript) {
            this.text.addClass('suber');
            value = '<sub>' + value + '</sub>';
        }
        this.text.html(value);
        this.text.click(function () {
            if (!self.isEnable()) {
                return;
            }
            self.fireEvent(FR.Events.CLICK);
        });
        this.setVisible(!o.invisible);
    },

    /**
     * 设置值，默认触发VALUECHANGE事件
     * @param v
     * @param notTrig {Boolean} 为true时不触发VALUECHANGE事件
     */
    setValue: function (v, notTrig) {
        v = v + '';
        var oldValue = this.options.value;
        this.options.value = v;
        if (v !== oldValue) {
            FR.html(this.text, v);
            if (notTrig !== true) {
                this.fireEvent(FR.Events.CHANGE, v);
            }
        }
    }

});