/**
 * 新填报下的菜单按钮
 * @class WT.MenuButton
 * @extends WT.BaseButton
 *
 * @cfg {JSON} options 配置属性
 * @cfg {Array} options.menu 菜单项按钮
 */
WT.MenuButton = FR.extend(WT.BaseButton, {

    TYPE: 'WT.MenuButton',

    _const: {
        menuWidth: 175
    },

    _defaultConfig: function () {
        return $.extend(WT.MenuButton.superclass._defaultConfig.apply(), {
            menu: []
        });
    },

    _initVariable: function () {
        WT.MenuButton.superclass._initVariable.apply(this);
        this._const = $.extend({}, WT.MenuButton.superclass._const, this._const);
    },

    _initView: function () {
        WT.MenuButton.superclass._initView.apply(this);
        var self = this;
        this.on(FR.Events.CLICK, function () {
            FR.showMenuByEl({
                items: this._transferDataFormat(this.options.menu),
                minWidth: this._const.menuWidth
            }, self.element);
        })
    },

    _createAdditionalItem: function(items) {
        this.dropdownIcon = BI.createWidget({
            type: "bi.icon",
            width: this._const.iconSize,
            height: this._const.iconSize
        });
        this.dropdownIcon.element.addClass('wt-menu-button-icon');
        items.push(this.dropdownIcon);
    },

    _transferDataFormat: function (menu) {
        return WT.map(menu, function (item) {
            var result = {
                showTextSrc: !!item.config.text,
                showIconSrc: !!item.config.icon,
                src: item.config.text,
                iconSrc: item.config.icon
            };
            if (item.xtype === 'WT.BaseButton') {
                var action = this._findClickAction(item.events);
                if (action) {
                    result.handler = action;
                }
            }
            else if (item.xtype === 'WT.MenuButton') {
                result.submenu = this._transferDataFormat(item.config.menu);
            }
            return result;
        }, this);
    },

    _findClickAction: function(events) {
        for (var i = 0; i < events.length; i++) {
            var e = events[i];
            if (e.eventName === 'click') {
                return e.action;
            }
        }
    }

});