/**
 * 数字框控件
 * @class WT.NumberEditor
 *
 * @cfg {JSON} options 配置属性
 * @cfg {Boolean} options.allowFloat 允许小数
 * @cfg {Number} options.floatLength 小数长度
 * @cfg {Boolean} options.allowNegative 允许负数
 * @cfg {Boolean} options.needMax 是否有最大值
 * @cfg {Number} options.maxValue 最大值
 * @cfg {Boolean} options.needMin 是否有最小值
 * @cfg {Number} options.minValue 最小值
 */
WT.NumberEditor = FR.extend(WT.BaseEditor, {

    TYPE: 'WT.NumberEditor',

    _defaultConfig: function () {
        return $.extend(WT.NumberEditor.superclass._defaultConfig.apply(), {
            allowFloat: true,
            floatLength: 16,
            allowNegative: true,
            needMax: false,
            maxValue: 0,
            needMin: false,
            minValue: 0
        });
    },

    /**
     * @extends WT.BaseEditor._initVariable()
     * @protected
     */
    _initVariable: function () {
        WT.NumberEditor.superclass._initVariable.apply(this);
    },

    /**
     * @extends WT.BaseEditor._initView()
     * @protected
     */
    _initView: function () {
        WT.NumberEditor.superclass._initView.apply(this);
        var o = this.options;
        var self = this;
        this.element.css('background', 'white');
        var allowed = '0123456789';
        if (o.allowFloat) {
            allowed += '.';
        }
        if (o.allowNegative) {
            allowed += "-";
        }
        this.editor.element.css("ime-mode", "disabled");
        this.editor.element.keypress(function (e) {
            // james:下面是抄的Ext的
            var k = e.keyCode;
            if (!FR.Browser.isIE()
                && (FR.isNavKeyPress(e) || FR.isSpecialKey(e) || k == FR.keyCode.BACKSPACE)) {
                return;
            }
            var c = e.charCode || e.keyCode;// james：IE当中要取keyCode
            var cc = String.fromCharCode(c);
            // if($.browser.msie && (FR.isSpecialKey(e) || !cc)){
            // 过滤掉回车(ASCII为13)和ESC(ASCII为27)
            if (FR.Browser.isIE() && (c === 27 || c === 13)) {
                return;
            }

            if (allowed.indexOf(cc) === -1) {
                // FR.Msg.toast(errorMsg);
                e.preventDefault();
            }

            if (this.value.indexOf('.') > -1 && cc == '.') {
                // FR.Msg.toast(errorMsg);
                e.preventDefault();
            }
        });
    },
    verify: function (cValue) {
        return this._verify(cValue);
    },

    /**
     * 检验值，内置校验下在编辑时会调用
     * @return {Boolean} 校验成功与否
     */
    _verify: function (cValue) {
        var value = cValue != null
            ? cValue
            : this.options.value;
        if (value && ('' + value).indexOf(',') != -1) {
            value = value.replace(/,/g, "");
        }
        if (value == null || value == undefined || value === "") {
            if (this.options.allowBlank !== false) {
                return true;
            } else {
                this.errMsg = this.options.errorMsg4Blank || FR.i18nText("Fine-Engine_Base_NOT_NULL_Des");
                return false;
            }
        }
        if (isNaN(value)) {
            this.errMsg = this.options.errorMsg4Regex
                || FR.i18nText("Fine-Engine_Report_Err_The_value_Must_Be_Number");
            return false;
        }
        // 数字太大parseFloat就变成科学计数法的形式了 而且总位数17以上的小数会被截断
        var str = value + '';
        var bd = new FR.BigDecimal({
            numStr: str
        });
        if (str.indexOf('.') > 0) {
            if (!this.options.allowFloat) {
                this.errMsg = this.options.errorMsg4Regex
                    || FR.i18nText("Fine-Engine_Report_Err_The_Value_Must_Be_Integer");
                return false;
            } else {
                if (str.length - str.indexOf('.') > this.options.floatLength + 1) {
                    this.errMsg = this.options.errorMsg4Regex
                        || FR.i18nText("Fine-Engine_Base_DecimalNumber_Out");
                    return false;
                }
            }
        }
        // 不允许为负数
        if (!this.options.allowNegative && !bd.isPositive()) {
            this.errMsg = this.options.errorMsg4Regex
                || FR.i18nText("Fine-Engine_Report_Invalid_Cell");
            return false;
        }
        if (this.options.needMin &&
            bd.lessThan(new FR.BigDecimal({numStr: String(this.options.minValue)}))) {
            this.errMsg = this.options.errorMsg4Regex
                || FR.i18nText("Fine-Engine_Report_Err_The_Number_Is_Less_Than_The_Minimum_Value")
                + this.options.minValue;
            return false;
        }
        if (this.options.needMax &&
            bd.greaterThan(new FR.BigDecimal({numStr: String(this.options.maxValue)}))) {
            this.errMsg = this.options.errorMsg4Regex
                || FR.i18nText("Fine-Engine_Report_Err_The_Number_Is_Larger_Than_The_Maximum_Value")
                + this.options.maxValue;
            return false;
        }
        return true;
    },

    _fixPrecision: function (value) {
        var nan = isNaN(value);
        if (!this.options.allowFloat || this.options.floatLength <= 0
            || nan || !value) {
            return nan ? '' : value;
        }
        return parseFloat(parseFloat(value).toFixed(this.options.floatLength));
    },

    _parseValue: function (value) {
        if (value && ('' + value).indexOf(',') !== -1) {
            value = value.replace(/,/g, "");
        }
        value = parseFloat(value);
        return isNaN(value) ? '' : value;
    },

    setValue: function (v, verify) {
        v = this._parseValue(v);
        this.options.value = v;
        if (this.options.format && this.options.format !== 'T' && !this.isEditing) {
            this.editor.setValue(FR.contentFormat(v, this.options.format));
        } else {
            this.editor.setValue(v);
        }
        if (verify === true) {
            this.stopEdit(true);
        }
    },

    setEditingValue: function (v) {
        v = this._parseValue(v);
        if (this.options.value !== v) {
            this.editor.setValue(v);
            this.verify4Style();
        }
    },

    getValue: function () {
        var val = this.editor.getValue();
        var value = this._verify(val) ? val : this.options.value;
        var txt = value + '';
        txt = txt.trim();
        if (txt.indexOf(',') !== -1) {
            txt = txt.replace(/,/g, "");
        }
        if (txt.length > 17) {
            return txt;
        }
        return this._fixPrecision(this._parseValue(txt));
    }
});