/**
 * 新填报下的单选按钮
 * @class WT.RadioButton
 * @extends WT.BaseButton
 *
 * @cfg {JSON} options 配置属性
 * @cfg {Boolean} options.value 控件值
 * @cfg {String} options.text 文本
 */
WT.RadioButton = FR.extend(WT.BaseButton, {

    TYPE: 'WT.RadioButton',

    _defaultConfig: function () {
        return $.extend(WT.RadioButton.superclass._defaultConfig.apply(), {
            value: false,
            icon: 'transparent',
            freeButton: true
        });
    },

    _initView: function () {
        WT.RadioButton.superclass._initView.apply(this);
        var o = this.options, self = this;
        this._setIcon();
        this.on(FR.Events.CLICK, function () {
            o.value = !o.value;
            self._setIcon();
            self.fireEvent(FR.Events.STATECHANGE, o.value);
        });
    },

    _setIcon: function () {
        this.icon.element.removeClass('wt-radio-icon-normal');
        this.icon.element.removeClass('wt-radio-icon-check');
        this.icon.element.addClass(this.options.value ? 'wt-radio-icon-check' : 'wt-radio-icon-normal');

        if (this.options.value) {
            this.icon.element.removeClass('wt-radio-icon-normal').addClass('wt-radio-icon-check');
        } else {
            this.icon.element.removeClass('wt-radio-icon-check').addClass('wt-radio-icon-normal');
        }
    },

    /**
     * 设置值，默认触发VALUECHANGE事件
     * @param v {Boolean}
     * @param notTrig {Boolean} 为true时不触发VALUECHANGE事件
     */
    setValue: function (v, notTrig) {
        var oldValue = this.options.value;
        this.options.value = v;
        if (v !== oldValue) {
            this._setIcon();
            if (notTrig !== true) {
                this.fireEvent(FR.Events.STATECHANGE, v);
            }
        }
    },

    getValue: function () {
        return this.options.value;
    },

    selected: function () {
        return !!this.options.value;
    }

});