/**
 * 新填报下单选框组控件
 * @class WT.RadioGroup
 * @extends WT.BaseGroup
 *
 * @cfg {JSON} options 配置属性
 */
WT.RadioGroup = FR.extend(WT.BaseGroup, {

    TYPE: 'WT.RadioGroup',

    _onItemValueChange: function (idx, value) {
        this._clearValue(idx);
        this.options.value = this._generateValue();
        this.fireEvent(FR.Events.STATECHANGE, this.getValue());
    },

    _createItemComp: function (data, idx) {
        var o = this.options;
        return new WT.RadioButton({
            text: data.text,
            fontSize: o.fontSize,
            height: o.fontSize + 4,
            horizontalPadding: 5
        });
    },

    _clearValue: function (excludeIndex) {
        WT.each(this.items, function (item, idx) {
            if (idx === excludeIndex) {
                return;
            }
            item.setValue(false, true);
        })
    },

    /**
     * 从UI组件的状态生成值
     * @return {null}
     */
    _generateValue: function () {
        for (var i = 0; i < this.data.length; i++) {
            var item = this.items[i];
            if (item.getValue()) {
                return this.data[i].value;
            }
        }
        return null;
    },

    _applyValue: function () {
        if (!this.items) {
            return;
        }
        var foundValue = false;
        for (var i = 0; i < this.items.length; i++) {
            var d = this.data[i];
            if (d.value + '' === this.options.value + '') {
                this._clearValue(i);
                this.items[i].setValue(true, true);
                foundValue = true;
                break;
            }
        }
        if (!foundValue) {
            for (var i = 0; i < this.items.length; i++) {
                var d = this.data[i];
                if (d.text + '' === this.options.value + '') {
                    this._clearValue(i);
                    this.items[i].setValue(true, true);
                    foundValue = true;
                    break;
                }
            }
        }
        if (!foundValue) {
            WT.each(this.items, function (item) {
                item.setValue(false, true);
            });
        }
        this.options.value = this._generateValue();
    },

    getText: function () {
        for (var i = 0; i < this.data.length; i++) {
            var item = this.items[i];
            if (item.getValue()) {
                return this.data[i].text;
            }
        }
        return null;
    }

});
