/**
 * 标签文本框控件
 * @class WT.TagEditor
 *
 * @cfg {JSON} options 配置属性
 */
WT.TagEditor = FR.extend(WT.BaseEditor, {

    TYPE: 'WT.TagEditor',

    _defaultConfig: function () {
        return $.extend(WT.TagEditor.superclass._defaultConfig.apply(), {
            value: null
        });
    },

    /**
     * @extends WT.BaseEditor._initVariable()
     * @protected
     */
    _initVariable: function () {
        WT.TagEditor.superclass._initVariable.apply(this);
        this.tagValue = [];
        this.tags = [];
        this.editorMinWidth = 40;
        if (WT.isNumber(this.options.width)) {
            var w = Math.floor(this.options.width / 4);
            if (w < 16) {
                w = 16;
            }
            if (w > 40) {
                w = 40;
            }
            this.editorMinWidth = w;
        }
        this.deletedTagIndexes = [];
    },

    /**
     * @extends WT.BaseEditor._initView()
     * @protected
     */
    _initView: function () {
        WT.TagEditor.superclass._initView.apply(this);
        var self = this, o = this.options;
        this.element.addClass('wt-tag-editor');
        this.contentWraper = $('<div class="wt-tag-content-wrapper"></div>');
        this.editor.element.wrap(this.contentWraper);
        this.tagWrapper = $('<div style="display: inline-block;"></div>');
        this.tagWrapper.insertBefore(this.editor.element);
        this.editor.element.css({
            width: 40,
            display: 'inline-block'
        });
        this.editor.element.keydown(function (e) {
            if (FR.isEmpty(self.editor.getValue())) {
                if (e.keyCode === FR.keyCode.BACKSPACE) {
                    self._deleteLastTag();
                    self.fireEvent(WT.TagEditor.EVENT_TAG_DELETE);
                    e.preventDefault();
                    e.stopPropagation();
                } else if (e.keyCode === FR.keyCode.LEFT) {
                    //左方向键
                    self._selectLastTag();
                    e.preventDefault();
                    e.stopPropagation();
                }
            }
            if (e.keyCode === FR.keyCode.ENTER) {
                self._commitTag();
                e.preventDefault();
                e.stopPropagation();
            }
        });
        this.editor._getEvents()[BI.Input.EVENT_BLUR.toLowerCase()] = [];
        this.editor.on(BI.Input.EVENT_BLUR, function () {
            //有可能并没有失焦，比如切换了tab页，操作系统的工具栏
            var selfInput = self.editor.element;
            if (selfInput[0]) {
                //activeElement支持的浏览器是chrome2.0，IE4.0，火狐3.0，opera9.6，sarfri4.0
                if (document.activeElement === selfInput[0]) {
                    //如果没有失焦，则什么都不干
                    return;
                }
            }
            self.isEditing = false;
            self._dealEditorStyleOnBlur();
            //恢复全局快捷键监听
            if (!WT.Keys.enable()) {
                WT.Keys.enable(true);
            }
            self._hideWaterMark();
            if (self.getTagValue().length === 0) {
                //没有值的时候显示水印
                self._showWaterMark();
            }
        });
        this.suffix = WT.dom.createJQueryElement('input', 'wt-tag-suffix').attr('readonly', true);
        this.suffix.insertAfter(this.editor.element);
        this.element.on('click', function (e) {
            self.editor.focus();
            e.preventDefault();
            e.stopPropagation();
        });
    },

    _createTag: function (text, idx) {
        var self = this, o = this.options;
        var clickTimer = null;
        var tag = WT.dom.createJQueryElement('span', 'wt-tag wt-no-select', text).attr('tabindex', idx + '');
        tag.css({
            'font-size': this.options.fontSize,
            'line-height': this.options.fontSize + 2 + 'px'
        });
        tag.on('focus', function () {
            tag.addClass('wt-tag-select');
        });
        tag.on('blur', function () {
            tag.removeClass('wt-tag-select');
        });
        tag.on('click', function (e) {
            e.preventDefault();
            e.stopPropagation();
            if (o.disabled) {
                return;
            }
            if (!clickTimer) {
                clickTimer = setTimeout(function () {
                    clickTimer = null;
                }, 500);
                self._selectTagByIndex(idx);
            } else {
                clearTimeout(clickTimer);
                var removedValue = self._deleteTagByIndex(idx);
                self.editor.setValue(removedValue);
                self.editedManually = true;
                self.fireEvent(WT.TagEditor.EVENT_TAG_DELETE);
                self.editor.focus();
            }
        });
        return tag;
    },

    _selectTagByIndex: function (idx) {
        if (idx < 0 || idx > this.tags.length - 1) {
            return;
        }
        var tag = this.tags[idx];
        tag.focus();
        this._registerKeyEvent(idx);
    },

    _selectLastTag: function () {
        if (this.tags && this.tags.length > 0) {
            this._selectTagByIndex(this.tags.length - 1);
        }
    },

    _commitTag: function () {
        var editorVal = this.editor.getValue();
        if (!FR.isEmpty(editorVal)) {
            this.tagValue.push(editorVal);
            this._applyValue();
            this.editor.setValue('');
        }
    },

    _registerKeyEvent: function (tagIdx) {
        var self = this;
        var tagOpts = function (e) {
            if (e.keyCode == FR.keyCode.DELETE || e.keyCode == FR.keyCode.BACKSPACE) {
                WT.Keys.unreg(tagOpts);
                self._deleteTagByIndex(tagIdx);
                self.fireEvent(WT.TagEditor.EVENT_TAG_DELETE);
                if (tagIdx + 1 >= self.tags.length) {
                    self.editor.focus();
                }
                else {
                    self._selectTagByIndex(tagIdx);
                }
            } else if (e.keyCode == FR.keyCode.LEFT) {
                WT.Keys.unreg(tagOpts);
                self._selectTagByIndex(tagIdx - 1);
            } else if (e.keyCode == FR.keyCode.RIGHT) {
                WT.Keys.unreg(tagOpts);
                if (tagIdx + 1 >= self.tags.length) {
                    WT.Keys.unreg(tagOpts);
                    self.editor.focus();
                }
                else {
                    self._selectTagByIndex(tagIdx + 1);
                }
            }
        };
        WT.Keys.reg(tagOpts);
    },

    /**
     * 将value应用到UI上
     * @private
     */
    _applyValue: function (noFocus) {
        if (!this.tagWrapper) {
            return;
        }
        var self = this;
        //先清空原来的
        this.tags = [];
        this.tagWrapper.empty();
        WT.each(this.tagValue, function (v, idx) {
            var tag = self._createTag(v, idx);
            self.tags.push(tag);
            WT.dom.appendChild(self.tagWrapper, tag);
        });
        this.editor.setValue("");
        WT.nextTick(function () {
            var tagWrapperWidth = this.tagWrapper.width();
            var editorWidth = Math.max(this.editorMinWidth, this.options.width - tagWrapperWidth - 3);
            this.editor.element.css('width', editorWidth);
            if (noFocus !== true) {
                this.suffix.focus();
            }
        }, this);
        if(this.tags!=null && this.tags.length !== 0){
            this._hideWaterMark();
        } else {
            this._showWaterMark();
        }
    },

    _deleteTagByIndex: function (idx) {
        var removedValue = WT.arrayPull(this.tagValue, idx);
        if (removedValue) {
            this._applyValue(true);
            this.deletedTagIndexes.push(idx);
        }
        return removedValue;
    },

    /**
     * 删除最后一个标签
     * @private
     */
    _deleteLastTag: function () {
        if (this.tags && this.tags.length > 0) {
            var removedValue = this._deleteTagByIndex(this.tags.length - 1);
            if (removedValue) {
                this.editor.focus();
            }
        }
    },

    _assertValueIsArray: function (value) {
        if (WT.isArray(value)) {
            return value;
        }
        if (WT.isString(value)) {
            return WT.map(value.split(','), function (str) {
                return WT.trim(str);
            });
        }
        return [];
    },

    _shouldRecoverValueOnFocus: function () {
        return false;
    },

    _onKeyDown: function(e) {
        if (e.keyCode !== 8 || this.editor.getValue() !== "") {
            this.editedManually = true;
            this.clearDeletedTagIndexes();
        }
    },

    /**
     * 检验输入数据，失败会设置波浪线样式
     */
    verify4Style: function () {
        //先判断是否允许空值
        var val = this.editor.getValue();
        if (!this.options.allowBlank && this.getTagValue().length === 0) {
            this.errMsg = this.options.errorMsg4Blank;
            this._dealEditorStyleOnInvalid();
        }
        else {
            if (this._verify(val)) {
                this._dealEditorStyleOnValid();
            }
            else {
                this._dealEditorStyleOnInvalid();
            }
        }
    },

    setTagValue: function (v) {
        this.clearDeletedTagIndexes();
        this.tagValue = this._assertValueIsArray(v);
        this._applyValue();
    },

    getTagValue: function () {
        var value = WT.copyArray(this.tagValue);
        var editorVal = this.editor.getValue();
        if (editorVal != '') {
            value.push(editorVal);
        }
        return value;
    },

    getDeletedTagIndexes: function () {
        return this.deletedTagIndexes;
    },

    clearDeletedTagIndexes: function () {
        this.deletedTagIndexes = [];
    },

    /**
     * 编辑结束，触发校验
     * 校验失败会提示错误以及恢复原值
     * @param isToastError 校验失败是否弹出错误提示
     * @return {Boolean} 校验成功与否
     */
    stopEdit: function (isToastError) {
        if (!this.editor) {
            return true;
        }
        var val = this.editor.getValue();
        var result = true;
        //先判断是否允许空值
        if (!this.options.allowBlank && this.getTagValue().length === 0) {
            isToastError && this._toastMsg(this.options.errorMsg4Blank);
            result = false;
        } else {
            if (this._verify(val)) {
                this.options.value = val;
                result = true;
            } else {
                isToastError && this._toastMsg(this.errMsg);
                result = false;
            }
        }
        this._dealEditorStyleOnValid();
        return result;
    },

    focus: function () {
        var self = this;
        setTimeout(function () {
            WT.TagEditor.superclass.focus.apply(self, arguments);
        }, 100);
    },

    setEditorDisplayValue: function () {
        // ignore
    }
});
WT.TagEditor.EVENT_TAG_DOUBLE_CLICK = 'EVENT_TAG_DOUBLE_CLICK';
WT.TagEditor.EVENT_TAG_DELETE = 'EVENT_TAG_DELETE';