/**
 新填报下的视图树
 * @class WT.TreeView
 * @extends WT.Widget
 *
 */
WT.TreeView = FR.extend(WT.Widget, {

    TYPE: 'WT.TreeView',

    _initView: function () {
        var o = this.options, self = this;
        this.element = WT.dom.createJQueryElement('div', 'wt-absolute-layout');
        this.element.css({
            left: o.left,
            top: o.top,
            width: o.width,
            height: o.height
        });
        var oo = $.extend({}, o, {
            renderEl: this.element
        });
        if (oo.location) {
            var l = FR.cellStr2ColumnRow(oo.location);
            oo.widgetUrl = FR.buildServletUrl({
                op: 'widget',
                location: {
                    column: l.col,
                    row: l.row,
                    reportIndex: oo.reportIndex
                },
                sessionID: FR.SessionMgr.getSessionID()
            });
        }
        //节点生成事件要在树视图创建前绑定
        oo.listeners = [{
            eventName: WT.TreeView.EVENT_NODEADD,
            action: function () {
                self.fireEvent(WT.TreeView.EVENT_NODEADD);
            }
        }];
        this.tree = new WT.TreeEditor(oo);
        this.tree.on(FR.Events.DEFAULTINIT, function () {
            self.fireEvent(FR.Events.DEFAULTINIT);
        });
        this.tree.on(FR.Events.BEFORESTATECHANGE, function () {
            self.fireEvent(FR.Events.BEFORESTATECHANGE);
        });
        this.tree.on(FR.Events.AFTERBUILD, function () {
            self.fireEvent(FR.Events.AFTERBUILD);
        });
        this.tree.on(FR.Events.CLICK, function (item) {
            self.fireEvent(FR.Events.CLICK, item);
        });
        this.tree.on(FR.Events.AFTEREDIT, function () {
            self.fireEvent(FR.Events.AFTEREDIT);
        });
        this.tree.on(FR.Events.NOSELECT, function () {
            self.fireEvent(FR.Events.NOSELECT);
        });
    },

    _handleInValue: function (v) {
        var o = this.options;
        while (WT.isArray(v)) {
            if (v.length === 0) {
                return [];
            }
            if (o.mutiSelection) {
                if (!WT.isArray(v[0]) || !WT.isArray(v[0][0])) {
                    return v;
                }
            } else {
                if (!WT.isArray(v[0])) {
                    return v;
                }
            }
            v = v[0];
        }
        return v;
    },

    getOriginValue: function () {
        return function () {
            //这里的this是外部的this.tree
            var res;
            if (!this.options.mutiSelection) {
                res = this.element.getTCPValue() || this.value;
            } else {
                res = this.element.getTCPValues() || this.value;
            }
            return FR.getTreeValueResult(res, this.options.mutiSelection, true);
        }.call(this.tree);
    },

    getValue: function () {
        if (this.tree) {
            return this.tree.getValue();
        }
    },

    getText: function () {
        if (this.tree) {
            return this.tree.getText();
        }
    },

    setValue: function (v) {
        this.tree.setValue(this._handleInValue(v));
    },

    setText: function (t) {
        this.tree.setText(t);
    },

    setEnable: function (enable) {
        this.tree.setEnable(enable);
    },

    isEnable: function () {
        return this.tree.isEnabled();
    },

    stopEdit: function (isToastError) {
        var valid = this.verify();
        if (isToastError === true && !valid) {
            FR.Msg.toast(this.errMsg)
        }
        return valid;
    },
    verify: function () {
        var isValidate = this.tree.isValidate();
        if (!isValidate) {
            this.errMsg = this.tree.getErrorMsg();
        }
        return isValidate;
    },

    reset: function () {
        this.tree.reset();
    },

    destroy: function () {
        this.tree && this.tree.purgeListeners();
        this.purgeListeners();
        this.element.empty();
        this.element.remove();
        this.tree = null;
        this.element = null;
    }

});
WT.TreeView.EVENT_NODEADD = 'nodeadd';

WT.TreeEditor = FR.extend(FR.TreeEditor, {
    _setCheckStateByDefaultValue: function () {
        var self = this;
        var initArray = this.options.value ? this.options.value : [];
        var treeConfig = FR.createTreeConfig4Widget(this);
        /*change [[1,11],[2,21]] to ["1,11","2,21"]*/
        var sortArray = function (initArray) {
            if (!$.isArray(initArray)) {
                return;
            }
            if (!FR.isEmptyArray(initArray)) {
                for (var i = 0; i < initArray.length; i++) {
                    if ($.isArray(initArray[i])) {
                        initArray[i] = initArray[i].toString();
                    }
                }
            }
        };
        if (!FR.isEmptyArray(initArray)) {
            var nodes = this.options.data.getData();
            // 把树节点中的某一项转化成具体的数组路径
            if (!FR.isArray(initArray)) {
                initArray = FR.getTreeTrace(initArray, nodes, self.options.mutiSelection, self.options.selectLeafOnly);
            }
            if (self.options.mutiSelection) {
                // 把一维数组转化成二维数组
                initArray = FR.parseTo2DArray(initArray);
                sortArray(initArray);
                FR.setAllNodeState(nodes, 0);
                for (var m = 0; m < initArray.length; m++) {
                    FR.setNodeStateByInitValue(nodes, 0, initArray[m].split(","), self.options.selectLeafOnly);
                }
            } else {
                FR.findSelectedNode(nodes, 0, initArray, treeConfig);
            }
        }
        if (!this.treeCreator) {
            this.treeCreator = {};
        }
        treeConfig.treeCreator = this.treeCreator;
        this._createTreeView(treeConfig);
    },

    _isStringArrayEqual: function (arr1, arr2) {
        if (!arr1 === !!arr2) {
            return false;
        }
        if (!arr1 && !arr2) {
            return true;
        }
        if (arr1.length !== arr2.length) {
            return false;
        }
        for (var i = 0; i < arr1.length; i++) {
            if (arr1[i] !== arr2[i]) {
                return false;
            }
        }
        return true;
    },

    _isValueArrayContains: function (value_array, valuePath) {
        for (var i = 0; i < value_array.length; i++) {
            if (this._isStringArrayEqual(value_array[i], valuePath)) {
                return true;
            }
        }
        return false;
    },

    createDependencePara4Data: function () {
        var o = this.options;
        var parameter = {};
        if (!FR.isEmptyObj(o.dependence) && o.paraProvider) {
            var dependence = {};
            for (var k in o.dependence) {
                var paraVal = o.paraProvider.getPara(o.dependence[k]);
                if (!FR.isEmpty(paraVal)) {
                    dependence[WT.isIntString(k) ? o.dependence[k].toUpperCase() : k.toUpperCase()] = paraVal;
                }
            }
            if (!WT.isEmpty(dependence)) {
                parameter.dependence = dependence;
            }
        }
        return {
            parameter: parameter
        }
    },

    setValue: function (value) {
        // 设置编辑器的值
        var self = this;
        var value_array;
        if (typeof value === "string") {
            value_array = value.split(this.options.delimiter);
        } else {
            value_array = $.makeArray(value);
        }
        var nodes = this.options.data.getData();
        if (!FR.isEmptyArray(value_array)) {
            var leafNodes = [];
            for (var i = 0; i < nodes.length; i++) {
                leafNodes = leafNodes.concat(nodes[i].getLeafNode());
            }
            var mutiSelectionMode = $.isArray(value_array[0]) && self.options.mutiSelection;
            var relatedNodes = [];
            var foundMatch = false;
            var match = false;
            for (var idx = 0; idx < leafNodes.length; idx++) {
                var item = leafNodes[idx];
                if (mutiSelectionMode) {
                    match = this._isValueArrayContains(value_array, item.getValuePath(false));
                } else {
                    match = FR.equals(item.getValuePath(false), value_array);
                }
                if (match) {
                    foundMatch = true;
                    item.setcheckstate(1);
                    if (!self.options.mutiSelection) {
                        break;
                    }
                } else {
                    item.setcheckstate(0);
                }
                relatedNodes.push(item);
            }
            if (!foundMatch) {
                for (var idx = 0; idx < leafNodes.length; idx++) {
                    var item = leafNodes[idx];
                    if (mutiSelectionMode) {
                        match = this._isValueArrayContains(value_array, item.getTextPath(false));
                    } else {
                        match = FR.equals(item.getTextPath(false), value_array);
                    }
                    if (match && !foundMatch) {
                        foundMatch = true;
                        item.setcheckstate(1);
                        break;
                    } else {
                        item.setcheckstate(0);
                    }
                    relatedNodes.push(item);
                }
            }
            // wei : 处理父节点们
            $.each(relatedNodes, function (idx, item) {
                var parentNode = item.getParent();
                while (parentNode) {
                    if (isNoChildChosen(parentNode)) {
                        parentNode.setcheckstate(0);
                        parentNode.setexpand(false);
                    } else {
                        parentNode.setcheckstate(parentNode.isAllChildrenChosen() ? 1 : 2);
                        // shoc 值选中的话父节点展开
                        parentNode.setexpand(true);
                    }
                    parentNode = parentNode.getParent();
                }
            });

            function isNoChildChosen(node) {
                if (!node.hasChild() || !node.isLoadComplete()) {
                    return true;
                }
                for (var i=0; i<node.childrenNodes.length; i++) {
                    if (node.childrenNodes[i].getcheckstate() !== 0) {
                        return false;
                    }
                }
                return true;
            }
        } else {
            FR.setAllNodeState(nodes, 0);
        }
        this.treeCreator.forest.applyTreesState();
    },

    reset: function () {
        this.setValue("");
    }
});