/**
 * 新填报下所有Widget的超类
 * @class WT.Widget
 * @extends FR.Widget
 * 
 * @abstract
 *
 * @cfg {JSON} options 配置属性
 * @cfg {String} options.widgetName 控件名称
 * @cfg {Boolean} options.disabled 可用性
 * @cfg {Boolean} options.invisible 可见性
 * @cfg {Boolean} options.usedAsCell 是否为单元格里的控件
 * @cfg {Object} options.value 控件值
 * @cfg {Number} options.top 控件距父控件顶部距离
 * @cfg {Number} options.left 控件距父控件左边距离
 * @cfg {Number} options.width 控件宽度
 * @cfg {Number} options.height 控件高度
 * @cfg {Number} options.fontSize 字体大小
 * @cfg {String} options.tag 控件标签
 * @cfg {function} options.globalConfigProvider 全局参数获取器
 */
WT.Widget = FR.extend(FR.OB, {

	TYPE: 'WT.Widget',

	/**
     * 初始化属性
	 * @extends FR.OB._defaultConfig()
	 * @protected
     */
    _defaultConfig: function () {
        return $.extend(WT.Widget.superclass._defaultConfig.apply(), {
			widgetName: "",
            disabled: false,
            invisible: false,
            usedAsCell: false,
            value: null,
			top: 0,
			left: 0,
			width: 80,
			height: 21,
			fontSize: 12,
			tag: "",
			globalConfigProvider: null
		});
    },

	/**
     * 初始化控件
	 * 注：该方法在子类中一般不实现
	 * @extends FR.OB._init()
	 * @protected
     */
    _init: function () {
		WT.Widget.superclass._init.apply(this, arguments);
        this._initVariable();
        this._initView();
    },

	/**
	 * 初始化其他变量
	 * @protected
	 */
	_initVariable: function () {
	},

    /**
     * 初始化dom
	 * @abstract
     */
    _initView: function () {
		/**
		 * Example:
         * this.element = $('<div/>'); 
		 */
    },

    _confirmEvents: function () {
    	if (this.options.delayInit === true) {
    		WT.nextTick(function () {
                this.fireEvent(FR.Events.AFTERINIT);
            }, this);
		} else {
            this.fireEvent(FR.Events.AFTERINIT);
        }
    },

	/**
	 * 获取根元素，根元素为jQuery对象
	 * @protected
	 */
	getRootElement: function() {
		return this.element;
	},

	/**
     * 设置可用性
	 * @protected
     */
    setEnable: function (enable) {
        this.options.disabled = !enable;
        if (enable) {
            this.element.removeClass('wt-disable');
        }
        else {
            this.element.addClass('wt-disable');
        }
    },

    disable: function () {
        this.setEnable(false);
    },

    enable: function () {
        this.setEnable(true);
    },

	/**
     * 获取可用性
	 * @protected
     */
	isEnable: function () {
		return !this.options.disabled;
	},

	/**
     * 设置可见性
	 * @protected
     */
    setVisible: function (visible) {
		if (visible === true) {
			this.options.invisible = false;
			this.element.show();
		}
		else if (visible === false) {
			this.options.invisible = true;
			this.element.hide();
		}
    },

    invisible: function () {
		this.setVisible(false);
    },

    visible: function () {
        this.setVisible(true);
    },

	/**
     * 获取可见性
	 * @protected
     */
	getVisible: function () {
		return !this.options.invisible;
	},

    isVisible: function () {
        return !this.options.invisible;
    },

	/**
     * 获取宽度
	 * @protected
     */
    getWidth: function () {
        return this.options.width;
    },

	/**
     * 获取高度
	 * @protected
     */
    getHeight: function () {
        return this.options.height;
    },

	/**
     * 获取控件名
	 * @protected
     */
    getName: function () {
        return this.options.widgetName;
    },

	/**
	 * 获取控件值
	 * @protected
	 */
	getValue: function () {
		return this.options.value;
	},

	/**
     * 设置标签
	 * @protected
     */
    setTag: function (tag) {
        this.options.tag = tag;
    },

	/**
     * 获取标签
	 * @protected
     */
    getTag: function () {
        return this.options.tag;
    },

    couldUsedAsEditor: function () {
		return false;
    },

    isStringTypeNumber: function () {
		return true;
    },

    getType: function () {
		return this.TYPE;
    },

    getText: function () {
		return this.getValue();
    },

    setText: function (text) {
		this.setValue && this.setValue(text);
    },

    reset: function () {
    	this.setValue && this.setValue(null);
    },

    /**
	 * 从DOM上移除
     */
	detach: function () {
        WT.dom.detachFromHTML(this.element);
    },

	/**
     * 销毁控件，移除dom，释放资源
     */
    destroy: function () {
    	(this.element instanceof $) && this.element.remove();
    },

	/************************************* ignored functions *************************************/

    doResize: function () {
		// ignore
    },

    getWidgetByName: function () {
    	// ignore
		return null;
    },

    setSource: function (source) {
        // ignore
    }
});